@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * arch-tag: Implementation of GStreamer encoding backend
- * 
+ *
  * Based on Sound-Juicer's ripping code
  *
  * Copyright (C) 2003 Ross Burton <ross@burtonini.com>
@@ -34,6 +34,10 @@
 #include <profiles/gnome-media-profiles.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_GSTREAMER_0_10
+#include <gst/tag/tag.h>
+#endif
+
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
 #include "rb-preferences.h"
@@ -48,24 +52,23 @@
 #define gst_caps_unref gst_caps_free
 #endif
 
-
 static void rb_encoder_gst_class_init (RBEncoderGstClass *klass);
 static void rb_encoder_gst_init       (RBEncoderGst *encoder);
 static void rb_encoder_gst_finalize   (GObject *object);
 static void rb_encoder_init (RBEncoderIface *iface);
 
-
 struct _RBEncoderGstPrivate {
 	GstElement *enc;
 	GstElement *pipeline;
-	
+
 	gboolean transcoding;
 	gint decoded_pads;
-	
+
 	gboolean error_emitted;
 	gboolean completion_emitted;
 
-	gulong total_length;
+	GstFormat position_format;
+	gint64 total_length;
 	guint progress_id;
 };
 
@@ -74,14 +77,13 @@ G_DEFINE_TYPE_WITH_CODE(RBEncoderGst, rb_encoder_gst, G_TYPE_OBJECT,
 					      rb_encoder_init))
 #define RB_ENCODER_GST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_ENCODER_GST, RBEncoderGstPrivate))
 
-
 static gboolean rb_encoder_gst_encode (RBEncoder *encoder,
 				       RhythmDBEntry *entry,
 				       const char *dest,
 				       const char *mime_type);
 static void rb_encoder_gst_cancel (RBEncoder *encoder);
 static void rb_encoder_gst_emit_completed (RBEncoderGst *encoder);
-	
+
 static void
 rb_encoder_gst_class_init (RBEncoderGstClass *klass)
 {
@@ -109,15 +111,15 @@ static void
 rb_encoder_gst_finalize (GObject *object)
 {
 	RBEncoderGst *encoder = RB_ENCODER_GST (object);
-	
+
 	if (encoder->priv->progress_id != 0)
-		g_source_remove (encoder->priv->progress_id);	
-	
+		g_source_remove (encoder->priv->progress_id);
+
 	if (encoder->priv->pipeline) {
 		gst_element_set_state (encoder->priv->pipeline, GST_STATE_NULL);
 		g_object_unref (encoder->priv->pipeline);
 		encoder->priv->pipeline = NULL;
-	}	
+	}
 
         G_OBJECT_CLASS (rb_encoder_gst_parent_class)->finalize (object);
 }
@@ -143,8 +145,7 @@ rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 	g_return_if_fail (encoder->priv->completion_emitted == FALSE);
 
 	if (encoder->priv->progress_id != 0)
-		g_source_remove (encoder->priv->progress_id);	
-
+		g_source_remove (encoder->priv->progress_id);
 
 	/* emit an error if no audio pad has been found and it wasn't due to an
 	 * error */
@@ -165,7 +166,6 @@ rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 	_rb_encoder_emit_completed (RB_ENCODER (encoder));
 }
 
-
 #ifdef HAVE_GSTREAMER_0_10
 static gboolean
 bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
@@ -184,7 +184,7 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 		rb_debug ("received error %s", string);
 		g_error_free (error);
 		g_free (string);
-		
+
 		rb_encoder_cancel (RB_ENCODER (encoder));
 		break;
 
@@ -194,7 +194,7 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 		g_error_free (error);
 		g_free (string);
 		break;
-	
+
 	case GST_MESSAGE_EOS:
 		rb_debug ("received EOS");
 
@@ -205,7 +205,7 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 		g_object_unref (encoder->priv->pipeline);
 		encoder->priv->pipeline = NULL;
 		break;
-	
+
 	default:
 		rb_debug ("message of type %s", gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
 		break;
@@ -239,7 +239,7 @@ gst_error_cb (GstElement *element,
 {
 	rb_encoder_gst_emit_error (encoder, error);
 	rb_debug ("received error %s", debug);
-		
+
 	rb_encoder_cancel (RB_ENCODER (encoder));
 }
 
@@ -248,20 +248,21 @@ gst_error_cb (GstElement *element,
 static gboolean
 progress_timeout_cb (RBEncoderGst *encoder)
 {
-	gint64 nanos;
-	gint secs;
-	static GstFormat format = GST_FORMAT_TIME;
+	gint64 position;
+	static GstFormat format;
 	GstState state;
 
 	if (encoder->priv->pipeline == NULL)
 		return FALSE;
 
+	format = encoder->priv->position_format;
+
 #ifdef HAVE_GSTREAMER_0_10
 	gst_element_get_state (encoder->priv->pipeline, &state, NULL, GST_CLOCK_TIME_NONE);
 	if (state != GST_STATE_PLAYING)
 		return FALSE;
 
-	if (!gst_element_query_position (encoder->priv->pipeline, &format, &nanos)) {
+	if (!gst_element_query_position (encoder->priv->pipeline, &format, &position)) {
 		g_warning ("Could not get current track position");
 		return TRUE;
 	}
@@ -270,15 +271,28 @@ progress_timeout_cb (RBEncoderGst *encoder)
 	if (state != GST_STATE_PLAYING)
 		return FALSE;
 
-	if (!gst_element_query (encoder->priv->pipeline, GST_QUERY_POSITION, &format, &nanos)) {
+	if (!gst_element_query (encoder->priv->pipeline, GST_QUERY_POSITION, &format, &position)) {
 		g_warning ("Could not get current track position");
 		return TRUE;
 	}
 #endif
 
-	secs = nanos / GST_SECOND;
-	rb_debug ("encoding progress at %d out of %lu", secs, encoder->priv->total_length);
-	_rb_encoder_emit_progress (RB_ENCODER (encoder), ((double)secs) / encoder->priv->total_length);
+	if (format == GST_FORMAT_TIME) {
+		gint secs;
+
+		secs = position / GST_SECOND;
+		rb_debug ("encoding progress at %d out of %" G_GINT64_FORMAT,
+			  secs,
+			  encoder->priv->total_length);
+		_rb_encoder_emit_progress (RB_ENCODER (encoder), 
+					   ((double)secs) / encoder->priv->total_length);
+	} else {
+		rb_debug ("encoding progress at %" G_GINT64_FORMAT " out of %" G_GINT64_FORMAT,
+			  position,
+			  encoder->priv->total_length);
+		_rb_encoder_emit_progress (RB_ENCODER (encoder),
+					   ((double) position) / encoder->priv->total_length);
+	}
 
 	return TRUE;
 }
@@ -291,15 +305,15 @@ start_pipeline (RBEncoderGst *encoder, GError **error)
 	GstBus *bus;
 
 	g_return_val_if_fail (encoder->priv->pipeline != NULL, FALSE);
-	
+
 	bus = gst_pipeline_get_bus (GST_PIPELINE (encoder->priv->pipeline));
 	gst_bus_add_watch (bus, bus_watch_cb, encoder);
 
 	result = gst_element_set_state (encoder->priv->pipeline, GST_STATE_PLAYING);
 #elif HAVE_GSTREAMER_0_8
-	
+
 	g_return_val_if_fail (encoder->priv->pipeline != NULL, FALSE);
-	
+
 	g_signal_connect_object (G_OBJECT (encoder->priv->pipeline),
 				 "error", G_CALLBACK (gst_error_cb),
 				 encoder, 0);
@@ -318,7 +332,7 @@ start_pipeline (RBEncoderGst *encoder, GError **error)
 			_rb_encoder_emit_progress (RB_ENCODER (encoder), -1);
 		}
 	}
-	
+
 	return (result != GST_STATE_CHANGE_FAILURE);
 }
 
@@ -367,12 +381,12 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	if (queue == NULL)
 		return NULL;
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), queue);
-	
+
 	queue2 = gst_element_factory_make ("queue", NULL);
 	if (queue2 == NULL)
 		return NULL;
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), queue2);
-	
+
 	/* Nice big buffers... */
 	g_object_set (queue, "max-size-time", 120 * GST_SECOND, NULL);
 
@@ -383,7 +397,6 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	if (encoding_bin == NULL)
 		return NULL;
 
-
 	/* find pads and ghost them if necessary */
 	if ((pad = gst_bin_find_unconnected_pad (GST_BIN (encoding_bin), GST_PAD_SRC)))
 		gst_element_add_pad (encoding_bin, gst_ghost_pad_new ("src", pad));
@@ -398,7 +411,7 @@ add_encoding_pipeline (RBEncoderGst *encoder,
 	/* store the first element of the encoding graph. new_decoded_pad_cb
 	 * will link to this once a decoded pad is found */
 	encoder->priv->enc = queue;
-	
+
 	return queue2;
 }
 
@@ -412,7 +425,7 @@ add_tags_from_entry (RBEncoderGst *encoder,
 	gulong day;
 
 	tags = gst_tag_list_new ();
-	
+
 	gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE_ALL,
 			  /* TODO: compute replay-gain */
 			  GST_TAG_TITLE, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE),
@@ -436,6 +449,13 @@ add_tags_from_entry (RBEncoderGst *encoder,
 				  NULL);
 		g_date_free (date);
 	}
+#ifdef GST_TAG_MUSICBRAINZ_TRACKID
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID)) {
+		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+				  GST_TAG_MUSICBRAINZ_TRACKID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID),
+				  NULL);
+	}
+#endif
 
 #ifdef HAVE_GSTREAMER_0_10
 	{
@@ -466,7 +486,7 @@ add_tags_from_entry (RBEncoderGst *encoder,
 				done = TRUE;
 				break;
 			}
-		
+
 			if (tagger)
 				gst_object_unref (tagger);
 		}
@@ -503,11 +523,11 @@ gnomevfs_allow_overwrite_cb (GstElement *element, GnomeVFSURI *uri, RBEncoderGst
 					 display_name);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
-	
+
 	g_free (display_name);
 	g_free (name);
-	
-	return (response == GTK_RESPONSE_YES);				 
+
+	return (response == GTK_RESPONSE_YES);
 }
 
 static void
@@ -525,7 +545,7 @@ new_decoded_pad_cb (GstElement *decodebin,
 	 * so common anyway */
 	if (encoder->priv->decoded_pads > 0)
 		return;
-	
+
 	caps = gst_pad_get_caps (new_pad);
 	caps_string = gst_caps_to_string (caps);
 	gst_caps_unref (caps);
@@ -557,13 +577,13 @@ add_decoding_pipeline (RBEncoderGst *encoder,
 				RB_ENCODER_ERROR,
 				RB_ENCODER_ERROR_INTERNAL,
 				"Could not create decodebin");
-		
+
 		return NULL;
 	}
-	
+
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), decodebin);
-	
-	g_signal_connect_object (decodebin, 
+
+	g_signal_connect_object (decodebin,
 			"new-decoded-pad",
 			G_CALLBACK (new_decoded_pad_cb),
 			encoder, 0);
@@ -624,7 +644,7 @@ profile_bin_find_encoder (GstBin *profile_bin)
 	GstElement *encoder = NULL;
 	GstIterator *iter;
 	gboolean done = FALSE;
-	
+
 	iter = gst_bin_iterate_elements (profile_bin);
 	while (!done) {
 		gpointer data;
@@ -652,7 +672,7 @@ profile_bin_find_encoder (GstBin *profile_bin)
 		}
 	}
 	gst_iterator_free (iter);
-	
+
 	return encoder;
 #else
 	return NULL;
@@ -669,17 +689,18 @@ get_profile_from_mime_type (const char *mime_type)
 	GMAudioProfile *profile;
 	GMAudioProfile *matching_profile = NULL;
 	GError *error = NULL;
-	
+
 	profiles = gm_audio_profile_get_active_list ();
 	for (walk = profiles; walk; walk = g_list_next (walk)) {
 		profile = (GMAudioProfile *) walk->data;
 		pipeline_description =
-			g_strdup_printf ("fakesrc ! %s ! fakesink", 
+			g_strdup_printf ("fakesrc ! %s ! fakesink",
 				gm_audio_profile_get_pipeline (profile));
-		pipeline = gst_parse_launch (pipeline_description, &error);	
+		pipeline = gst_parse_launch (pipeline_description, &error);
 		g_free (pipeline_description);
 		if (error) {
 			g_error_free (error);
+			error = NULL;
 			continue;
 		}
 
@@ -688,14 +709,14 @@ get_profile_from_mime_type (const char *mime_type)
 			g_object_unref (pipeline);
 			continue;
 		}
-		
+
 		if (encoder_match_mime (encoder, mime_type)) {
 			matching_profile = profile;
 			gst_object_unref (GST_OBJECT (encoder));
 			gst_object_unref (GST_OBJECT (pipeline));
 			break;
 		}
-		
+
 		gst_object_unref (GST_OBJECT (encoder));
 		gst_object_unref (GST_OBJECT (pipeline));
 	}
@@ -809,7 +830,7 @@ extract_track (RBEncoderGst *encoder,
 		return FALSE;
 	if (gst_element_link (src, encoder->priv->enc) == FALSE)
 		return FALSE;
-	
+
 	if (!attach_output_pipeline (encoder, end, dest, error))
 		return FALSE;
 	if (!add_tags_from_entry (encoder, entry, error))
@@ -830,9 +851,9 @@ transcode_track (RBEncoderGst *encoder,
 	/* src ! decodebin ! queue ! encoding_profile ! queue ! sink */
 	GMAudioProfile *profile;
 	GstElement *src, *decoder, *end;
-	
+
 	g_assert (encoder->priv->pipeline == NULL);
-	
+
 	profile = get_profile_from_mime_type (mime_type);
 	if (profile == NULL) {
 		g_set_error (error,
@@ -871,34 +892,39 @@ transcode_track (RBEncoderGst *encoder,
 	return TRUE;
 }
 
-static void
+static GnomeVFSResult
 create_parent_dirs_uri (GnomeVFSURI *uri)
 {
 	GnomeVFSURI *parent_uri;
+	GnomeVFSResult result;
 
 	if (gnome_vfs_uri_exists (uri))
-		return;
+		return GNOME_VFS_OK;
 
 	parent_uri = gnome_vfs_uri_get_parent (uri);
-	create_parent_dirs_uri (parent_uri);
+	result = create_parent_dirs_uri (parent_uri);
 	gnome_vfs_uri_unref (parent_uri);
+	if (result != GNOME_VFS_OK)
+		return result;
 
-	gnome_vfs_make_directory_for_uri (uri, 0750);
+	return gnome_vfs_make_directory_for_uri (uri, 0750);
 }
 
-static void
+static GnomeVFSResult
 create_parent_dirs (const char *uri)
 {
 	GnomeVFSURI *vfs_uri;
 	GnomeVFSURI *parent_uri;
+	GnomeVFSResult result;
 
 	vfs_uri = gnome_vfs_uri_new (uri);
 	parent_uri = gnome_vfs_uri_get_parent (vfs_uri);
 
-	create_parent_dirs_uri (parent_uri);
+	result = create_parent_dirs_uri (parent_uri);
 
 	gnome_vfs_uri_unref (parent_uri);
 	gnome_vfs_uri_unref (vfs_uri);
+	return result;
 }
 
 static void
@@ -927,19 +953,34 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 	gboolean was_raw;
 	gboolean result;
 	GError *error = NULL;
+	GnomeVFSResult vfsresult;
 
 	g_return_val_if_fail (priv->pipeline == NULL, FALSE);
-	
+
 	entry_mime_type = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE);
 	was_raw = g_str_has_prefix (entry_mime_type, "audio/x-raw");
 
-	priv->total_length = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+	vfsresult = create_parent_dirs (dest);
+	if (vfsresult != GNOME_VFS_OK) {
+		error = g_error_new_literal (RB_ENCODER_ERROR,
+					     RB_ENCODER_ERROR_FILE_ACCESS,
+					     gnome_vfs_result_to_string (vfsresult));
 
-	create_parent_dirs (dest);
+		_rb_encoder_emit_error (encoder, error);
+		_rb_encoder_emit_completed (encoder);
+		g_error_free (error);
+		return FALSE;
+	}
 
 	if ((mime_type == NULL && !was_raw) || (mime_type && (strcmp (mime_type, entry_mime_type) == 0))) {
+		priv->total_length = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
+		priv->position_format = GST_FORMAT_BYTES;
+
 		result = copy_track (RB_ENCODER_GST (encoder), entry, dest, &error);
 	} else {
+		priv->total_length = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+		priv->position_format = GST_FORMAT_TIME;
+
 		if (mime_type == NULL) {
 			result = extract_track (RB_ENCODER_GST (encoder), entry, dest, &error);
 		} else {
@@ -949,7 +990,7 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 
 	if (error) {
 		RBEncoderGst *enc = RB_ENCODER_GST (encoder);
-		
+
 		rb_encoder_gst_emit_error (enc, error);
 		g_error_free (error);
 		if (enc->priv->pipeline == NULL)
