@@ -28,7 +28,7 @@
 
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
-#include <gst/gst.h>
+#include <gst/tag/tag.h>
 #ifdef HAVE_GSTREAMER_0_8
 #include <gst/gconf/gconf.h>
 #endif
@@ -41,11 +41,6 @@
 #include "rb-debug.h"
 #include "rb-marshal.h"
 
-#ifdef WITH_DAAP_SUPPORT
-#include "rb-daap-src.h"
-#endif
-
-
 static void rb_player_init (RBPlayerIface *iface);
 static void rb_player_gst_finalize (GObject *object);
 
@@ -73,7 +68,6 @@ G_DEFINE_TYPE_WITH_CODE(RBPlayerGst, rb_player_gst, G_TYPE_OBJECT,
 
 #define RB_PLAYER_GST_TICK_HZ 5
 
-
 struct _RBPlayerGstPrivate
 {
 	char *uri;
@@ -82,8 +76,10 @@ struct _RBPlayerGstPrivate
 
 	gboolean can_signal_direct_error;
 	GError *error;
+	gboolean emitted_error;
 
 	gboolean playing;
+	gboolean buffering;
 
 	guint idle_error_id;
 	guint idle_eos_id;
@@ -100,7 +96,6 @@ struct _RBPlayerGstPrivate
 	guint tick_timeout_id;
 };
 
-
 typedef enum
 {
 	EOS,
@@ -108,6 +103,7 @@ typedef enum
 	ERROR,
 	TICK,
 	BUFFERING,
+	EVENT,
 } RBPlayerGstSignalType;
 
 typedef struct
@@ -170,7 +166,7 @@ rb_player_gst_init (RBPlayerGst *mp)
 
 	mp->priv->tick_timeout_id = g_timeout_add (ms_period, (GSourceFunc) tick_timeout, mp);
 	mp->priv->idle_info_ids = g_hash_table_new (NULL, NULL);
-	
+
 }
 
 static void
@@ -190,10 +186,10 @@ rb_player_gst_finalize (GObject *object)
 		g_signal_handler_disconnect (G_OBJECT (mp->priv->playbin),
 					     mp->priv->buffering_signal_id);
 #endif
-		
+
 		gst_element_set_state (mp->priv->playbin,
 				       GST_STATE_NULL);
-		
+
 		rb_player_gst_gst_free_playbin (mp);
 	}
 
@@ -205,7 +201,7 @@ rb_player_gst_gst_free_playbin (RBPlayerGst *player)
 {
 	if (player->priv->playbin == NULL)
 		return;
-	
+
 	gst_object_unref (GST_OBJECT (player->priv->playbin));
 	player->priv->playbin = NULL;
 }
@@ -227,7 +223,7 @@ destroy_idle_signal (gpointer signal_pointer)
 		g_hash_table_remove (signal->object->priv->idle_info_ids,
 				     GUINT_TO_POINTER (signal->id));
 	}
-	
+
 	g_object_unref (G_OBJECT (signal->object));
 	g_free (signal);
 
@@ -259,6 +255,9 @@ emit_signal_idle (RBPlayerGstSignal *signal)
 		_rb_player_emit_buffering (RB_PLAYER (signal->object), g_value_get_uint (signal->info));
 		signal->object->priv->idle_buffering_id = 0;
 		break;
+	case EVENT:
+		_rb_player_emit_event (RB_PLAYER (signal->object), g_value_get_string (signal->info), NULL);
+		break;
 	}
 
 	return FALSE;
@@ -304,6 +303,16 @@ error_cb (GstElement *element,
 		code = RB_PLAYER_ERROR_GENERAL;
 	}
 
+	/* If we've already got an error, ignore 'internal data flow error'
+	 * type messages, as they're too generic to be helpful.
+	 */
+	if (mp->priv->emitted_error &&
+	    error->domain == GST_STREAM_ERROR &&
+	    error->code == GST_STREAM_ERROR_FAILED) {
+		rb_debug ("Ignoring generic error \"%s\"", error->message);
+		return;
+	}
+
 	/* If we're in a synchronous op, we can signal the error directly */
 	if (mp->priv->can_signal_direct_error) {
 		if (mp->priv->error) {
@@ -312,6 +321,7 @@ error_cb (GstElement *element,
 				   error->message);
 			g_error_free (mp->priv->error);
 		}
+		mp->priv->emitted_error = TRUE;
 		mp->priv->error = g_error_copy (error);
 		return;
 	}
@@ -331,6 +341,7 @@ error_cb (GstElement *element,
 						   (GSourceFunc) emit_signal_idle,
 						   signal,
 						   destroy_idle_signal);
+	mp->priv->emitted_error = TRUE;
 }
 #endif
 
@@ -346,7 +357,7 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 	count = gst_tag_list_get_tag_size (list, tag);
 	if (count < 1)
 		return;
-	
+
 	/* only handle the subset of fields we use for iradio */
 	if (!strcmp (tag, GST_TAG_TITLE))
 		field = RB_METADATA_FIELD_TITLE;
@@ -356,6 +367,10 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 		field = RB_METADATA_FIELD_COMMENT;
 	else if (!strcmp (tag, GST_TAG_BITRATE))
 		field = RB_METADATA_FIELD_BITRATE;
+#ifdef GST_TAG_MUSICBRAINZ_TRACKID
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID))
+		field = RB_METADATA_FIELD_MUSICBRAINZ_TRACKID;
+#endif
 	else
 		return;
 
@@ -369,6 +384,7 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 	case RB_METADATA_FIELD_TITLE:
 	case RB_METADATA_FIELD_GENRE:
 	case RB_METADATA_FIELD_COMMENT:
+	case RB_METADATA_FIELD_MUSICBRAINZ_TRACKID:
 	default:
 		g_value_init (newval, G_TYPE_STRING);
 		break;
@@ -431,9 +447,20 @@ rb_player_gst_bus_cb (GstBus * bus, GstMessage * message, RBPlayerGst *mp)
 		char *debug;
 		GError *error, *sig_error;
 		int code;
+		gboolean emit = TRUE;
 
 		gst_message_parse_error (message, &error, &debug);
 
+		/* If we've already got an error, ignore 'internal data flow error'
+		 * type messages, as they're too generic to be helpful.
+		 */
+		if (mp->priv->emitted_error &&
+		    error->domain == GST_STREAM_ERROR &&
+		    error->code == GST_STREAM_ERROR_FAILED) {
+			rb_debug ("Ignoring generic error \"%s\"", error->message);
+			emit = FALSE;
+		}
+
 		if ((error->domain == GST_CORE_ERROR)
 			|| (error->domain == GST_LIBRARY_ERROR)
 			|| (error->domain == GST_RESOURCE_ERROR && error->code == GST_RESOURCE_ERROR_BUSY)) {
@@ -442,10 +469,13 @@ rb_player_gst_bus_cb (GstBus * bus, GstMessage * message, RBPlayerGst *mp)
 			code = RB_PLAYER_ERROR_GENERAL;
 		}
 
-		sig_error = g_error_new_literal (RB_PLAYER_ERROR,
-						 code,
-						 error->message);
-		_rb_player_emit_error (RB_PLAYER (mp), sig_error);
+		if (emit) {
+			sig_error = g_error_new_literal (RB_PLAYER_ERROR,
+							 code,
+							 error->message);
+			mp->priv->emitted_error = TRUE;
+			_rb_player_emit_error (RB_PLAYER (mp), sig_error);
+		}
 
 		/* close if not already closing */
 		if (mp->priv->uri != NULL)
@@ -476,6 +506,26 @@ rb_player_gst_bus_cb (GstBus * bus, GstMessage * message, RBPlayerGst *mp)
 			g_warning ("Could not get value from BUFFERING message");
 			break;
 		}
+		if (progress >= 100) {
+			mp->priv->buffering = FALSE;
+			if (mp->priv->playing) {
+				rb_debug ("buffering done, setting pipeline back to PLAYING");
+				gst_element_set_state (mp->priv->playbin, GST_STATE_PLAYING);
+			} else {
+				rb_debug ("buffering done, leaving pipeline PAUSED");
+			}
+		} else if (mp->priv->buffering == FALSE && mp->priv->playing) {
+			GstState cur_state;
+
+			gst_element_get_state (mp->priv->playbin, &cur_state, NULL, 0);
+			if (cur_state == GST_STATE_PLAYING) {
+				rb_debug ("buffering - temporarily pausing playback");
+				gst_element_set_state (mp->priv->playbin, GST_STATE_PAUSED);
+			} else {
+				rb_debug ("buffering - during preroll; doing nothing");
+			}
+			mp->priv->buffering = TRUE;
+		}
 		signal = g_new0 (RBPlayerGstSignal, 1);
 		signal->type = BUFFERING;
 
@@ -486,14 +536,29 @@ rb_player_gst_bus_cb (GstBus * bus, GstMessage * message, RBPlayerGst *mp)
 		g_value_set_uint (signal->info, (guint)progress);
 		g_idle_add ((GSourceFunc) emit_signal_idle, signal);
 		break;
-	} 
+	}
+	case GST_MESSAGE_APPLICATION: {
+		RBPlayerGstSignal *signal;
+		const GstStructure *structure;
+
+		structure = gst_message_get_structure (message);
+		signal = g_new0 (RBPlayerGstSignal, 1);
+		signal->type = EVENT;
+
+		g_object_ref (G_OBJECT (mp));
+		signal->object = mp;
+		signal->info = g_new0 (GValue, 1);
+		g_value_init (signal->info, G_TYPE_STRING);
+		g_value_set_string (signal->info, gst_structure_get_name (structure));
+		g_idle_add ((GSourceFunc) emit_signal_idle, signal);
+	}
 	default:
 		break;
 	}
 
 	return TRUE;
 }
- 
+
 #endif
 
 static gboolean
@@ -523,7 +588,7 @@ rb_player_gst_construct (RBPlayerGst *mp, GError **error)
 					 "buffering",
 					 G_CALLBACK (buffering_cb),
 					 mp, 0);
-	
+
 	mp->priv->error_signal_id =
 		g_signal_connect_object (G_OBJECT (mp->priv->playbin),
 					 "error",
@@ -543,10 +608,15 @@ rb_player_gst_construct (RBPlayerGst *mp, GError **error)
 	/* Output sink */
 	sink = gst_element_factory_make ("gconfaudiosink", "audiosink");
 #endif
-	
+
 	/* if we could create the gconf sink use that, otherwise let playbin decide */
-	if (sink != NULL)
+	if (sink != NULL) {
+		/* set the profile property on the gconfaudiosink to "music and movies" */
+		if (g_object_class_find_property (G_OBJECT_GET_CLASS (sink), "profile"))
+			g_object_set (G_OBJECT (sink), "profile", 1, NULL);
+
 		g_object_set (G_OBJECT (mp->priv->playbin), "audio-sink", sink, NULL);
+	}
 
 	if (mp->priv->cur_volume > 1.0)
 		mp->priv->cur_volume = 1.0;
@@ -604,7 +674,7 @@ rb_player_gst_sync_pipeline (RBPlayerGst *mp)
 		}
 	}
 #ifdef HAVE_GSTREAMER_0_10
-	/* FIXME: Set up a timeout to watch if the pipeline doesn't 
+	/* FIXME: Set up a timeout to watch if the pipeline doesn't
          * go to PAUSED/PLAYING within some time (5 secs maybe?)
 	 */
 #endif
@@ -640,7 +710,7 @@ end_gstreamer_operation (RBPlayerGst *mp, gboolean op_failed, GError **error)
 }
 
 static void
-cdda_got_source_cb (GObject *object, GParamSpec *pspec, char *uri)
+cdda_got_source_cb (GObject *object, GParamSpec *pspec, char *device)
 {
 	GstElement *source;
 
@@ -651,13 +721,16 @@ cdda_got_source_cb (GObject *object, GParamSpec *pspec, char *uri)
 #endif
 	rb_debug ("got source %p", source);
 	if (source) {
-		g_signal_handlers_disconnect_by_func (object, cdda_got_source_cb, uri);
+		g_signal_handlers_disconnect_by_func (object, cdda_got_source_cb, device);
 
-		g_object_set (G_OBJECT (source), "device", uri, NULL);
-		g_free (uri);
+		g_object_set (G_OBJECT (source), "device", device, NULL);
+		g_free (device);
 
 		if (g_object_class_find_property (G_OBJECT_GET_CLASS (source), "paranoia-mode"))
 			g_object_set (G_OBJECT (source), "paranoia-mode", 0, NULL);
+
+		if (g_object_class_find_property (G_OBJECT_GET_CLASS (source), "read-speed"))
+			g_object_set (G_OBJECT (source), "read-speed", 1, NULL);
 	}
 }
 
@@ -679,23 +752,24 @@ rb_player_gst_open (RBPlayer *player,
 
 	g_assert (mp->priv->playbin != NULL);
 
-	g_free (mp->priv->uri);
-	mp->priv->uri = NULL;
-
 	if (uri == NULL) {
+		g_free (mp->priv->uri);
+		mp->priv->uri = NULL;
 		mp->priv->playing = FALSE;
+		mp->priv->buffering = FALSE;
 		return TRUE;
 	}
 
 	/* check if we are switching tracks on a cd, so we don't have to close the device */
-	if (mp->priv->uri && g_str_has_prefix (uri, "cdda://")) {
-		const char *old_device;
+	if (g_str_has_prefix (uri, "cdda://")) {
+		const char *old_device = NULL;
 		const char *new_device;
 
-		old_device = g_utf8_strchr (mp->priv->uri, -1, '#');
+		if (mp->priv->uri && g_str_has_prefix (mp->priv->uri, "cdda://"))
+			old_device = g_utf8_strchr (mp->priv->uri, -1, '#');
 		new_device = g_utf8_strchr (uri, -1, '#');
-	
-		if (strcmp (old_device, new_device) == 0) {
+
+		if (old_device && strcmp (old_device, new_device) == 0) {
 			/* just seek, instead of having playbin close the device */
 			GstFormat track_format = gst_format_get_by_nick ("track");
 			char *track_str;
@@ -707,9 +781,11 @@ rb_player_gst_open (RBPlayer *player,
 			track = atoi (track_str);
 			g_free (track_str);
 
+			rb_debug ("seeking to track %d on CD device %s", track, new_device);
+
 #ifdef HAVE_GSTREAMER_0_10
-			if (gst_element_seek (mp->priv->playbin, 1.0, 
-					      track_format, GST_SEEK_FLAG_FLUSH, 
+			if (gst_element_seek (mp->priv->playbin, 1.0,
+					      track_format, GST_SEEK_FLAG_FLUSH,
 					      GST_SEEK_TYPE_SET, track,
 					      GST_SEEK_TYPE_NONE, -1))
 				cdda_seek = TRUE;
@@ -717,18 +793,28 @@ rb_player_gst_open (RBPlayer *player,
 			{
 				GstEvent *event;
 
-				event = gst_event_new_seek (track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, 
+				event = gst_event_new_seek (track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH,
 						    (guint64) track);
 				if (gst_element_send_event (mp->priv->playbin, event))
 					cdda_seek = TRUE;
 			}
 #endif
+		} else {
+			/* +1 to skip the '#' */
+			char *device = g_strdup (new_device + 1);
 
+			rb_debug ("waiting for source element for CD device %s", device);
+			g_signal_connect (G_OBJECT (mp->priv->playbin),
+					  "notify::source",
+					  G_CALLBACK (cdda_got_source_cb),
+					  device);
 		}
 	}
 
 	begin_gstreamer_operation (mp);
+	g_free (mp->priv->uri);
 	mp->priv->uri = g_strdup (uri);
+	mp->priv->emitted_error = FALSE;
 
 	if (!cdda_seek) {
 		g_object_set (G_OBJECT (mp->priv->playbin), "uri", uri, NULL);
@@ -759,6 +845,7 @@ rb_player_gst_close (RBPlayer *player, GError **error)
 	gboolean ret;
 
 	mp->priv->playing = FALSE;
+	mp->priv->buffering = FALSE;
 
 	g_free (mp->priv->uri);
 	mp->priv->uri = NULL;
@@ -805,6 +892,7 @@ rb_player_gst_play (RBPlayer *player, GError **error)
 	gboolean ret;
 
 	mp->priv->playing = TRUE;
+	mp->priv->buffering = FALSE;
 
 	g_return_val_if_fail (mp->priv->playbin != NULL, FALSE);
 
@@ -854,7 +942,7 @@ rb_player_gst_set_replaygain (RBPlayer *player,
 
 	if (gain == 0)
 		return;
-	  
+
 	scale = pow (10., gain / 20);
 
 	/* anti clip */
@@ -865,25 +953,25 @@ rb_player_gst_set_replaygain (RBPlayer *player,
 
 	if (peak != 0 && (scale * peak) > 1)
 		scale = 1.0 / peak;
-		
+
 	/* For security */
 	if (scale > 15)
 		scale = 15;
-	
+
         rb_debug ("Scale : %f New volume : %f", scale, mp->priv->cur_volume * scale);
 
 	if (mp->priv->playbin != NULL) {
 		GParamSpec *volume_pspec;
 		GValue val = {0,};
-		
+
 		volume_pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (mp->priv->playbin),
 							     "volume");
 		g_value_init (&val, G_TYPE_DOUBLE);
-		
+
 		g_value_set_double (&val, mp->priv->cur_volume * scale);
 		if (g_param_value_validate (volume_pspec, &val))
 			rb_debug ("replay gain too high, reducing value to %f", g_value_get_double (&val));
-	
+
 		g_object_set_property (G_OBJECT (mp->priv->playbin), "volume", &val);
 		g_value_unset (&val);
 	}
@@ -928,7 +1016,9 @@ rb_player_gst_seekable (RBPlayer *player)
 	GstQuery *query;
 #endif
 
-	g_return_val_if_fail (mp->priv->playbin != NULL, FALSE);
+	if (mp->priv->playbin == NULL)
+		return FALSE;
+
 #ifdef HAVE_GSTREAMER_0_10
 	query = gst_query_new_seeking (GST_FORMAT_TIME);
 	if (gst_element_query (mp->priv->playbin, query)) {
@@ -964,40 +1054,19 @@ rb_player_gst_set_time (RBPlayer *player, long time)
 	}
 #endif
 
-#ifdef WITH_DAAP_SUPPORT
-	/* FIXME?
-	 * This is sorta hack/sorta best way to do it.
-	 * If we set up the daapsrc to do regular GStreamer seeking,
-	 * GStreamer goes ape-shit and tries to seek all over the 
-	 * place (typefinding), which can cause iTunes to return errors
-	 * (probably cause we're requesting the same file too often)
- 	 *
-	 * So, we do it this way.  Doesn't that suck?
-	 */
-	if (mp->priv->uri && g_strncasecmp (mp->priv->uri, "daap://", 7) == 0) {
-		GstElement *src;
-
-		g_object_get (G_OBJECT (mp->priv->playbin), "source", &src, NULL);
-		if (src)
-			rb_daap_src_set_time (src, time);
-	} else {
-#endif
 #ifdef HAVE_GSTREAMER_0_8
-	gst_element_seek (mp->priv->playbin, 
-			  GST_FORMAT_TIME 
-			  | GST_SEEK_METHOD_SET 
-			  | GST_SEEK_FLAG_FLUSH, 
+	gst_element_seek (mp->priv->playbin,
+			  GST_FORMAT_TIME
+			  | GST_SEEK_METHOD_SET
+			  | GST_SEEK_FLAG_FLUSH,
 			  time * GST_SECOND);
 #elif HAVE_GSTREAMER_0_10
-	gst_element_seek (mp->priv->playbin, 1.0, 
-			  GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, 
+	gst_element_seek (mp->priv->playbin, 1.0,
+			  GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH,
 			  GST_SEEK_TYPE_SET, time * GST_SECOND,
 			  GST_SEEK_TYPE_NONE, -1);
 #endif
-	
-#ifdef WITH_DAAP_SUPPORT
-	}
-#endif
+
 	if (mp->priv->playing) {
 		gst_element_set_state (mp->priv->playbin, GST_STATE_PLAYING);
 	}
@@ -1011,7 +1080,7 @@ rb_player_gst_get_time (RBPlayer *player)
 	if (mp->priv->playbin != NULL) {
 		gint64 position = -1;
 		GstFormat fmt = GST_FORMAT_TIME;
-		
+
 #ifdef HAVE_GSTREAMER_0_8
 		gst_element_query (mp->priv->playbin, GST_QUERY_POSITION, &fmt, &position);
 #elif HAVE_GSTREAMER_0_10
@@ -1020,23 +1089,7 @@ rb_player_gst_get_time (RBPlayer *player)
 		if (position != -1)
 			position /= GST_SECOND;
 
-		/* FIXME whether this is a good idea or not depends on gstreamer version 
-		 * and filetype.  ugh.  for now, disabling it with gstreamer 0.8 seems to 
-		 * give the best results.
-		 */
-#ifdef HAVE_GSTREAMER_0_8
-#ifdef WITH_DAAP_SUPPORT
-		if (mp->priv->uri && g_strncasecmp (mp->priv->uri, "daap://", 7) == 0) {
-			GstElement *src;
-			g_object_get (G_OBJECT (mp->priv->playbin), "source", &src, NULL);
-			if (src) 
-				position += rb_daap_src_get_time (src);
-		}
-#endif
-#endif
-
 		return (long) position;
 	} else
 		return -1;
 }
-
