@@ -47,6 +47,7 @@
 #include "rb-daap-source.h"
 #include "rb-daap-src.h"
 #include "rb-debug.h"
+#include "rb-daap-plugin.h"
 
 #define RB_TYPE_DAAP_SRC (rb_daap_src_get_type())
 #define RB_DAAP_SRC(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),RB_TYPE_DAAP_SRC,RBDAAPSrc))
@@ -127,6 +128,8 @@ GST_ELEMENT_DETAILS ("RBDAAP Source",
 	"Read a DAAP (music share) file",
 	"Charles Schmidt <cschmidt2@emich.edu");
 
+static RBDaapPlugin *daap_plugin = NULL;
+
 static void rb_daap_src_uri_handler_init (gpointer g_iface, gpointer iface_data);
 
 static void
@@ -162,8 +165,6 @@ static void rb_daap_src_get_property (GObject *object,
 			  GParamSpec *pspec);
 
 #ifdef HAVE_GSTREAMER_0_8
-static GstCaps *rb_daap_src_getcaps (GstPad *pad);
-
 static GstData *rb_daap_src_get (GstPad *pad);
 
 static GstElementStateReturn rb_daap_src_change_state (GstElement *element);
@@ -185,6 +186,13 @@ static gboolean rb_daap_src_do_seek (GstBaseSrc *src, GstSegment *segment);
 static GstFlowReturn rb_daap_src_create (GstPushSrc *psrc, GstBuffer **outbuf);
 #endif
 
+void
+rb_daap_src_set_plugin (RBPlugin *plugin)
+{
+	g_assert (RB_IS_DAAP_PLUGIN (plugin));
+	daap_plugin = RB_DAAP_PLUGIN (plugin);
+}
+
 #ifdef HAVE_GSTREAMER_0_8
 
 static const GstFormat *
@@ -328,8 +336,6 @@ rb_daap_src_init (RBDAAPSrc *src, RBDAAPSrcClass *klass)
 	src->need_flush = FALSE;
 
 	src->srcpad = gst_pad_new ("src", GST_PAD_SRC);
-	gst_pad_set_getcaps_function (src->srcpad,
-				      rb_daap_src_getcaps);
 	gst_pad_set_get_function (src->srcpad,
 				  rb_daap_src_get);
 	gst_pad_set_event_mask_function (src->srcpad,
@@ -639,7 +645,7 @@ rb_daap_src_open (RBDAAPSrc *src)
 	}
 
 	/* construct request */
-	source = rb_daap_source_find_for_uri (src->daap_uri);
+	source = rb_daap_plugin_find_source_for_uri (daap_plugin, src->daap_uri);
 	if (source == NULL) {
 		g_warning ("Unable to lookup source for URI: %s", src->daap_uri);
 		return FALSE;
@@ -730,6 +736,9 @@ rb_daap_src_open (RBDAAPSrc *src)
 								   ("Couldn't read HTTP content length \"%s\"", val->data));
 						ok = FALSE;
 					}
+				} else {
+					GST_DEBUG_OBJECT (src, "Response doesn't have a content length");
+					src->size = 0;
 				}
 			}
 
@@ -1116,7 +1125,7 @@ gboolean
 rb_daap_src_get_size (GstBaseSrc *bsrc, guint64 *size)
 {
 	RBDAAPSrc *src = RB_DAAP_SRC (bsrc);
-	if (!src->chunked) {
+	if (src->chunked == FALSE && src->size > 0) {
 		*size = src->size;
 		return TRUE;
 	}
