@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  *  NOTES: log domain hack stolen from nautilus
  *
@@ -48,14 +48,15 @@ void
 rb_debug_real (const char *func,
 	       const char *file,
 	       const int line,
+	       gboolean newline,
 	       const char *format, ...)
 {
 	va_list args;
 	char buffer[1025];
-	char *str_time;
+	char str_time[255];
 	time_t the_time;
 
-	if (debug_match == NULL || 
+	if (debug_match == NULL ||
 	   (debug_match != debug_everything &&
 	   (strstr (file, debug_match) == NULL) &&
 	   (strstr (func, debug_match) == NULL)))
@@ -64,17 +65,14 @@ rb_debug_real (const char *func,
 	va_start (args, format);
 
 	g_vsnprintf (buffer, 1024, format, args);
-	
+
 	va_end (args);
 
 	time (&the_time);
-	str_time = g_new0 (char, 255);
 	strftime (str_time, 254, "%H:%M:%S", localtime (&the_time));
 
-	g_printerr ("(%s) [%p] [%s] %s:%d: %s\n", str_time, g_thread_self (),
-		    func, file, line, buffer);
-	
-	g_free (str_time);
+	g_printerr (newline ? "(%s) [%p] [%s] %s:%d: %s\n" : "(%s) [%p] [%s] %s:%d: %s",
+		    str_time, g_thread_self (), func, file, line, buffer);
 }
 
 void
@@ -185,7 +183,7 @@ rb_profiler_new (const char *name)
 {
 	RBProfiler *profiler;
 	
-	if (debug_match == FALSE)
+	if (debug_match == NULL)
 		return NULL;
 
 	profiler = g_new0 (RBProfiler, 1);
