@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -42,12 +42,27 @@ char *		rb_uri_resolve_relative	(const char *uri);
 gboolean	rb_uri_is_readable	(const char *uri);
 gboolean	rb_uri_is_writable	(const char *uri);
 gboolean	rb_uri_is_local		(const char *uri);
+char *		rb_uri_get_dir_name	(const char *uri);
+char *		rb_uri_get_short_path_name (const char *uri);
+
+typedef void (*RBUriRecurseFunc) (const char *uri, gboolean dir, gpointer data);
 
 void		rb_uri_handle_recursively(const char *uri,
-					  GFunc func,
+					  RBUriRecurseFunc func,
 					  gboolean *cancelflag,
 					  gpointer user_data);
 
+void		rb_uri_handle_recursively_async(const char *uri,
+						RBUriRecurseFunc func,
+						gboolean *cancelflag,
+						gpointer user_data,
+						GDestroyNotify data_destroy);
+
+char*		rb_uri_append_path	(const char *uri,
+					 const char *path);
+char*		rb_uri_append_uri	(const char *uri,
+					 const char *fragment);
+
 void		rb_file_helpers_init	(void);
 void		rb_file_helpers_shutdown(void);
 
