@@ -117,7 +117,7 @@ struct _RBAudioscrobblerPrivate
 	time_t submit_interval;
 
 	/* Whether this song should be queued once enough of it has been played;
-	 * will be set to false when queued, or if the song shouldn't be submitted 
+	 * will be set to false when queued, or if the song shouldn't be submitted
 	 */
 	gboolean should_queue;
 	/* Only write the queue to a file if it has been changed */
@@ -133,6 +133,7 @@ struct _RBAudioscrobblerPrivate
 	gchar *artist;
 	gchar *album;
 	gchar *title;
+	gchar *mbid;
 	guint duration;
 	guint elapsed;
 
@@ -259,6 +260,7 @@ rb_audioscrobbler_init (RBAudioscrobbler *audioscrobbler)
 	audioscrobbler->priv->artist = g_strdup ("");
 	audioscrobbler->priv->album = g_strdup ("");
 	audioscrobbler->priv->title = g_strdup ("");
+	audioscrobbler->priv->mbid = g_strdup ("");
 	audioscrobbler->priv->duration = 0;
 	audioscrobbler->priv->elapsed = 0;
 	audioscrobbler->priv->timeout_id = 0;
@@ -268,11 +270,11 @@ rb_audioscrobbler_init (RBAudioscrobbler *audioscrobbler)
 	rb_audioscrobbler_import_settings (audioscrobbler);
 
 	/* gconf notifications: */
-	audioscrobbler->priv->notification_username_id = 
+	audioscrobbler->priv->notification_username_id =
 		eel_gconf_notification_add (CONF_AUDIOSCROBBLER_USERNAME,
 				    (GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
 				    audioscrobbler);
-	audioscrobbler->priv->notification_password_id = 
+	audioscrobbler->priv->notification_password_id =
 		eel_gconf_notification_add (CONF_AUDIOSCROBBLER_PASSWORD,
 				    (GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
 				    audioscrobbler);
@@ -309,8 +311,11 @@ rb_audioscrobbler_finalize (GObject *object)
 	g_free (audioscrobbler->priv->artist);
 	g_free (audioscrobbler->priv->album);
 	g_free (audioscrobbler->priv->title);
-	if (audioscrobbler->priv->soup_session)
+	g_free (audioscrobbler->priv->mbid);
+	if (audioscrobbler->priv->soup_session) {
+		soup_session_abort (audioscrobbler->priv->soup_session);
 		g_object_unref (G_OBJECT (audioscrobbler->priv->soup_session));
+	}
 	g_object_unref (G_OBJECT (audioscrobbler->priv->proxy_config));
 	g_object_unref (G_OBJECT (audioscrobbler->priv->shell_player));
 
@@ -423,8 +428,8 @@ rb_audioscrobbler_timeout_cb (RBAudioscrobbler *audioscrobbler)
 				else
 					entry->album = g_strdup ("");
 				entry->title = soup_uri_encode (audioscrobbler->priv->title, EXTRA_URI_ENCODE_CHARS);
+				entry->mbid = soup_uri_encode (audioscrobbler->priv->mbid, EXTRA_URI_ENCODE_CHARS);
 				entry->length = audioscrobbler->priv->duration;
-				entry->mbid = g_strdup ("");
 				entry->timestamp = g_new0 (gchar, 30);
 				strftime (entry->timestamp, 30, "%Y%%2D%m%%2D%d%%20%H%%3A%M%%3A%S", gmtime (&tt));
 
@@ -441,7 +446,7 @@ rb_audioscrobbler_timeout_cb (RBAudioscrobbler *audioscrobbler)
 
 			/* Mark current track as having been queued. */
 			audioscrobbler->priv->should_queue = FALSE;
-			
+
 			rb_audioscrobbler_preferences_sync (audioscrobbler);
 		} else if (elapsed_delta > 20) {
 			rb_debug ("Skipping detected; not submitting current song");
@@ -504,10 +509,10 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL) {
 		gchar *body;
 		gchar **breaks;
-		
+
 		body = g_malloc0 ((msg->response).length + 1);
 		memcpy (body, (msg->response).body, (msg->response).length);
-		
+
 		g_strstrip (body);
 		breaks = g_strsplit (body, "\n", 4);
 		int i;
@@ -589,6 +594,9 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 	}
 }
 
+/*
+ * NOTE: the caller *must* unref the audioscrobbler object in the callback
+ */
 static void
 rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 			   char *url,
@@ -598,21 +606,20 @@ rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 	SoupMessage *msg;
 
 	msg = soup_message_new (post_data == NULL ? "GET" : "POST", url);
-	soup_message_set_http_version (msg, SOUP_HTTP_1_0);
 
 	if (post_data != NULL) {
 		rb_debug ("Submitting to Audioscrobbler: %s", post_data);
-		soup_message_set_request (msg, 
+		soup_message_set_request (msg,
 					  "application/x-www-form-urlencoded",
-					  SOUP_BUFFER_SYSTEM_OWNED, 
-					  post_data, 
+					  SOUP_BUFFER_SYSTEM_OWNED,
+					  post_data,
 					  strlen (post_data));
 	}
 
 	/* create soup session, if we haven't got one yet */
 	if (!audioscrobbler->priv->soup_session) {
 		SoupUri *uri;
-		
+
 		uri = rb_proxy_config_get_libsoup_uri (audioscrobbler->priv->proxy_config);
 		audioscrobbler->priv->soup_session = soup_session_async_new_with_options (
 					"proxy-uri", uri,
@@ -620,11 +627,11 @@ rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 		if (uri)
 			soup_uri_free (uri);
 	}
-	
+
 	soup_session_queue_message (audioscrobbler->priv->soup_session,
 				    msg,
 				    response_handler,
-				    audioscrobbler);
+				    g_object_ref (audioscrobbler));
 }
 
 static void
@@ -678,14 +685,14 @@ rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler)
 
 
 static void
-rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data) 
+rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER(user_data);
 
 	rb_debug ("Handshake response");
 	rb_audioscrobbler_parse_response (audioscrobbler, msg);
 	rb_audioscrobbler_preferences_sync (audioscrobbler);
-	
+
 	switch (audioscrobbler->priv->status) {
 	case STATUS_OK:
 	case CLIENT_UPDATE_REQUIRED:
@@ -697,6 +704,8 @@ rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
 		++audioscrobbler->priv->failures;
 		break;
 	}
+
+	g_object_unref (audioscrobbler);
 }
 
 static void
@@ -784,13 +793,13 @@ rb_audioscrobbler_submit_queue (RBAudioscrobbler *audioscrobbler)
 }
 
 static void
-rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data) 
+rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER (user_data);
 
 	rb_debug ("Submission response");
 	rb_audioscrobbler_parse_response (audioscrobbler, msg);
-	
+
 	if (audioscrobbler->priv->status == STATUS_OK) {
 		rb_debug ("Queue submitted successfully");
 		rb_audioscrobbler_free_queue_entries (audioscrobbler, &audioscrobbler->priv->submission);
@@ -805,7 +814,7 @@ rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
 		++audioscrobbler->priv->failures;
 
 		/* add failed submission entries back to queue */
-		audioscrobbler->priv->queue = 
+		audioscrobbler->priv->queue =
 			g_slist_concat (audioscrobbler->priv->submission, audioscrobbler->priv->queue);
 		audioscrobbler->priv->submission = NULL;
 		rb_audioscrobbler_save_queue (audioscrobbler);
@@ -813,7 +822,7 @@ rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
 		rb_audioscrobbler_print_queue (audioscrobbler, FALSE);
 
 		if (audioscrobbler->priv->failures >= 3) {
-			rb_debug ("Queue submission has failed %d times; caching tracks locally", 
+			rb_debug ("Queue submission has failed %d times; caching tracks locally",
 				  audioscrobbler->priv->failures);
 			g_free (audioscrobbler->priv->status_msg);
 
@@ -824,8 +833,9 @@ rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
 			rb_debug ("Queue submission failed %d times", audioscrobbler->priv->failures);
 		}
 	}
-	
+
 	rb_audioscrobbler_preferences_sync (audioscrobbler);
+	g_object_unref (audioscrobbler);
 }
 
 /* Configuration functions: */
@@ -903,7 +913,7 @@ rb_audioscrobbler_preferences_sync (RBAudioscrobbler *audioscrobbler)
 	gtk_label_set_text (GTK_LABEL (audioscrobbler->priv->status_label),
 			    status);
 	g_free (free_this);
-	
+
 	free_this = g_strdup_printf ("%u", audioscrobbler->priv->submit_count);
 	gtk_label_set_text (GTK_LABEL (audioscrobbler->priv->submit_count_label), free_this);
 	g_free (free_this);
@@ -912,7 +922,7 @@ rb_audioscrobbler_preferences_sync (RBAudioscrobbler *audioscrobbler)
 	gtk_label_set_text (GTK_LABEL (audioscrobbler->priv->queue_count_label), free_this);
 	g_free (free_this);
 
-	gtk_label_set_text (GTK_LABEL (audioscrobbler->priv->submit_time_label), 
+	gtk_label_set_text (GTK_LABEL (audioscrobbler->priv->submit_time_label),
 			    audioscrobbler->priv->submit_time);
 }
 
@@ -983,7 +993,7 @@ rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
 	} else if (strcmp (entry->key, CONF_AUDIOSCROBBLER_PASSWORD) == 0) {
 		g_free (audioscrobbler->priv->password);
 		audioscrobbler->priv->password = g_strdup (gconf_value_get_string (entry->value));
-	
+
 		if (audioscrobbler->priv->password_entry)
 			gtk_entry_set_text (GTK_ENTRY (audioscrobbler->priv->password_entry),
 					    gconf_value_get_string (entry->value));
@@ -998,9 +1008,6 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 				   RBAudioscrobbler *audioscrobbler)
 {
 	RhythmDBEntryType type;
-	gchar *old_artist;
-	gchar *old_album;
-	gchar *old_title;
 
 	if (entry == NULL) {
 		audioscrobbler->priv->should_queue = FALSE;
@@ -1008,22 +1015,18 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 	}
 
 	type = rhythmdb_entry_get_entry_type (entry);
-	if (type == RHYTHMDB_ENTRY_TYPE_IRADIO_STATION ||
+	if (type->category != RHYTHMDB_ENTRY_NORMAL ||
 	    type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST ||
-	    type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED ||
 	    rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR) != NULL) {
 		audioscrobbler->priv->should_queue = FALSE;
 		return;
 	}
 
-	old_artist = audioscrobbler->priv->artist;
-	old_album = audioscrobbler->priv->album;
-	old_title = audioscrobbler->priv->title;
-
 	audioscrobbler->priv->title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	audioscrobbler->priv->artist = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST);
 	audioscrobbler->priv->album = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ALBUM);
 	audioscrobbler->priv->duration = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+	audioscrobbler->priv->mbid = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID);
 	guint time;
 	rb_shell_player_get_playing_time (audioscrobbler->priv->shell_player, &time, NULL);
 	audioscrobbler->priv->elapsed = (int) time;
@@ -1033,15 +1036,12 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 	    ! strcmp (audioscrobbler->priv->artist, _("Unknown")) ||
 	    ! strcmp (audioscrobbler->priv->title, _("Unknown"))) {
 		audioscrobbler->priv->should_queue = FALSE;
-	} else if (strcmp(audioscrobbler->priv->artist, old_artist) ||
-		   strcmp(audioscrobbler->priv->album, old_album) ||
-		   strcmp(audioscrobbler->priv->title, old_title)) {
+	} else if (time < 15) {
+		/* even if it's the same song, it's being played again from
+		 * the start so we can queue it again.
+		 */
 		audioscrobbler->priv->should_queue = TRUE;
 	}
-
-	g_free (old_artist);
-	g_free (old_album);
-	g_free (old_title);
 }
 
 
@@ -1171,7 +1171,7 @@ rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler)
 
 	result = gnome_vfs_read_entire_file (uri, &size, &data);
 	g_free (uri);
-	
+
 	/* do stuff */
 	if (result == GNOME_VFS_OK) {
 		char *start = data, *end;
@@ -1226,9 +1226,9 @@ rb_audioscrobbler_save_queue (RBAudioscrobbler *audioscrobbler)
 		GString *s = g_string_new (NULL);
 		GSList *l;
 
-		for (l = audioscrobbler->priv->queue; l; l = g_slist_next (l)) { 
+		for (l = audioscrobbler->priv->queue; l; l = g_slist_next (l)) {
 			AudioscrobblerEntry *entry;
-		
+
 			entry = (AudioscrobblerEntry *) l->data;
 			g_string_printf (s, "a=%s&t=%s&b=%s&m=%s&l=%d&i=%s\n",
 					    entry->artist,
