@@ -409,7 +409,7 @@ gboolean
 rb_podcast_parse_load_feed (RBPodcastChannel *data,
 			    const char *file_name)
 {
-	xmlParserCtxtPtr ctxt;
+	xmlParserCtxtPtr parser;
 	xmlSAXHandlerPtr sax_handler = NULL;
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
@@ -429,6 +429,7 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 
 		if (info != NULL
 		    && info->mime_type != NULL
+		    && strstr (info->mime_type, "html") == NULL
 		    && strstr (info->mime_type, "xml") == NULL
 		    && strstr (info->mime_type, "rss") == NULL) {
 			invalid_mime_type = TRUE;
@@ -484,8 +485,8 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 	ctx->channel_data = data;
 	ctx->prop_value = g_string_sized_new (512);
 
-	ctxt = xmlCreateMemoryParserCtxt (buffer, file_size);
-	if (ctx == NULL) {
+	parser = xmlCreateMemoryParserCtxt (buffer, file_size);
+	if (parser == NULL) {
 		g_free (sax_handler);
 		g_free (buffer);
 		g_string_free (ctx->prop_value, TRUE);
@@ -493,14 +494,14 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 		return FALSE;
 	}
 
-	ctx->xmlctx = ctxt;
-	ctxt->userData = ctx;
-	ctxt->sax = sax_handler;
-	xmlParseDocument (ctxt);
+	ctx->xmlctx = parser;
+	parser->userData = ctx;
+	parser->sax = sax_handler;
+	xmlParseDocument (parser);
 
 	g_free (sax_handler);
-	ctxt->sax = NULL;
-	xmlFreeParserCtxt (ctxt);
+	parser->sax = NULL;
+	xmlFreeParserCtxt (parser);
 
 	g_free (buffer);
 	g_string_free (ctx->prop_value, TRUE);
@@ -531,6 +532,12 @@ rb_podcast_parse_date (const char *date_str)
 		result = strptime (date_str, "%a, %d %b %Y 0%T", &tm);
 	}
 
+	/* close-to-RFC 2822, but with no seconds */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a, %d %b %Y %R", &tm);
+	}
+
 	/* format without weekday */
 	if (result == NULL) {
 		memset (&tm, 0, sizeof (struct tm));
@@ -601,11 +608,10 @@ rb_podcast_parse_date (const char *date_str)
 	}
 
 	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
 		rb_debug ("unable to convert date string %s", date_str);
 	}
 
-	return (uintmax_t) mktime (&tm);
+	return (uintmax_t) ( (result==NULL) ? 0 : mktime (&tm) );
 }
 
 static gulong
