@@ -30,7 +30,6 @@
 GHashTable *rb_refstrings;
 GMutex *rb_refstrings_mutex;
 
-
 struct RBRefString
 {
 	gint refcount;
@@ -85,15 +84,35 @@ rb_refstring_new (const char *init)
 	return ret;
 }
 
+RBRefString *
+rb_refstring_find (const char *init)
+{
+	RBRefString *ret;
+
+	g_mutex_lock (rb_refstrings_mutex);
+	ret = g_hash_table_lookup (rb_refstrings, init);
+
+	if (ret)
+		rb_refstring_ref (ret);
+
+	g_mutex_unlock (rb_refstrings_mutex);
+	return ret;
+}
+
 void
 rb_refstring_unref (RBRefString *val)
 {
-	if (!val)
+	if (val == NULL)
 		return;
 
+	g_return_if_fail (val->refcount > 0);
+
 	if (g_atomic_int_dec_and_test (&val->refcount)) {
 		g_mutex_lock (rb_refstrings_mutex);
-		g_hash_table_remove (rb_refstrings, val->value);
+		/* ensure it's still not referenced, as something may have called
+		 * rb_refstring_new since we decremented the count */
+		if (g_atomic_int_get (&val->refcount) == 0)
+			g_hash_table_remove (rb_refstrings, val->value);
 		g_mutex_unlock (rb_refstrings_mutex);
 	}
 }
@@ -104,11 +123,15 @@ rb_refstring_system_shutdown (void)
 	g_hash_table_destroy (rb_refstrings);
 	g_mutex_free (rb_refstrings_mutex);
 }
-	
 
 RBRefString *
 rb_refstring_ref (RBRefString *val)
 {
+	if (val == NULL)
+		return NULL;
+
+	g_return_val_if_fail (val->refcount > 0, NULL);
+
 	g_atomic_int_inc (&val->refcount);
 	return val;
 }
@@ -119,7 +142,6 @@ rb_refstring_get (const RBRefString *val)
 	return val ? val->value : NULL;
 }
 
-
 /*
  * The next two functions will compute the values if they haven't
  * been already done. Using g_atomic_* is much more efficient than
@@ -164,11 +186,14 @@ rb_refstring_get_sort_key (RBRefString *val)
 		return NULL;
 
 	ptr = &val->sortkey;
-	string = (const char*)g_atomic_pointer_get (ptr);
+	string = (const char *)g_atomic_pointer_get (ptr);
 	if (string == NULL) {
 		char *newstring;
+		const char *s;
+
+		s = rb_refstring_get_folded (val);
+		newstring = rb_utf8_collate_key_for_filename (s, -1);
 
-		newstring = rb_utf8_collate_key_for_filename (rb_refstring_get_folded (val), -1);
 		if (g_atomic_pointer_compare_and_exchange (ptr, NULL, newstring)) {
 			string = newstring;
 		} else {
