@@ -57,6 +57,9 @@ typedef struct {
 enum {
 	RHYTHMDB_ENTRY_HIDDEN = 1,
 	RHYTHMDB_ENTRY_INSERTED = 2,
+	RHYTHMDB_ENTRY_LAST_PLAYED_DIRTY = 4,
+	RHYTHMDB_ENTRY_FIRST_SEEN_DIRTY = 8,
+	RHYTHMDB_ENTRY_LAST_SEEN_DIRTY = 16,
 
 	/* the backend can use the top 16 bits for private flags */
 	RHYTHMDB_ENTRY_PRIVATE_FLAG_BASE = 65536,
@@ -65,15 +68,21 @@ enum {
 struct RhythmDBEntry_ {
 	/* internal bits */
 	guint flags;
+#if (GLIB_MAJOR_VERSION >= 2 && GLIB_MINOR_VERSION >= 10)
+	volatile gint refcount;
+#else
 	gint refcount;
+#endif
 	void *data;
 	RhythmDBEntryType type;
-	
+	guint id;
+
 	/* metadata */
 	RBRefString *title;
 	RBRefString *artist;
 	RBRefString *album;
 	RBRefString *genre;
+	RBRefString *musicbrainz_trackid;
 	gulong tracknum;
 	gulong discnum;
 	gulong duration;
@@ -85,7 +94,7 @@ struct RhythmDBEntry_ {
 	GDate date;
 
 	/* filesystem */
-	char *location;
+	RBRefString *location;
 	RBRefString *mountpoint;
 	guint64 file_size;
 	RBRefString *mimetype;
@@ -99,12 +108,12 @@ struct RhythmDBEntry_ {
 	gulong last_played;
 
 	/* cached data */
-	RBRefString *last_played_str;
-	RBRefString *first_seen_str;
-	RBRefString *last_seen_str;
+	gpointer last_played_str;
+	gpointer first_seen_str;
+	gpointer last_seen_str;
 
 	/* playback error string */
-	char *playback_error;
+	RBRefString *playback_error;
 };
 
 struct RhythmDBPrivate
@@ -125,10 +134,13 @@ struct RhythmDBPrivate
 	GAsyncQueue *action_queue;
 	GAsyncQueue *event_queue;
 	GAsyncQueue *restored_queue;
+	GThreadPool *query_thread_pool;
+	GThreadPool *add_thread_pool;
 
 	GList *stat_list;
 	GHashTable *stat_events;
 	GnomeVFSAsyncHandle *stat_handle;
+	GList *outstanding_stats;
 	GMutex *stat_mutex;
 
 	GHashTable *monitored_directories;
@@ -149,7 +161,7 @@ struct RhythmDBPrivate
 
 	GMutex *exit_mutex;
 	gboolean exiting;
-	
+
 	GCond *saving_condition;
 	GMutex *saving_mutex;
 
@@ -161,8 +173,15 @@ struct RhythmDBPrivate
 	GList *added_entries_to_emit;
 	GList *deleted_entries_to_emit;
 
+	gboolean can_save;
 	gboolean saving;
 	gboolean dirty;
+
+	GHashTable *entry_type_map;
+	GMutex *entry_type_map_mutex;
+	GMutex *entry_type_mutex;
+
+	gint next_entry_id;
 };
 
 typedef struct
@@ -178,8 +197,8 @@ typedef struct
 		RHYTHMDB_EVENT_FILE_DELETED,
 		RHYTHMDB_EVENT_ENTRY_SET
 	} type;
-	char *uri;
-	char *real_uri; /* Target of a symlink, if any */
+	RBRefString *uri;
+	RBRefString *real_uri; /* Target of a symlink, if any */
 	RhythmDBEntryType entry_type;
 
 	GError *error;
@@ -201,11 +220,13 @@ typedef struct
 
 /* from rhythmdb.c */
 void queue_stat_uri (const char *uri, RhythmDB *db, RhythmDBEntryType type);
-void rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry, 
+void rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry,
 				    gboolean visibility);
-void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry, 
-				  gboolean notify_if_inserted, guint propid, 
+void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
+				  gboolean notify_if_inserted, guint propid,
 				  const GValue *value);
+void rhythmdb_entry_type_foreach (RhythmDB *db, GHFunc func, gpointer data);
+RhythmDBEntry *	rhythmdb_entry_lookup_by_location_refstring (RhythmDB *db, RBRefString *uri);
 
 /* from rhythmdb-monitor.c */
 void rhythmdb_init_monitoring (RhythmDB *db);
