@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Implementation of RhythmDB property GtkTreeModel
  *
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
@@ -20,7 +20,8 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,7 +48,6 @@ typedef struct {
 	guint refcount;
 } RhythmDBPropertyModelEntry;
 
-
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -113,27 +113,31 @@ enum {
 	TARGET_GENRE,
 	TARGET_ARTISTS,
 	TARGET_LOCATION,
+	TARGET_ENTRIES,
 	TARGET_URIS,
 };
 
 static const GtkTargetEntry targets_album  [] = {
 	{ "text/x-rhythmbox-album",  0, TARGET_ALBUMS },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 static const GtkTargetEntry targets_genre  [] = {
 	{ "text/x-rhythmbox-genre",  0, TARGET_GENRE },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 static const GtkTargetEntry targets_artist [] = {
 	{ "text/x-rhythmbox-artist", 0, TARGET_ARTISTS },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 static const GtkTargetEntry targets_location [] = {
 	{ "text/x-rhythmbox-location", 0, TARGET_LOCATION },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 
-
 static GtkTargetList *rhythmdb_property_model_album_drag_target_list = NULL;
 static GtkTargetList *rhythmdb_property_model_artist_drag_target_list = NULL;
 static GtkTargetList *rhythmdb_property_model_genre_drag_target_list = NULL;
@@ -151,14 +155,11 @@ struct RhythmDBPropertyModelPrivate
 
 	guint stamp;
 
-	GPtrArray *query;
-
 	GSequence *properties;
 	GHashTable *reverse_map;
 
 	RhythmDBPropertyModelEntry *all;
-	
-	gboolean complete;
+
 	guint syncing_id;
 };
 
@@ -264,18 +265,24 @@ rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface)
 static void
 rhythmdb_property_model_drag_source_init (RbTreeDragSourceIface *iface)
 {
-	iface->row_draggable = rhythmdb_property_model_row_draggable;
-	iface->drag_data_delete = rhythmdb_property_model_drag_data_delete;
-	iface->drag_data_get = rhythmdb_property_model_drag_data_get;
+	iface->rb_row_draggable = rhythmdb_property_model_row_draggable;
+	iface->rb_drag_data_delete = rhythmdb_property_model_drag_data_delete;
+	iface->rb_drag_data_get = rhythmdb_property_model_drag_data_get;
 }
 
 static gboolean
-_remove_entry_cb (RhythmDBEntry *entry,
-		  gpointer unused,
-		  RhythmDBPropertyModel *model)
+_remove_entry_cb (GtkTreeModel *model,
+		  GtkTreePath *path,
+		  GtkTreeIter *iter,
+		  RhythmDBPropertyModel *propmodel)
 {
-	rhythmdb_property_model_delete (model, entry);
-	return TRUE;
+	RhythmDBEntry *entry;
+
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
+	rhythmdb_property_model_entry_removed_cb (RHYTHMDB_QUERY_MODEL (model),
+						  entry,
+						  propmodel);
+	return FALSE;
 }
 
 static gboolean
@@ -288,6 +295,55 @@ _add_entry_cb (GtkTreeModel *model,
 	return FALSE;
 }
 
+static void
+rhythmdb_property_model_set_query_model_internal (RhythmDBPropertyModel *model,
+						  RhythmDBQueryModel    *query_model)
+{
+	if (model->priv->query_model != NULL) {
+		g_signal_handlers_disconnect_by_func (model->priv->query_model,
+						      G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
+						      model);
+		g_signal_handlers_disconnect_by_func (model->priv->query_model,
+						      G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
+						      model);
+		g_signal_handlers_disconnect_by_func (model->priv->query_model,
+						      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
+						      model);
+
+		gtk_tree_model_foreach (GTK_TREE_MODEL (model->priv->query_model),
+					(GtkTreeModelForeachFunc)_remove_entry_cb,
+					model);
+
+		g_object_unref (model->priv->query_model);
+	}
+
+	model->priv->query_model = query_model;
+	g_assert (rhythmdb_property_model_iter_n_children (GTK_TREE_MODEL (model), NULL) == 1);
+
+	if (model->priv->query_model != NULL) {
+		g_object_ref (model->priv->query_model);
+
+		g_signal_connect_object (model->priv->query_model,
+					 "row_inserted",
+					 G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
+					 model,
+					 0);
+		g_signal_connect_object (model->priv->query_model,
+					 "post-entry-delete",
+					 G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
+					 model,
+					 0);
+		g_signal_connect_object (model->priv->query_model,
+					 "entry-prop-changed",
+					 G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
+					 model,
+					 0);
+		gtk_tree_model_foreach (GTK_TREE_MODEL (model->priv->query_model),
+					(GtkTreeModelForeachFunc)_add_entry_cb,
+					model);
+	}
+}
+
 static void
 rhythmdb_property_model_set_property (GObject *object,
 				      guint prop_id,
@@ -298,10 +354,8 @@ rhythmdb_property_model_set_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_RHYTHMDB:
-	{
 		model->priv->db = g_value_get_object (value);
 		break;
-	}
 	case PROP_PROP:
 		model->priv->propid = g_value_get_int (value);
 		switch (model->priv->propid) {
@@ -327,58 +381,19 @@ rhythmdb_property_model_set_property (GObject *object,
 		}
 		break;
 	case PROP_QUERY_MODEL:
-	{
-		if (model->priv->query_model) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
-							      model);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
-							      model);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
-							      model);
-			g_hash_table_foreach_remove (model->priv->entries, (GHRFunc)_remove_entry_cb, model);
-			g_object_unref (G_OBJECT (model->priv->query_model));
-		}
-		
-		model->priv->query_model = g_value_get_object (value);
-		g_assert (rhythmdb_property_model_iter_n_children (GTK_TREE_MODEL (model), NULL) == 1);
-
-		if (model->priv->query_model) {
-			g_object_ref (G_OBJECT (model->priv->query_model));
-			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "row_inserted",
-						 G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
-						 model,
-						 0);
-			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "entry-removed",
-						 G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
-						 model,
-						 0);
-			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "entry-prop-changed",
-						 G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
-						 model,
-						 0);
-			gtk_tree_model_foreach (GTK_TREE_MODEL (model->priv->query_model),
-						(GtkTreeModelForeachFunc)_add_entry_cb,
-						model);
-		}
+		rhythmdb_property_model_set_query_model_internal (model, g_value_get_object (value));
 		break;
-	}
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static void 
+static void
 rhythmdb_property_model_get_property (GObject *object,
-				   guint prop_id,
-				   GValue *value,
-				   GParamSpec *pspec)
+				      guint prop_id,
+				      GValue *value,
+				      GParamSpec *pspec)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (object);
 
@@ -432,26 +447,30 @@ rhythmdb_property_model_finalize (GObject *object)
 	if (model->priv->syncing_id != 0)
 		g_source_remove (model->priv->syncing_id);
 
+	g_hash_table_destroy (model->priv->reverse_map);
+
 	end_ptr = g_sequence_get_end_ptr (model->priv->properties);
 	for (ptr = g_sequence_get_begin_ptr (model->priv->properties); ptr != end_ptr;
 	     ptr = g_sequence_ptr_next (ptr)) {
 		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (ptr);
 		rb_refstring_unref (prop->string);
 		rb_refstring_unref (prop->sort_string);
+		g_free (prop);
 	}
-
-	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->properties);
+
 	g_hash_table_destroy (model->priv->entries);
 
-	if (model->priv->query_model)
-		g_object_unref (G_OBJECT (model->priv->query_model));
+	if (model->priv->query_model != NULL) {
+		g_object_unref (model->priv->query_model);
+	}
 
 	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
 
 RhythmDBPropertyModel *
-rhythmdb_property_model_new (RhythmDB *db, RhythmDBPropType propid)
+rhythmdb_property_model_new (RhythmDB *db,
+			     RhythmDBPropType propid)
 {
 	return g_object_new (RHYTHMDB_TYPE_PROPERTY_MODEL, "db", db, "prop", propid, NULL);
 }
@@ -462,22 +481,22 @@ rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 					 GtkTreeIter *iter,
 					 RhythmDBPropertyModel *propmodel)
 {
-	RhythmDBEntry *entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
+	RhythmDBEntry *entry;
 	RhythmDBPropertyModelEntry *prop;
 
-	if (g_hash_table_lookup (propmodel->priv->entries, entry))
-		return;
-
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
 
 	prop = rhythmdb_property_model_insert (propmodel, entry);
-	g_hash_table_insert (propmodel->priv->entries, entry, prop);
-
 	rhythmdb_property_model_sync (propmodel);
+
+	rhythmdb_entry_unref (entry);
 }
 
 static void
-rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-					 RhythmDBPropType propid, const GValue *old,
+rhythmdb_property_model_prop_changed_cb (RhythmDB *db,
+					 RhythmDBEntry *entry,
+					 RhythmDBPropType propid,
+					 const GValue *old,
 					 const GValue *new,
 					 RhythmDBPropertyModel *propmodel)
 {
@@ -487,33 +506,26 @@ rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 
 		if (old_val != new_val) {
 			if (new_val == FALSE) {
-				RhythmDBPropertyModelEntry *prop;
-
-				if (g_hash_table_lookup (propmodel->priv->entries, entry) != NULL)
-					return;
-
-				prop = rhythmdb_property_model_insert (propmodel, entry);
-				g_hash_table_insert (propmodel->priv->entries, entry, prop);
+				g_assert (g_hash_table_remove (propmodel->priv->entries, entry));
+				rhythmdb_property_model_insert (propmodel, entry);
 			} else {
-				if (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL)
-					return;
+				g_assert (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL);
 
 				rhythmdb_property_model_delete (propmodel, entry);
-				g_hash_table_remove (propmodel->priv->entries, entry);
+				g_hash_table_insert (propmodel->priv->entries, entry, GINT_TO_POINTER (1));
 			}
 		}
 	} else {
 		RhythmDBPropertyModelEntry *prop;
-		
+
 		if (propid != propmodel->priv->propid)
 			return;
 
-		if (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL)
+		if (g_hash_table_lookup (propmodel->priv->entries, entry) != NULL)
 			return;
 
-		rhythmdb_property_model_delete (propmodel, entry);
+		rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
 		prop = rhythmdb_property_model_insert (propmodel, entry);
-		g_hash_table_insert (propmodel->priv->entries, entry, prop);
 	}
 
 	rhythmdb_property_model_sync (propmodel);
@@ -524,27 +536,29 @@ rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
 					  RhythmDBEntry *entry,
 					  RhythmDBPropertyModel *propmodel)
 {
-	if (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL)
+	if (g_hash_table_remove (propmodel->priv->entries, entry))
 		return;
 
 	rhythmdb_property_model_delete (propmodel, entry);
-	g_hash_table_remove (propmodel->priv->entries, entry);
 	rhythmdb_property_model_sync (propmodel);
 }
 
 static gint
-rhythmdb_property_model_compare (RhythmDBPropertyModelEntry *a, RhythmDBPropertyModelEntry *b, RhythmDBPropertyModel *model)
+rhythmdb_property_model_compare (RhythmDBPropertyModelEntry *a,
+				 RhythmDBPropertyModelEntry *b,
+				 RhythmDBPropertyModel *model)
 {
 	const char *a_str, *b_str;
 
 	a_str = rb_refstring_get_sort_key (a->sort_string);
 	b_str = rb_refstring_get_sort_key (b->sort_string);
-	
+
 	return strcmp (a_str, b_str);
 }
 
 static RhythmDBPropertyModelEntry *
-rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry)
+rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
+				RhythmDBEntry *entry)
 {
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreeIter iter;
@@ -589,14 +603,15 @@ static void
 rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 				RhythmDBEntry *entry)
 {
-	RhythmDBPropertyModelEntry *prop;
 	const char *propstr;
 
-	prop = g_hash_table_lookup (model->priv->entries, entry);
-	propstr = rb_refstring_get (prop->string);
+	if (g_hash_table_lookup (model->priv->entries, entry))
+		return;
+
+	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
 	rhythmdb_property_model_delete_prop (model, propstr);
 }
-	
+
 static void
 rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 				     const char *propstr)
@@ -633,7 +648,8 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 
 gboolean
 rhythmdb_property_model_iter_from_string (RhythmDBPropertyModel *model,
-					  const char *name, GtkTreeIter *iter)
+					  const char *name,
+					  GtkTreeIter *iter)
 {
 	GSequencePtr ptr;
 
@@ -670,7 +686,8 @@ rhythmdb_property_model_get_n_columns (GtkTreeModel *tree_model)
 }
 
 static GType
-rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index)
+rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model,
+					 int index)
 {
 	switch (index) {
 	case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
@@ -686,7 +703,8 @@ rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index)
 }
 
 static gboolean
-rhythmdb_property_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
+rhythmdb_property_model_get_iter (GtkTreeModel *tree_model,
+				  GtkTreeIter *iter,
 				  GtkTreePath *path)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
@@ -722,6 +740,10 @@ rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
 
 	g_return_val_if_fail (iter->stamp == model->priv->stamp, NULL);
 
+	if (iter->user_data == model->priv->all) {
+		return gtk_tree_path_new_first ();
+	}
+
 	if (g_sequence_ptr_is_end (iter->user_data))
 		return NULL;
 
@@ -729,17 +751,18 @@ rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
 	if (iter->user_data == model->priv->all)
 		gtk_tree_path_append_index (path, 0);
 	else
-		gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data)+1);
+		gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data) + 1);
 	return path;
 }
 
 static void
-rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
-				   gint column, GValue *value)
+rhythmdb_property_model_get_value (GtkTreeModel *tree_model,
+				   GtkTreeIter *iter,
+				   gint column,
+				   GValue *value)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 
-	g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
 	g_return_if_fail (model->priv->stamp == iter->stamp);
 
 	if (iter->user_data == model->priv->all) {
@@ -760,8 +783,11 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 			g_assert_not_reached ();
 		}
 	} else {
-		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (iter->user_data);
-		
+		RhythmDBPropertyModelEntry *prop;
+
+		g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
+
+		prop = g_sequence_ptr_get_data (iter->user_data);
 		switch (column) {
 		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
@@ -789,10 +815,12 @@ rhythmdb_property_model_iter_next (GtkTreeModel  *tree_model,
 
 	g_return_val_if_fail (iter->stamp == model->priv->stamp, FALSE);
 
-	if (iter->user_data == model->priv->all)
+	if (iter->user_data == model->priv->all) {
 		iter->user_data = g_sequence_get_begin_ptr (model->priv->properties);
-	else
+	} else {
+		g_return_val_if_fail (!g_sequence_ptr_is_end (iter->user_data), FALSE);
 		iter->user_data = g_sequence_ptr_next (iter->user_data);
+	}
 
 	return !g_sequence_ptr_is_end (iter->user_data);
 }
@@ -837,7 +865,8 @@ rhythmdb_property_model_iter_n_children (GtkTreeModel *tree_model,
 
 static gboolean
 rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
-					GtkTreeIter *iter, GtkTreeIter *parent,
+					GtkTreeIter *iter,
+					GtkTreeIter *parent,
 					gint n)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
@@ -848,7 +877,7 @@ rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
 
 	if (n != 0) {
 		child = g_sequence_get_ptr_at_pos (model->priv->properties, n);
-		
+
 		if (g_sequence_ptr_is_end (child))
 			return FALSE;
 		iter->user_data = child;
@@ -877,7 +906,7 @@ rhythmdb_property_model_row_draggable (RbTreeDragSource *dragsource,
 }
 
 static gboolean
-rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource, 
+rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource,
 					  GList *paths)
 {
 	/* not supported */
@@ -885,25 +914,33 @@ rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource,
 }
 
 /*Going through hoops to avoid nested functions*/
-struct QueryModelCbStruct { 
-	RhythmDB* db; 
-	GString* reply; 
-}; 
+struct QueryModelCbStruct {
+	RhythmDB *db;
+	GString *reply;
+	gint target;
+};
 
-static gboolean 
+static gboolean
 query_model_cb (GtkTreeModel *query_model,
- 		GtkTreePath *path, GtkTreeIter *iter,
- 		struct QueryModelCbStruct *data) 
+ 		GtkTreePath *path,
+		GtkTreeIter *iter,
+ 		struct QueryModelCbStruct *data)
 {
- 	const char* uri;
- 	RhythmDBEntry* entry;
-	
+ 	RhythmDBEntry *entry;
+
  	gtk_tree_model_get (query_model, iter, 0, &entry, -1);
- 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
- 	g_string_append (data->reply, uri);
+	if (data->target == TARGET_ENTRIES) {
+		g_string_append_printf (data->reply,
+					"%ld",
+					rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_ENTRY_ID));
+	} else if (data->target == TARGET_URIS) {
+		const char *uri;
+		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+		g_string_append (data->reply, uri);
+	}
  	g_string_append (data->reply, "\r\n");
-	
- 	rhythmdb_entry_unref (data->db, entry);
+
+ 	rhythmdb_entry_unref (entry);
  	return FALSE;
 }
 
@@ -934,17 +971,17 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 	}
 
 	if (!gtk_target_list_find (drag_target_list,
-				   selection_data->target, 
+				   selection_data->target,
 				   &target)) {
 		return FALSE;
 	}
 
-	if (target == TARGET_URIS) {
+	if (target == TARGET_URIS || target == TARGET_ENTRIES) {
 		RhythmDB *db = model->priv->db;
  		RhythmDBQueryModel *query_model;
  		GString* reply = g_string_new ("");
  		GtkTreeIter iter;
- 		gboolean is_all;
+ 		gboolean is_all = FALSE;
  		struct QueryModelCbStruct tmp;
 		GtkTreePath *path;
 		GCompareDataFunc sort_func = NULL;
@@ -966,68 +1003,66 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 		rb_debug ("getting drag data as uri list");
 		/* check if first selected row is 'All' */
 		path = gtk_tree_row_reference_get_path (paths->data);
-		gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
-		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-				    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY,
-				    &is_all, -1);
+		if (path && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path))
+			gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY,
+					    &is_all, -1);
 		gtk_tree_path_free (path);
 		if (is_all) {
-			g_object_set (G_OBJECT (query_model), 
-				      "base-model", model->priv->query_model, 
+			g_object_set (query_model,
+				      "base-model", model->priv->query_model,
 				      NULL);
 		} else {
  			GList *row;
-			GPtrArray *query;
 			GPtrArray *subquery = g_ptr_array_new ();
- 
+
  			for (row = paths; row; row = row->next) {
  				char* name;
 				path = gtk_tree_row_reference_get_path (row->data);
- 				gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
- 				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
- 						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, 
-						    &name, -1);
- 				if (row == paths) {
- 					rhythmdb_query_append (db, subquery,
- 							       RHYTHMDB_QUERY_PROP_EQUALS,
- 							       model->priv->propid, name,
-							       RHYTHMDB_QUERY_END);
-				} else {
- 					rhythmdb_query_append (db, subquery,
- 							       RHYTHMDB_QUERY_DISJUNCTION,
- 							       RHYTHMDB_QUERY_PROP_EQUALS,
- 							       model->priv->propid, name,
- 							       RHYTHMDB_QUERY_END);
- 				}
- 
+ 				if (path && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path)) {
+	 				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+	 						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE,
+							    &name, -1);
+	 				if (row == paths) {
+	 					rhythmdb_query_append (db, subquery,
+	 							       RHYTHMDB_QUERY_PROP_EQUALS,
+	 							       model->priv->propid, name,
+								       RHYTHMDB_QUERY_END);
+					} else {
+	 					rhythmdb_query_append (db, subquery,
+	 							       RHYTHMDB_QUERY_DISJUNCTION,
+	 							       RHYTHMDB_QUERY_PROP_EQUALS,
+	 							       model->priv->propid, name,
+	 							       RHYTHMDB_QUERY_END);
+	 				}
+				}
+
 				gtk_tree_path_free (path);
  				g_free (name);
  			}
-			query = rhythmdb_query_parse (db,
-						      RHYTHMDB_QUERY_SUBQUERY, subquery,
-						      RHYTHMDB_QUERY_END);
 
-			g_object_set (G_OBJECT (query_model), 
-				      "query", query,
-				      "base-model", model->priv->query_model, 
+			g_object_set (query_model,
+				      "query", subquery,
+				      "base-model", model->priv->query_model,
 				      NULL);
-			rhythmdb_query_free (query);
+			rhythmdb_query_free (subquery);
 		}
- 		
-		tmp.db = db; 
+
+		tmp.db = db;
  		tmp.reply = reply;
+		tmp.target = target;
  		/* Too bad that we're on the main thread. Why doesn't gtk call us async?
- 		 * How does file-roller manage? - it seems it refuses the drop when it isn't 
-		 * done unpacking. In which case, we should tweak the drop acknowledgement, 
-		 * and prepare the query using do_full_query_async. The query would be 
+ 		 * How does file-roller manage? - it seems it refuses the drop when it isn't
+		 * done unpacking. In which case, we should tweak the drop acknowledgement,
+		 * and prepare the query using do_full_query_async. The query would be
 		 * hooked to the drag context.
 		 */
  		gtk_tree_model_foreach (GTK_TREE_MODEL (query_model),
  					(GtkTreeModelForeachFunc) query_model_cb,
  					&tmp);
- 		
+
 		g_object_unref (query_model);
- 
+
  		gtk_selection_data_set (selection_data,
  		                        selection_data->target,
  		                        8, (guchar *)reply->str,
@@ -1038,7 +1073,7 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 		char* title;
 		GList *p;
 		GString* reply = g_string_new ("");
- 
+
 		rb_debug ("getting drag data as list of property values");
 
 		for (p = paths; p; p = p->next) {
@@ -1046,13 +1081,14 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 			GtkTreePath *path;
 
 			path = gtk_tree_row_reference_get_path (p->data);
-			gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
-			gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-					    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title, -1);
-			g_string_append (reply, title);
-			if (p->next)
-				g_string_append (reply, "\r\n");
-			g_free (title);
+			if (path && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path)) {
+				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title, -1);
+				g_string_append (reply, title);
+				if (p->next)
+					g_string_append (reply, "\r\n");
+				g_free (title);
+			}
 			gtk_tree_path_free (path);
 		}
 		gtk_selection_data_set (selection_data,
@@ -1065,8 +1101,9 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 	return TRUE;
 }
 
-void			
-rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model, GtkTreeView *view)
+void
+rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model,
+				     GtkTreeView *view)
 {
 	const GtkTargetEntry *targets;
 	gint n_elements;
@@ -1086,12 +1123,12 @@ rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model, GtkTreeView *
 		break;
 	case RHYTHMDB_PROP_LOCATION:
 		targets = targets_location;
-		n_elements = G_N_ELEMENTS (targets_location);		
+		n_elements = G_N_ELEMENTS (targets_location);
 		break;
 	default:
 		g_assert_not_reached ();
 	}
-	
+
 	rb_tree_dnd_add_drag_source_support (view,
 					     GDK_BUTTON1_MASK,
 					     targets, n_elements,
@@ -1104,6 +1141,8 @@ rhythmdb_property_model_perform_sync (RhythmDBPropertyModel *model)
 	GtkTreeIter iter;
 	GtkTreePath *path;
 
+	GDK_THREADS_ENTER ();
+
 	iter.stamp = model->priv->stamp;
 	iter.user_data = model->priv->all;
 	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
@@ -1111,6 +1150,7 @@ rhythmdb_property_model_perform_sync (RhythmDBPropertyModel *model)
 	gtk_tree_path_free (path);
 
 	model->priv->syncing_id = 0;
+	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
@@ -1144,4 +1184,3 @@ rhythmdb_property_model_column_get_type (void)
 
 	return etype;
 }
-
