@@ -155,6 +155,12 @@ static void rhythmdb_query_model_base_rows_reordered (GtkTreeModel *base_model,
 static void rhythmdb_query_model_base_entry_removed (RhythmDBQueryModel *base_model,
 						     RhythmDBEntry *entry,
 						     RhythmDBQueryModel *model);
+static void rhythmdb_query_model_base_entry_prop_changed (RhythmDBQueryModel *base_model,
+							  RhythmDBEntry *entry,
+							  RhythmDBPropType prop,
+							  const GValue *old,
+							  const GValue *new_value,
+							  RhythmDBQueryModel *model);
 static int rhythmdb_query_model_child_index_to_base_index (RhythmDBQueryModel *model, int index);
 
 static gint _reverse_sorting_func (gpointer a, gpointer b, struct ReverseSortData *model);
@@ -184,7 +190,15 @@ static void rhythmdb_query_model_process_update (struct RhythmDBQueryModelUpdate
 
 static void idle_process_update (struct RhythmDBQueryModelUpdate *update);
 
-static const GtkTargetEntry rhythmdb_query_model_drag_types[] = { { "text/uri-list", 0, 0 },};
+enum {
+	TARGET_ENTRIES,
+	TARGET_URIS
+};
+
+static const GtkTargetEntry rhythmdb_query_model_drag_types[] = {
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
+	{ "text/uri-list", 0, TARGET_URIS },
+};
 
 static GtkTargetList *rhythmdb_query_model_drag_target_list = NULL;
 
@@ -214,6 +228,7 @@ struct RhythmDBQueryModelPrivate
 	GHashTable *reverse_map;
 	GSequence *limited_entries;
 	GHashTable *limited_reverse_map;
+	GHashTable *hidden_entry_map;
 
 	gint pending_update_count;
 
@@ -412,17 +427,17 @@ rhythmdb_query_model_tree_model_init (GtkTreeModelIface *iface)
 static void
 rhythmdb_query_model_drag_source_init (RbTreeDragSourceIface *iface)
 {
-	iface->row_draggable = rhythmdb_query_model_row_draggable;
-	iface->drag_data_delete = rhythmdb_query_model_drag_data_delete;
-	iface->drag_data_get = rhythmdb_query_model_drag_data_get;
+	iface->rb_row_draggable = rhythmdb_query_model_row_draggable;
+	iface->rb_drag_data_delete = rhythmdb_query_model_drag_data_delete;
+	iface->rb_drag_data_get = rhythmdb_query_model_drag_data_get;
 }
 
 static void
 rhythmdb_query_model_drag_dest_init (RbTreeDragDestIface *iface)
 {
-	iface->drag_data_received = rhythmdb_query_model_drag_data_received;
-	iface->row_drop_possible = rhythmdb_query_model_row_drop_possible;
-	iface->row_drop_position = rhythmdb_query_model_row_drop_position;
+	iface->rb_drag_data_received = rhythmdb_query_model_drag_data_received;
+	iface->rb_row_drop_possible = rhythmdb_query_model_row_drop_possible;
+	iface->rb_row_drop_position = rhythmdb_query_model_row_drop_position;
 }
 
 static void
@@ -569,6 +584,11 @@ rhythmdb_query_model_init (RhythmDBQueryModel *model)
 								  (GDestroyNotify)rhythmdb_entry_unref,
 								  NULL);
 
+	model->priv->hidden_entry_map = g_hash_table_new_full (g_direct_hash,
+							       g_direct_equal,
+							       (GDestroyNotify)rhythmdb_entry_unref,
+							       NULL);
+
 	model->priv->reorder_drag_and_drop = FALSE;
 }
 
@@ -631,6 +651,9 @@ rhythmdb_query_model_dispose (GObject *object)
 		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
 						      G_CALLBACK (rhythmdb_query_model_base_entry_removed),
 						      model);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
+						      G_CALLBACK (rhythmdb_query_model_base_entry_prop_changed),
+						      model);
 		g_object_unref (model->priv->base_model);
 		model->priv->base_model = NULL;
 	}
@@ -753,6 +776,9 @@ rhythmdb_query_model_chain (RhythmDBQueryModel *model,
 		g_signal_handlers_disconnect_by_func (model->priv->base_model,
 						      G_CALLBACK (rhythmdb_query_model_base_entry_removed),
 						      model);
+		g_signal_handlers_disconnect_by_func (model->priv->base_model,
+						      G_CALLBACK (rhythmdb_query_model_base_entry_prop_changed),
+						      model);
 		g_object_unref (model->priv->base_model);
 	}
 
@@ -787,6 +813,10 @@ rhythmdb_query_model_chain (RhythmDBQueryModel *model,
 					 "entry-removed",
 					 G_CALLBACK (rhythmdb_query_model_base_entry_removed),
 					 model, 0);
+		g_signal_connect_object (model->priv->base_model,
+					 "entry-prop-changed",
+					 G_CALLBACK (rhythmdb_query_model_base_entry_prop_changed),
+					 model, 0);
 
 		if (import_entries)
 			rhythmdb_query_model_copy_contents (model, model->priv->base_model);
@@ -810,18 +840,30 @@ rhythmdb_query_model_entry_added_cb (RhythmDB *db,
 				     RhythmDBEntry *entry,
 				     RhythmDBQueryModel *model)
 {
-	if (model->priv->query) {
-		if (!model->priv->show_hidden && rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN))
-			return;
+	int index = -1;
+	gboolean insert = FALSE;
+	if (!model->priv->show_hidden && rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN)) {
+		return;
+	}
 
-		/* check if it's in the base model */
-		if (model->priv->base_model)
-		       if (g_hash_table_lookup (model->priv->base_model->priv->reverse_map, entry) == NULL)
-			       return;
+	/* check if it's in the base model */
+	if (model->priv->base_model) {
+	       if (g_hash_table_lookup (model->priv->base_model->priv->reverse_map, entry) == NULL) {
+		       return;
+	       }
+	}
 
-		if (rhythmdb_evaluate_query (db, model->priv->query, entry)) {
-			rhythmdb_query_model_do_insert (model, entry, -1);
-		}
+	if (model->priv->query != NULL) {
+		insert = rhythmdb_evaluate_query (db, model->priv->query, entry);
+	} else {
+		index = GPOINTER_TO_INT (g_hash_table_lookup (model->priv->hidden_entry_map, entry));
+		insert = g_hash_table_remove (model->priv->hidden_entry_map, entry);
+		if (insert)
+			rb_debug ("adding unhidden entry at index %d", index);
+	}
+
+	if (insert) {
+		rhythmdb_query_model_do_insert (model, entry, index);
 	}
 }
 
@@ -846,34 +888,67 @@ rhythmdb_query_model_entry_changed_cb (RhythmDB *db,
 	}
 
 	if (hidden) {
-		/* emit this, so things know why it was removed.
-		 * this is needed to update propery models correctly */
-		GValue true_val = { 0, };
-		GValue false_val = { 0, };
+		/* emit an entry-prop-changed signal so property models
+		 * can be updated correctly.  if we have a base model,
+		 * we'll propagate the parent's signal instead.
+		 */
+		if (model->priv->base_model == NULL) {
+			GValue true_val = { 0, };
+			GValue false_val = { 0, };
 
-		g_value_init (&true_val, G_TYPE_BOOLEAN);
-		g_value_set_boolean (&true_val, TRUE);
-		g_value_init (&false_val, G_TYPE_BOOLEAN);
-		g_value_set_boolean (&false_val, FALSE);
+			g_value_init (&true_val, G_TYPE_BOOLEAN);
+			g_value_set_boolean (&true_val, TRUE);
+			g_value_init (&false_val, G_TYPE_BOOLEAN);
+			g_value_set_boolean (&false_val, FALSE);
 
-		rb_debug ("emitting hidden-removal notification for %s",
-			  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
-		g_signal_emit (G_OBJECT (model),
-			       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
-			       entry, RHYTHMDB_PROP_HIDDEN, &false_val, &true_val);
-		g_value_unset (&true_val);
-		g_value_unset (&false_val);
+			rb_debug ("emitting hidden-removal notification for %s",
+				  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+			g_signal_emit (G_OBJECT (model),
+				       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
+				       entry, RHYTHMDB_PROP_HIDDEN, &false_val, &true_val);
+			g_value_unset (&true_val);
+			g_value_unset (&false_val);
+		}
+
+		/* if we don't have a query to help us decide, we need to
+		 * track hidden entries that were in this query model,
+		 * so we can add them back in if they become visible again.
+		 * if we have a query, any entry that matches will be added.
+		 */
+		if (model->priv->query == NULL) {
+			GtkTreeIter iter;
+			GtkTreePath *path;
+			gint index;
+
+			/* find the entry's position so we can restore it there
+			 * if it reappears
+			 */
+			g_assert (rhythmdb_query_model_entry_to_iter (model, entry, &iter));
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+			index = gtk_tree_path_get_indices (path)[0];
+			rb_debug ("adding hidden entry to map with index %d", index);
+
+			g_hash_table_insert (model->priv->hidden_entry_map,
+					     rhythmdb_entry_ref (entry),
+					     GINT_TO_POINTER (index));
+		}
 
 		rhythmdb_query_model_filter_out_entry (model, entry);
 		return;
 	}
 
-	/* emit separate change signals for each property */
+	/* emit separate change signals for each property
+	 * unless this is a chained query model, in which
+	 * case we propagate the parent model's signals instead.
+	 */
 	for (t = changes; t; t = t->next) {
 		RhythmDBEntryChange *change = t->data;
-		g_signal_emit (G_OBJECT (model),
-			       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
-			       entry, change->prop, &change->old, &change->new);
+
+		if (model->priv->base_model == NULL) {
+			g_signal_emit (G_OBJECT (model),
+				       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
+				       entry, change->prop, &change->old, &change->new);
+		}
 
 		if (change->prop == RHYTHMDB_PROP_DURATION) {
 			model->priv->total_duration -= g_value_get_ulong (&change->old);
@@ -906,6 +981,22 @@ rhythmdb_query_model_entry_changed_cb (RhythmDB *db,
 	}
 }
 
+static void
+rhythmdb_query_model_base_entry_prop_changed (RhythmDBQueryModel *base_model,
+					      RhythmDBEntry *entry,
+					      RhythmDBPropType prop,
+					      const GValue *old,
+					      const GValue *new_value,
+					      RhythmDBQueryModel *model)
+{
+	if (g_hash_table_lookup (model->priv->reverse_map, entry)) {
+		/* propagate the signal */
+		g_signal_emit (G_OBJECT (model),
+			       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
+			       entry, prop, old, new_value);
+	}
+}
+
 static void
 rhythmdb_query_model_entry_deleted_cb (RhythmDB *db,
 				       RhythmDBEntry *entry,
@@ -1117,10 +1208,11 @@ static void
 rhythmdb_query_model_remove_from_main_list (RhythmDBQueryModel *model,
 					    RhythmDBEntry *entry)
 {
-	GSequencePtr ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
+	GSequencePtr ptr;
 	int index;
 	GtkTreePath *path;
 
+	ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
 	index = g_sequence_ptr_get_position (ptr);
 
 	path = gtk_tree_path_new ();
@@ -1135,6 +1227,10 @@ rhythmdb_query_model_remove_from_main_list (RhythmDBQueryModel *model,
 	/* take temporary ref */
 	rhythmdb_entry_ref (entry);
 
+	/* find the sequence pointer again in case a row-deleted
+	 * signal handler moved it.
+	 */
+	ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
 	g_sequence_remove (ptr);
 	g_assert (g_hash_table_remove (model->priv->reverse_map, entry));
 
@@ -1581,52 +1677,62 @@ rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 				    GtkSelectionData *selection_data)
 {
 	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (dragsource);
+	RhythmDBEntry *entry;
+	GString *data;
 	guint target;
+	GList *tem;
+	gboolean need_newline = FALSE;
 
 	rb_debug ("getting drag data");
 
-	if (gtk_target_list_find (rhythmdb_query_model_drag_target_list,
+	if (!gtk_target_list_find (rhythmdb_query_model_drag_target_list,
 				  selection_data->target, &target)) {
-		RhythmDBEntry *entry;
-		GList *tem;
-		GString *data;
-		gboolean need_newline = FALSE;
-
-		data = g_string_new ("");
-
-		for (tem = paths; tem; tem = tem->next) {
-			GtkTreeIter iter;
-			GtkTreePath *path;
-			const char *location;
-
-			path = gtk_tree_row_reference_get_path (tem->data);
-
-			gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
-
-			entry = g_sequence_ptr_get_data (iter.user_data);
-
-			location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-			if (location == NULL)
-				continue;
-
-			if (need_newline)
-				g_string_append (data, "\r\n");
-
-			g_string_append (data, location);
-			need_newline = TRUE;
-		}
-
-		gtk_selection_data_set (selection_data,
-					selection_data->target,
-					8, (guchar *) data->str,
-					data->len);
-
-		g_string_free (data, TRUE);
-
-		return TRUE;
+		return FALSE;
 	}
 
-	return FALSE;
+
+	data = g_string_new ("");
+
+	for (tem = paths; tem; tem = tem->next) {
+		GtkTreeIter iter;
+		GtkTreePath *path;
+
+		path = gtk_tree_row_reference_get_path (tem->data);
+
+		gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
+
+		entry = g_sequence_ptr_get_data (iter.user_data);
+
+		if (need_newline)
+			g_string_append (data, "\r\n");
+
+		if (target == TARGET_URIS) {
+			char *location;
+
+			location = rhythmdb_entry_get_playback_uri (entry);
+			if (location == NULL) {
+				need_newline = FALSE;
+				continue;
+			}
+
+			g_string_append (data, location);
+			g_free (location);
+		} else if (target == TARGET_ENTRIES) {
+			g_string_append_printf (data,
+						"%lu",
+						rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_ENTRY_ID));
+		}
+		need_newline = TRUE;
+	}
+
+	gtk_selection_data_set (selection_data,
+				selection_data->target,
+				8, (guchar *) data->str,
+				data->len);
+
+	g_string_free (data, TRUE);
+
+	return TRUE;
 }
 
 static gboolean
@@ -1671,8 +1777,11 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 		GSequencePtr ptr;
 		char **strv;
 		RhythmDBEntry *entry;
+		gboolean uri_list;
 		int i = 0;
 
+		uri_list = (selection_data->type == gdk_atom_intern ("text/uri-list", TRUE));
+
 		strv = g_strsplit ((char *) selection_data->data, "\r\n", -1);
 
 		if (dest == NULL || !rhythmdb_query_model_get_iter (GTK_TREE_MODEL (model), &iter, dest))
@@ -1690,20 +1799,22 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 			if (g_utf8_strlen (strv[i], -1) == 0)
 				continue;
 
-			entry = rhythmdb_entry_lookup_by_location (model->priv->db,
-								   strv[i]);
-
+			entry = rhythmdb_entry_lookup_from_string (model->priv->db, strv[i], !uri_list);
 			if (entry == NULL) {
 				int pos;
 
-				if (g_sequence_ptr_is_end (ptr))
-					pos = -1;
-				else
-					pos = g_sequence_ptr_get_position (ptr);
+				if (uri_list) {
+					if (g_sequence_ptr_is_end (ptr))
+						pos = -1;
+					else
+						pos = g_sequence_ptr_get_position (ptr);
 
-				g_signal_emit (G_OBJECT (model),
-					       rhythmdb_query_model_signals[NON_ENTRY_DROPPED],
-					       0, strv[i], pos);
+					g_signal_emit (G_OBJECT (model),
+						       rhythmdb_query_model_signals[NON_ENTRY_DROPPED],
+						       0, strv[i], pos);
+				} else {
+					rb_debug ("got drop with entry id %s, but can't find the entry", strv[i]);
+				}
 			} else {
 				GSequencePtr old_ptr;
 				GtkTreePath *tem_path;
@@ -2287,10 +2398,12 @@ rhythmdb_query_model_base_entry_removed (RhythmDBQueryModel *base_model,
 					 RhythmDBEntry *entry,
 					 RhythmDBQueryModel *model)
 {
-	/* propagate the signal out to any attached property models */
-	g_signal_emit (G_OBJECT (model),
-		       rhythmdb_query_model_signals[ENTRY_REMOVED], 0,
-		       entry);
+	if (g_hash_table_lookup (model->priv->reverse_map, entry)) {
+		/* propagate the signal out to any attached property models */
+		g_signal_emit (G_OBJECT (model),
+			       rhythmdb_query_model_signals[ENTRY_REMOVED], 0,
+			       entry);
+	}
 }
 
 void
@@ -2456,13 +2569,15 @@ rhythmdb_query_model_album_sort_func (RhythmDBEntry *a,
 	if (ret != 0)
 		return ret;
 
-	/* Then by disc number, */
+	/* Then by disc number (assume 1 if non-existent) */
 	a_num = rhythmdb_entry_get_ulong (a, RHYTHMDB_PROP_DISC_NUMBER);
 	b_num = rhythmdb_entry_get_ulong (b, RHYTHMDB_PROP_DISC_NUMBER);
+	a_num = (a_num ? a_num : 1);
+	b_num = (b_num ? b_num : 1);
 	if (a_num != b_num)
 		return (a_num < b_num ? -1 : 1);
 
-	/* by track number */
+	/* by track number (assume 0 if non-existent) */
 	a_num = rhythmdb_entry_get_ulong (a, RHYTHMDB_PROP_TRACK_NUMBER);
 	b_num = rhythmdb_entry_get_ulong (b, RHYTHMDB_PROP_TRACK_NUMBER);
 	if (a_num != b_num)
