@@ -36,6 +36,7 @@
 #include <glade/glade-init.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-app-helper.h>
 #include <libgnomeui/gnome-authentication-manager.h>
 
 #ifdef HAVE_GSTREAMER
@@ -181,6 +182,15 @@ main (int argc, char **argv)
 	/* initialize i18n */
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
+	/* ask for utf-8 message text from GStreamer too,
+	 * since it doesn't do that itself.
+	 */
+#ifdef HAVE_GSTREAMER_0_10
+	bind_textdomain_codeset ("gstreamer-0.10", "UTF-8");
+#else
+	bind_textdomain_codeset ("gstreamer-0.8", "UTF-8");
+#endif
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
@@ -391,6 +401,8 @@ main (int argc, char **argv)
 	rb_profile_end ("starting rhythmbox");
 	g_object_unref (program);
 
+	gnome_accelerators_sync ();
+
 	exit (0);
 }
 
