@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Implementation of base class for play order classes
  *
  *  Copyright (C) 2003 Jeffrey Yasskin <jyasskin@mail.utexas.edu>
@@ -20,6 +20,8 @@
  *
  */
 
+#include "config.h"
+
 #include <string.h>
 
 #include <glib/gi18n.h>
@@ -66,7 +68,7 @@ static void rb_play_order_entry_added_cb (GtkTreeModel *model,
 static void rb_play_order_row_deleted_cb (GtkTreeModel *model,
 					  GtkTreePath *path,
 					  RBPlayOrder *porder);
-static void rb_play_order_query_model_changed_cb (GObject *source, 
+static void rb_play_order_query_model_changed_cb (GObject *source,
 						  GParamSpec *arg,
 						  RBPlayOrder *porder);
 static void rb_play_order_update_have_next_previous (RBPlayOrder *porder);
@@ -124,14 +126,15 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 						 	      "Rhythmbox Player",
 						 	      RB_TYPE_SHELL_PLAYER,
 						 	      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	
+
 	g_object_class_install_property (object_class,
 					 PROP_PLAYING_ENTRY,
-					 g_param_spec_pointer ("playing-entry",
-						 	       "RhythmDBEntry",
-						 	       "Playing entry",
-						 	       G_PARAM_READWRITE));
-	
+					 g_param_spec_boxed ("playing-entry",
+						 	     "RhythmDBEntry",
+						 	     "Playing entry",
+							     RHYTHMDB_TYPE_ENTRY,
+						 	     G_PARAM_READWRITE));
+
 	rb_play_order_signals[HAVE_NEXT_PREVIOUS_CHANGED] =
 		g_signal_new ("have_next_previous_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -152,7 +155,8 @@ rb_play_order_init (RBPlayOrder *porder)
 }
 
 static GObject *
-rb_play_order_constructor (GType type, guint n_construct_properties,
+rb_play_order_constructor (GType type,
+			   guint n_construct_properties,
 			   GObjectConstructParam *construct_properties)
 {
 	RBPlayOrder *porder;
@@ -173,18 +177,57 @@ rb_play_order_finalize (GObject *object)
 
 	porder = RB_PLAY_ORDER (object);
 
-	if (porder->priv->query_model) {
+	if (porder->priv->playing_entry != NULL) {
+		rhythmdb_entry_unref (porder->priv->playing_entry);
+	}
+
+	if (porder->priv->query_model != NULL) {
 		g_signal_handlers_disconnect_by_func (G_OBJECT (porder->priv->query_model),
 						      G_CALLBACK (rb_play_order_entry_added_cb),
 						      porder);
 		g_signal_handlers_disconnect_by_func (G_OBJECT (porder->priv->query_model),
 						      G_CALLBACK (rb_play_order_row_deleted_cb),
 						      porder);
+		g_object_unref (porder->priv->query_model);
+	}
+
+	if (porder->priv->db != NULL) {
+		g_object_unref (porder->priv->db);
 	}
 
 	G_OBJECT_CLASS (rb_play_order_parent_class)->finalize (object);
 }
 
+static void
+rb_play_order_set_player (RBPlayOrder   *porder,
+			  RBShellPlayer *player)
+{
+	porder->priv->player = player;
+}
+
+static void
+rb_play_order_set_playing_entry_internal (RBPlayOrder   *porder,
+					  RhythmDBEntry *entry)
+{
+	RhythmDBEntry *old_entry;
+
+	old_entry = porder->priv->playing_entry;
+	porder->priv->playing_entry = entry;
+
+	if (porder->priv->playing_entry != NULL) {
+		rhythmdb_entry_ref (porder->priv->playing_entry);
+	}
+
+	if (RB_PLAY_ORDER_GET_CLASS (porder)->playing_entry_changed)
+		RB_PLAY_ORDER_GET_CLASS (porder)->playing_entry_changed (porder, old_entry, entry);
+
+	if (old_entry != NULL) {
+		rhythmdb_entry_unref (old_entry);
+	}
+
+	rb_play_order_update_have_next_previous (porder);
+}
+
 static void
 rb_play_order_set_property (GObject *object,
 			    guint prop_id,
@@ -192,26 +235,13 @@ rb_play_order_set_property (GObject *object,
 			    GParamSpec *pspec)
 {
 	RBPlayOrder *porder = RB_PLAY_ORDER (object);
-	RhythmDBEntry *entry;
-	RhythmDBEntry *old_entry;
 
 	switch (prop_id) {
 	case PROP_PLAYER:
-		porder->priv->player = g_value_get_object (value);
+		rb_play_order_set_player (porder, g_value_get_object (value));
 		break;
 	case PROP_PLAYING_ENTRY:
-		old_entry = porder->priv->playing_entry;
-		entry = g_value_get_pointer (value); 
-		porder->priv->playing_entry = entry;
-
-		if (RB_PLAY_ORDER_GET_CLASS (porder)->playing_entry_changed)
-			RB_PLAY_ORDER_GET_CLASS (porder)->playing_entry_changed (porder, old_entry, entry);
-		if (old_entry)
-			rhythmdb_entry_unref (porder->priv->db, old_entry);
-		if (entry)
-			rhythmdb_entry_ref (porder->priv->db, entry);
-
-		rb_play_order_update_have_next_previous (porder);
+		rb_play_order_set_playing_entry_internal (porder, g_value_get_boxed (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -233,7 +263,7 @@ rb_play_order_get_property (GObject *object,
 		g_value_set_object (value, porder->priv->player);
 		break;
 	case PROP_PLAYING_ENTRY:
-		g_value_set_pointer (value, porder->priv->playing_entry);
+		g_value_set_boxed (value, porder->priv->playing_entry);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -245,7 +275,7 @@ rb_play_order_get_property (GObject *object,
  * rb_play_order_get_orders:
  *
  * Defines the set of available play orders, their translatable descriptions, their
- * constructor functions, whether they should appear in a drop-down list of 
+ * constructor functions, whether they should appear in a drop-down list of
  * play orders, and which one is the default.
  *
  * This should be the only function with full knowledge of what play orders are
@@ -280,7 +310,8 @@ rb_play_order_get_orders (void)
  * Returns: #RBPlayOrder instance
  **/
 RBPlayOrder *
-rb_play_order_new (const char* porder_name, RBShellPlayer *player)
+rb_play_order_new (const char* porder_name,
+		   RBShellPlayer *player)
 {
 	int default_index = -1;
 	const RBPlayOrderDescription *orders = rb_play_order_get_orders ();
@@ -317,6 +348,8 @@ rb_play_order_new (const char* porder_name, RBShellPlayer *player)
 RBShellPlayer *
 rb_play_order_get_player (RBPlayOrder *porder)
 {
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
+
 	return porder->priv->player;
 }
 
@@ -331,6 +364,8 @@ rb_play_order_get_player (RBPlayOrder *porder)
 RBSource *
 rb_play_order_get_source (RBPlayOrder *porder)
 {
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
+
 	return porder->priv->source;
 }
 
@@ -345,6 +380,8 @@ rb_play_order_get_source (RBPlayOrder *porder)
 RhythmDB *
 rb_play_order_get_db (RBPlayOrder *porder)
 {
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
+
 	return porder->priv->db;
 }
 
@@ -359,6 +396,8 @@ rb_play_order_get_db (RBPlayOrder *porder)
 RhythmDBQueryModel *
 rb_play_order_get_query_model (RBPlayOrder *porder)
 {
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
+
 	return porder->priv->query_model;
 }
 
@@ -371,6 +410,8 @@ rb_play_order_get_query_model (RBPlayOrder *porder)
 gboolean
 rb_play_order_player_is_playing (RBPlayOrder *porder)
 {
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), FALSE);
+
 	return (porder->priv->playing_entry != NULL);
 }
 
@@ -385,7 +426,9 @@ void
 rb_play_order_set_playing_entry (RBPlayOrder *porder,
 				 RhythmDBEntry *entry)
 {
-	g_object_set (G_OBJECT (porder), "playing-entry", entry, NULL);
+	g_return_if_fail (RB_IS_PLAY_ORDER (porder));
+
+	rb_play_order_set_playing_entry_internal (porder, entry);
 }
 
 /**
@@ -397,7 +440,16 @@ rb_play_order_set_playing_entry (RBPlayOrder *porder,
 RhythmDBEntry *
 rb_play_order_get_playing_entry (RBPlayOrder *porder)
 {
-	return porder->priv->playing_entry;
+	RhythmDBEntry *entry;
+
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
+
+	entry = porder->priv->playing_entry;
+	if (entry != NULL) {
+		rhythmdb_entry_ref (entry);
+	}
+
+	return entry;
 }
 
 /**
@@ -416,24 +468,31 @@ rb_play_order_playing_source_changed (RBPlayOrder *porder,
 {
 	RhythmDB *db = NULL;
 
-	g_return_if_fail (porder != NULL);
+	g_return_if_fail (RB_IS_PLAY_ORDER (porder));
 
-	g_object_get (G_OBJECT (porder->priv->player),
+	g_object_get (porder->priv->player,
 		      "db", &db,
 		      NULL);
 
 	if (db != porder->priv->db) {
 		if (RB_PLAY_ORDER_GET_CLASS (porder)->db_changed)
 			RB_PLAY_ORDER_GET_CLASS (porder)->db_changed (porder, db);
-		porder->priv->db = db;
+
+		if (porder->priv->db != NULL) {
+			g_object_unref (porder->priv->db);
+		}
+
+		porder->priv->db = g_object_ref (db);
 	}
 
+	g_object_unref (db);
+
 	if (source != porder->priv->source) {
 		if (porder->priv->source) {
 			g_signal_handler_disconnect (G_OBJECT (porder->priv->source),
 						     porder->priv->query_model_change_id);
 		}
-		
+
 		porder->priv->source = source;
 		if (porder->priv->source) {
 			porder->priv->query_model_change_id =
@@ -447,13 +506,13 @@ rb_play_order_playing_source_changed (RBPlayOrder *porder,
 
 		if (RB_PLAY_ORDER_GET_CLASS (porder)->playing_source_changed)
 			RB_PLAY_ORDER_GET_CLASS (porder)->playing_source_changed (porder);
-		
+
 		rb_play_order_update_have_next_previous (porder);
 	}
 }
 
 static void
-rb_play_order_query_model_changed_cb (GObject *source, 
+rb_play_order_query_model_changed_cb (GObject *source,
 				      GParamSpec *arg,
 				      RBPlayOrder *porder)
 {
@@ -470,16 +529,20 @@ rb_play_order_query_model_changed_cb (GObject *source,
  * query_model_changed() to make any necessary adjustments if they
  * store any state based on the contents of the #RhythmDBQueryModel.
  */
-void 
+void
 rb_play_order_query_model_changed (RBPlayOrder *porder)
 {
 	RhythmDBQueryModel *new_model = NULL;
 
-	if (porder->priv->source)
-		g_object_get (G_OBJECT (porder->priv->source), "query-model", &new_model, NULL);
+	g_return_if_fail (RB_IS_PLAY_ORDER (porder));
 
-	if (porder->priv->query_model == new_model)
+	if (porder->priv->source)
+		g_object_get (porder->priv->source, "query-model", &new_model, NULL);
+
+	if (porder->priv->query_model == new_model) {
+		g_object_unref (new_model);
 		return;
+	}
 
 	if (porder->priv->query_model != NULL) {
 		g_signal_handlers_disconnect_by_func (G_OBJECT (porder->priv->query_model),
@@ -506,7 +569,7 @@ rb_play_order_query_model_changed (RBPlayOrder *porder)
 
 	if (RB_PLAY_ORDER_GET_CLASS (porder)->query_model_changed)
 		RB_PLAY_ORDER_GET_CLASS (porder)->query_model_changed (porder);
-		
+
 	rb_play_order_update_have_next_previous (porder);
 }
 
@@ -518,20 +581,27 @@ rb_play_order_query_model_changed (RBPlayOrder *porder)
  * @porder: #RBPlayOrder instance
  *
  * Called when a new entry is added to the active #RhythmDBQueryModel.
- * Subclasses should implement entry_added() to make any necessary 
- * changes if they store any state based on the contents of the 
+ * Subclasses should implement entry_added() to make any necessary
+ * changes if they store any state based on the contents of the
  * #RhythmDBQueryModel.
  */
 static void
-rb_play_order_entry_added_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, RBPlayOrder *porder)
+rb_play_order_entry_added_cb (GtkTreeModel *model,
+			      GtkTreePath *path,
+			      GtkTreeIter *iter,
+			      RBPlayOrder *porder)
 {
-	RhythmDBEntry *entry = rhythmdb_query_model_iter_to_entry(RHYTHMDB_QUERY_MODEL (model),
-								  iter);
+	RhythmDBEntry *entry;
+
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model),
+						    iter);
 	if (RB_PLAY_ORDER_GET_CLASS (porder)->entry_added)
 		RB_PLAY_ORDER_GET_CLASS (porder)->entry_added (porder, entry);
-	
+
 	if (!rhythmdb_query_model_has_pending_changes (RHYTHMDB_QUERY_MODEL (model)))
 		rb_play_order_update_have_next_previous (porder);
+
+	rhythmdb_entry_unref (entry);
 }
 
 /**
@@ -541,15 +611,17 @@ rb_play_order_entry_added_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIte
  * @porder: #RBPlayOrder instance
  *
  * Called when an entry is removed from the active #RhythmDBQueryModel.
- * Subclasses should implement entry_removed() to make any necessary 
- * changes if they store any state based on the contents of the 
+ * Subclasses should implement entry_removed() to make any necessary
+ * changes if they store any state based on the contents of the
  * #RhythmDBQueryModel.
  *
  * If the removed entry is the current playing entry, the playing-entry-deleted
  * signal is emitted.
  */
 static void
-rb_play_order_row_deleted_cb (GtkTreeModel *model, GtkTreePath *row, RBPlayOrder *porder)
+rb_play_order_row_deleted_cb (GtkTreeModel *model,
+			      GtkTreePath *row,
+			      RBPlayOrder *porder)
 {
 	RhythmDBEntry *entry;
 
@@ -564,24 +636,48 @@ rb_play_order_row_deleted_cb (GtkTreeModel *model, GtkTreePath *row, RBPlayOrder
 	if (!rhythmdb_query_model_has_pending_changes (RHYTHMDB_QUERY_MODEL (model)))
 		rb_play_order_update_have_next_previous (porder);
 
+	rhythmdb_entry_unref (entry);
 }
 
 static gboolean
 default_has_next (RBPlayOrder *porder)
 {
-	return rb_play_order_get_next (porder) != NULL;
+	RhythmDBEntry *entry;
+	gboolean res;
+
+	entry = rb_play_order_get_next (porder);
+
+	res = (entry != NULL);
+	if (entry)
+		rhythmdb_entry_unref (entry);
+
+	return res;
 }
 
 static gboolean
 default_has_previous (RBPlayOrder *porder)
 {
-	return rb_play_order_get_previous (porder) != NULL;
+	RhythmDBEntry *entry;
+	gboolean res;
+
+	entry = rb_play_order_get_previous (porder);
+
+	res = (entry != NULL);
+	if (entry)
+		rhythmdb_entry_unref (entry);
+
+	return res;
 }
 
 static gboolean
 sync_playing_entry_cb (RBPlayOrder *porder)
 {
-	RBShellPlayer *player = rb_play_order_get_player (porder);
+	RBShellPlayer *player;
+
+	GDK_THREADS_ENTER ();
+	
+	player = rb_play_order_get_player (porder);
+
 	if (porder->priv->playing_entry) {
 		rb_shell_player_play_entry (player,
 					    porder->priv->playing_entry,
@@ -598,11 +694,14 @@ sync_playing_entry_cb (RBPlayOrder *porder)
 		}
 	}
 	porder->priv->sync_playing_entry_id = 0;
+
+	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
 static void
-default_playing_entry_removed (RBPlayOrder *porder, RhythmDBEntry *entry)
+default_playing_entry_removed (RBPlayOrder *porder,
+			       RhythmDBEntry *entry)
 {
 	RBShellPlayer *player = rb_play_order_get_player (porder);
 	RBSource *source = rb_shell_player_get_playing_source (player);
@@ -612,7 +711,7 @@ default_playing_entry_removed (RBPlayOrder *porder, RhythmDBEntry *entry)
 	/* only clear the playing source if the source this play order is using
 	 * is currently playing.
 	 */
-	if (source == rb_play_order_get_source (porder)) {	
+	if (source == rb_play_order_get_source (porder)) {
 		switch (rb_source_handle_eos (source)) {
 		case RB_SOURCE_EOF_ERROR:
 		case RB_SOURCE_EOF_STOP:
@@ -626,17 +725,25 @@ default_playing_entry_removed (RBPlayOrder *porder, RhythmDBEntry *entry)
 
 				/* go to next song, in an idle function so that other handlers run first */
 				next_entry = rb_play_order_get_next (porder);
-				if (next_entry == entry)
-					next_entry = NULL;
 
-				g_object_set (G_OBJECT (porder), "playing-entry", next_entry, NULL);
+				if (next_entry == entry) {
+					rhythmdb_entry_unref (next_entry);
+					next_entry = NULL;
+				}
+
+				rb_play_order_set_playing_entry_internal (porder, next_entry);
 				if (porder->priv->sync_playing_entry_id == 0) {
-					porder->priv->sync_playing_entry_id = 
-						g_idle_add_full (G_PRIORITY_HIGH_IDLE, 
-								 (GSourceFunc) sync_playing_entry_cb, 
-								 porder, 
+					porder->priv->sync_playing_entry_id =
+						g_idle_add_full (G_PRIORITY_HIGH_IDLE,
+								 (GSourceFunc) sync_playing_entry_cb,
+								 porder,
 								 NULL);
 				}
+
+				if (next_entry != NULL) {
+					rhythmdb_entry_unref (next_entry);
+				}
+
 				break;
 			}
 		}
@@ -654,10 +761,11 @@ default_playing_entry_removed (RBPlayOrder *porder, RhythmDBEntry *entry)
  * Returns: true if there is an entry after the current playing entry in the play order.
  */
 gboolean
-rb_play_order_has_next (RBPlayOrder* porder)
+rb_play_order_has_next (RBPlayOrder *porder)
 {
-	g_return_val_if_fail (porder != NULL, FALSE);
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), FALSE);
 	g_return_val_if_fail (RB_PLAY_ORDER_GET_CLASS (porder)->has_next != NULL, FALSE);
+
 	return RB_PLAY_ORDER_GET_CLASS (porder)->has_next (porder);
 }
 
@@ -670,8 +778,9 @@ rb_play_order_has_next (RBPlayOrder* porder)
 RhythmDBEntry *
 rb_play_order_get_next (RBPlayOrder *porder)
 {
-	g_return_val_if_fail (porder != NULL, NULL);
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
 	g_return_val_if_fail (RB_PLAY_ORDER_GET_CLASS (porder)->get_next != NULL, NULL);
+
 	return RB_PLAY_ORDER_GET_CLASS (porder)->get_next (porder);
 }
 
@@ -685,12 +794,19 @@ rb_play_order_get_next (RBPlayOrder *porder)
 void
 rb_play_order_go_next (RBPlayOrder *porder)
 {
-	g_return_if_fail (porder != NULL);
-	if (RB_PLAY_ORDER_GET_CLASS (porder)->go_next)
+	g_return_if_fail (RB_IS_PLAY_ORDER (porder));
+
+	if (RB_PLAY_ORDER_GET_CLASS (porder)->go_next) {
 		RB_PLAY_ORDER_GET_CLASS (porder)->go_next (porder);
-	else if (RB_PLAY_ORDER_GET_CLASS (porder)->get_next)
-		rb_play_order_set_playing_entry (porder,
-						 RB_PLAY_ORDER_GET_CLASS (porder)->get_next (porder));
+	} else if (RB_PLAY_ORDER_GET_CLASS (porder)->get_next) {
+		RhythmDBEntry *entry;
+
+		entry = RB_PLAY_ORDER_GET_CLASS (porder)->get_next (porder);
+		rb_play_order_set_playing_entry (porder, entry);
+		if (entry != NULL) {
+			rhythmdb_entry_unref (entry);
+		}
+	}
 }
 
 /**
@@ -701,10 +817,11 @@ rb_play_order_go_next (RBPlayOrder *porder)
  * If not currently playing, returns false.
  */
 gboolean
-rb_play_order_has_previous (RBPlayOrder* porder)
+rb_play_order_has_previous (RBPlayOrder *porder)
 {
-	g_return_val_if_fail (porder != NULL, FALSE);
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), FALSE);
 	g_return_val_if_fail (RB_PLAY_ORDER_GET_CLASS (porder)->has_previous != NULL, FALSE);
+
 	return RB_PLAY_ORDER_GET_CLASS (porder)->has_previous (porder);
 }
 
@@ -717,8 +834,9 @@ rb_play_order_has_previous (RBPlayOrder* porder)
 RhythmDBEntry *
 rb_play_order_get_previous (RBPlayOrder *porder)
 {
-	g_return_val_if_fail (porder != NULL, NULL);
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), NULL);
 	g_return_val_if_fail (RB_PLAY_ORDER_GET_CLASS (porder)->get_previous != NULL, NULL);
+
 	return RB_PLAY_ORDER_GET_CLASS (porder)->get_previous (porder);
 }
 
@@ -731,12 +849,19 @@ rb_play_order_get_previous (RBPlayOrder *porder)
 void
 rb_play_order_go_previous (RBPlayOrder *porder)
 {
-	g_return_if_fail (porder != NULL);
-	if (RB_PLAY_ORDER_GET_CLASS (porder)->go_previous)
+	g_return_if_fail (RB_IS_PLAY_ORDER (porder));
+
+	if (RB_PLAY_ORDER_GET_CLASS (porder)->go_previous) {
 		RB_PLAY_ORDER_GET_CLASS (porder)->go_previous (porder);
-	else if (RB_PLAY_ORDER_GET_CLASS (porder)->get_previous)
-		rb_play_order_set_playing_entry (porder,
-						 RB_PLAY_ORDER_GET_CLASS (porder)->get_previous (porder));
+	} else if (RB_PLAY_ORDER_GET_CLASS (porder)->get_previous) {
+		RhythmDBEntry *entry;
+
+		entry = RB_PLAY_ORDER_GET_CLASS (porder)->get_previous (porder);
+		rb_play_order_set_playing_entry (porder, entry);
+		if (entry != NULL) {
+			rhythmdb_entry_unref (entry);
+		}
+	}
 }
 
 /**
@@ -751,6 +876,9 @@ gboolean
 rb_play_order_model_not_empty (RBPlayOrder *porder)
 {
 	GtkTreeIter iter;
+
+	g_return_val_if_fail (RB_IS_PLAY_ORDER (porder), FALSE);
+
 	if (!porder->priv->query_model)
 		return FALSE;
 	return gtk_tree_model_get_iter_first (GTK_TREE_MODEL (porder->priv->query_model), &iter);
@@ -767,8 +895,13 @@ rb_play_order_model_not_empty (RBPlayOrder *porder)
 void
 rb_play_order_update_have_next_previous (RBPlayOrder *porder)
 {
-	gboolean have_next = rb_play_order_has_next (porder);
-	gboolean have_previous = rb_play_order_has_previous (porder);
+	gboolean have_next;
+	gboolean have_previous;
+
+	g_return_if_fail (RB_IS_PLAY_ORDER (porder));
+
+	have_next = rb_play_order_has_next (porder);
+	have_previous = rb_play_order_has_previous (porder);
 
 	if ((have_next != porder->priv->have_next) ||
 	    (have_previous != porder->priv->have_previous)) {
@@ -778,31 +911,3 @@ rb_play_order_update_have_next_previous (RBPlayOrder *porder)
 		porder->priv->have_previous = have_previous;
 	}
 }
-
-/**
- * rb_play_order_ref_entry_swapped:
- * @entry: #RhythmDBEntry to reference
- * @db: #RhythmDB instance
- *
- * Utility function that can be used with #RBHistory to implement play orders
- * that maintain an internal list of entries.
- */
-void
-rb_play_order_ref_entry_swapped (RhythmDBEntry *entry, RhythmDB *db)
-{
-	rhythmdb_entry_ref (db, entry);
-}
-
-/**
- * rb_play_order_unref_entry_swapped:
- * @entry: #RhythmDBEntry to unreference
- * @db: #RhythmDB instance
- *
- * Utility function that can be used with #RBHistory to implement play orders
- * that maintain an internal list of entries.
- */
-void
-rb_play_order_unref_entry_swapped (RhythmDBEntry *entry, RhythmDB *db)
-{
-	rhythmdb_entry_unref (db, entry);
-}
