@@ -16,30 +16,21 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
-#include <config.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <nautilus-burn-drive.h>
+#include "config.h"
+
 #include <string.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include "rb-removable-media-manager.h"
 #include "rb-library-source.h"
 #include "rb-sourcelist.h"
 #include "rb-removable-media-source.h"
-#include "rb-generic-player-source.h"
-#include "rb-audiocd-source.h"
-#include "rb-psp-source.h"
-#ifdef HAVE_HAL
-#include "rb-nokia770-source.h"
-#endif
-#ifdef WITH_IPOD_SUPPORT
-#include "rb-ipod-source.h"
-#endif
 
 #include "rb-shell.h"
 #include "rb-shell-player.h"
@@ -54,10 +45,6 @@
 #include "rb-encoder.h"
 #endif
 
-#ifndef HAVE_BURN_DRIVE_UNREF
-#define nautilus_burn_drive_unref nautilus_burn_drive_free
-#endif
-
 static void rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass);
 static void rb_removable_media_manager_init (RBRemovableMediaManager *mgr);
 static void rb_removable_media_manager_dispose (GObject *object);
@@ -75,7 +62,7 @@ static void rb_removable_media_manager_cmd_scan_media (GtkAction *action,
 						       RBRemovableMediaManager *manager);
 static void rb_removable_media_manager_cmd_eject_medium (GtkAction *action,
 					       RBRemovableMediaManager *mgr);
-static void rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr, 
+static void rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					     GtkUIManager *uimanager);
 
 static void rb_removable_media_manager_append_media_source (RBRemovableMediaManager *mgr, RBRemovableMediaSource *source);
@@ -86,12 +73,12 @@ static void rb_removable_media_manager_unmount_volume (RBRemovableMediaManager *
 				GnomeVFSVolume *volume);
 
 static void  rb_removable_media_manager_volume_mounted_cb (GnomeVFSVolumeMonitor *monitor,
-				GnomeVFSVolume *volume, 
+				GnomeVFSVolume *volume,
 				gpointer data);
 static void  rb_removable_media_manager_volume_unmounted_cb (GnomeVFSVolumeMonitor *monitor,
-				GnomeVFSVolume *volume, 
+				GnomeVFSVolume *volume,
 				gpointer data);
-static void rb_removable_media_manager_scan (RBRemovableMediaManager *manager);
+static gboolean rb_removable_media_manager_load_media (RBRemovableMediaManager *manager);
 
 #ifdef ENABLE_TRACK_TRANSFER
 static void do_transfer (RBRemovableMediaManager *manager);
@@ -99,14 +86,6 @@ static void do_transfer (RBRemovableMediaManager *manager);
 static void rb_removable_media_manager_cmd_copy_tracks (GtkAction *action,
 							RBRemovableMediaManager *mgr);
 
-typedef struct
-{
-	gboolean removed;
-	gboolean tray_opened;
-	RBRemovableMediaManager *manager;
-	NautilusBurnDrive *drive;
-} RbCdDriveInfo;
-
 typedef struct
 {
 	RBShell *shell;
@@ -120,10 +99,8 @@ typedef struct
 
 	GList *sources;
 	GHashTable *volume_mapping;
-	GHashTable *cd_drive_mapping;
 	GList *cur_volume_list;
-
-	char *playing_uri;
+	gboolean scanned;
 
 	GAsyncQueue *transfer_queue;
 	gboolean transfer_running;
@@ -141,12 +118,14 @@ enum
 	PROP_SHELL,
 	PROP_SOURCELIST,
 	PROP_SOURCE,
+	PROP_SCANNED
 };
 
 enum
 {
 	MEDIUM_ADDED,
 	TRANSFER_PROGRESS,
+	CREATE_SOURCE,
 	LAST_SIGNAL
 };
 
@@ -166,7 +145,6 @@ static GtkActionEntry rb_removable_media_manager_actions [] =
 };
 static guint rb_removable_media_manager_n_actions = G_N_ELEMENTS (rb_removable_media_manager_actions);
 
-
 static void
 rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 {
@@ -199,6 +177,13 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 							      "RBSourceList",
 							      RB_TYPE_SOURCELIST,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_SCANNED,
+					 g_param_spec_boolean ("scanned",
+						 	       "scanned",
+							       "Whether a scan has been performed",
+							       FALSE,
+							       G_PARAM_READABLE));
 
 	rb_removable_media_manager_signals[MEDIUM_ADDED] =
 		g_signal_new ("medium_added",
@@ -220,6 +205,15 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 			      G_TYPE_NONE,
 			      3, G_TYPE_INT, G_TYPE_INT, G_TYPE_DOUBLE);
 
+	rb_removable_media_manager_signals[CREATE_SOURCE] =
+		g_signal_new ("create-source",
+			      RB_TYPE_REMOVABLE_MEDIA_MANAGER,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBRemovableMediaManagerClass, create_source),
+			      rb_signal_accumulator_object_handled, NULL,
+			      rb_marshal_OBJECT__OBJECT,
+			      RB_TYPE_SOURCE,
+			      1, GNOME_VFS_TYPE_VOLUME);
 
 	g_type_class_add_private (klass, sizeof (RBRemovableMediaManagerPrivate));
 }
@@ -231,6 +225,8 @@ rb_removable_media_manager_init (RBRemovableMediaManager *mgr)
 
 	priv->volume_mapping = g_hash_table_new (NULL, NULL);
 	priv->transfer_queue = g_async_queue_new ();
+
+	g_idle_add ((GSourceFunc)rb_removable_media_manager_load_media, mgr);
 }
 
 static void
@@ -243,11 +239,11 @@ rb_removable_media_manager_dispose (GObject *object)
 	{
 		GnomeVFSVolumeMonitor *monitor = gnome_vfs_get_volume_monitor ();
 
-		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor), 
-						      G_CALLBACK (rb_removable_media_manager_volume_mounted_cb), 
+		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor),
+						      G_CALLBACK (rb_removable_media_manager_volume_mounted_cb),
 						      mgr);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor), 
-						      G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb), 
+		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor),
+						      G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb),
 						      mgr);
 	}
 
@@ -267,9 +263,8 @@ rb_removable_media_manager_finalize (GObject *object)
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (object);
 
 	g_hash_table_destroy (priv->volume_mapping);
-	g_hash_table_destroy (priv->cd_drive_mapping);
 	g_async_queue_unref (priv->transfer_queue);
-	
+
 	G_OBJECT_CLASS (rb_removable_media_manager_parent_class)->finalize (object);
 }
 
@@ -293,10 +288,11 @@ rb_removable_media_manager_set_property (GObject *object,
 		GtkUIManager *uimanager;
 
 		priv->shell = g_value_get_object (value);
-		g_object_get (G_OBJECT (priv->shell), 
-			      "ui-manager", &uimanager, 
+		g_object_get (priv->shell,
+			      "ui-manager", &uimanager,
 			      NULL);
 		rb_removable_media_manager_set_uimanager (RB_REMOVABLE_MEDIA_MANAGER (object), uimanager);
+		g_object_unref (uimanager);
 		break;
 	}
 	case PROP_SOURCELIST:
@@ -308,7 +304,7 @@ rb_removable_media_manager_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_removable_media_manager_get_property (GObject *object,
 			      guint prop_id,
 			      GValue *value,
@@ -327,6 +323,9 @@ rb_removable_media_manager_get_property (GObject *object,
 	case PROP_SOURCELIST:
 		g_value_set_object (value, priv->sourcelist);
 		break;
+	case PROP_SCANNED:
+		g_value_set_boolean (value, priv->scanned);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -343,178 +342,13 @@ rb_removable_media_manager_new (RBShell *shell,
 			     NULL);
 }
 
-#ifdef HAVE_BURN_DRIVE_DOOR
-static
-gboolean poll_tray_opened (RbCdDriveInfo *info)
-{
-	GnomeVFSVolumeMonitor *monitor =  gnome_vfs_get_volume_monitor ();
-	gboolean new_status;
-	GnomeVFSVolume *volume;
-
-	if (info->removed) {
-		nautilus_burn_drive_unref (info->drive);
-		g_free (info);
-		return FALSE;
-	}
-
-	new_status = nautilus_burn_drive_door_is_open (info->drive);
-
-	if (new_status != info->tray_opened) {
-		volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, info->drive->device);
-		if (volume) {
-			if (new_status)
-				rb_removable_media_manager_unmount_volume (info->manager, volume);
-			else
-				rb_removable_media_manager_mount_volume (info->manager, volume);
-			gnome_vfs_volume_unref (volume);
-		}
-	}
-	info->tray_opened = new_status;
-
-	return TRUE;
-}
-#endif
-
-static
-void end_cd_drive_monitor (RbCdDriveInfo *info, RBRemovableMediaManager *manager)
-{
-	/* this will be freed when the poll next gets called */
-	info->removed = TRUE;
-}
-
-static
-void begin_cd_drive_monitor (NautilusBurnDrive *drive, RBRemovableMediaManager *manager)
-{
-#ifdef HAVE_BURN_DRIVE_DOOR
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
-	RbCdDriveInfo *info = g_new0 (RbCdDriveInfo, 1);
-	GnomeVFSVolumeMonitor *monitor=  gnome_vfs_get_volume_monitor ();
-	GnomeVFSVolume *volume;
-
-	info->drive = drive;
-	info->tray_opened = nautilus_burn_drive_door_is_open (drive);
-	info->manager = manager;
-
-	g_hash_table_insert (priv->cd_drive_mapping, drive, info);
-	g_timeout_add (1000, (GSourceFunc)poll_tray_opened, info);
-
-	volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, drive->device);
-
-	if (volume) {
-		if (!nautilus_burn_drive_door_is_open (drive)) {
-			rb_removable_media_manager_mount_volume (manager, volume);
-		} else {
-			/* it may have got ejected while we weren't monitoring */
-			rb_removable_media_manager_unmount_volume (manager, volume);
-		}
-	}
-#endif
-}
-
-static char *
-split_drive_from_cdda_uri (const char *uri)
-{
-	gchar *copy, *temp, *split;
-	int len;
-
-	if (!g_str_has_prefix (uri, "cdda://"))
-		return NULL;
-
-	len = strlen ("cdda://");
-	
-	copy = g_strdup (uri);
-	split = g_utf8_strrchr (copy + len, -1, ':');
-
-	if (split == NULL) {
-		/* invalid URI, it doesn't contain a ':' */
-		g_free (copy);
-		return NULL;
-	}
-
-	*split = 0;
-	temp = g_strdup (copy + len);
-	g_free (copy);
-
-	return temp;
-}
-
-static NautilusBurnDrive *
-get_nautilus_burn_drive_for_path (const char *path)
-{
-#ifdef HAVE_BURN_DRIVE_NEW_FROM_PATH
-	return nautilus_burn_drive_new_from_path (path);
-#else
-	GList *drives, *l;
-	NautilusBurnDrive *path_drive = NULL;
-
-	drives = nautilus_burn_drive_get_list (FALSE, FALSE);
-	for (l = drives; l != NULL; l = g_list_next (l)) {
-		NautilusBurnDrive *drive = (NautilusBurnDrive*)l->data;
-
-		if (path_drive == NULL && strcmp (drive->device, path) == 0) {
-			path_drive = drive;
-		} else {
-			nautilus_burn_drive_unref (drive);
-		}
-	}
-	g_list_free (drives);
-
-	return path_drive;
-#endif
-}
-
-static void
-rb_removable_media_manager_playing_uri_changed_cb (RBShellPlayer *player,
-						   const char *uri,
-						   RBRemovableMediaManager *manager)
-{
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
-	char *old_drive = NULL;
-	char *new_drive = NULL;
-
-	/* extract the drive paths */
-	if (priv->playing_uri)
-		old_drive = split_drive_from_cdda_uri (priv->playing_uri);
-
-	if (uri)
-		new_drive = split_drive_from_cdda_uri (uri);
-
-
-	/* if the drive we're playing from has changed, adjust the polling */
-	if (old_drive == NULL || new_drive == NULL || strcmp (old_drive, new_drive) != 0) {
-		if (old_drive) {
-			NautilusBurnDrive *drive;
-
-			rb_debug ("restarting monitoring of drive %s after playing", old_drive);
-			drive = get_nautilus_burn_drive_for_path (old_drive);
-			begin_cd_drive_monitor (drive, manager);
-			nautilus_burn_drive_unref (drive);
-		}
-		
-		if (new_drive) {
-			NautilusBurnDrive *drive;
-
-			rb_debug ("stopping monitoring of drive %s while playing", new_drive);
-			drive = get_nautilus_burn_drive_for_path (new_drive);
-			/* removing it from the hash table makes it stop monitoring */
-			g_hash_table_remove (priv->cd_drive_mapping, drive);
-			nautilus_burn_drive_unref (drive);
-		}
-	}
-
-	g_free (priv->playing_uri);
-	priv->playing_uri = (uri) ? g_strdup (uri) : NULL;
-}
-
-	
-gboolean
+static gboolean
 rb_removable_media_manager_load_media (RBRemovableMediaManager *manager)
 {
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
 	GnomeVFSVolumeMonitor *monitor = gnome_vfs_get_volume_monitor ();
-	GList *drives;
-	GObject *shell_player;
 
+	GDK_THREADS_ENTER ();
+	
 	/*
 	 * Monitor new (un)mounted file systems to look for new media
 	 *
@@ -523,42 +357,23 @@ rb_removable_media_manager_load_media (RBRemovableMediaManager *manager)
 	 * (e.g. someone pressing the eject button on a cd drive). If we get the
 	 * pre-unmount signal, the corrosponding unmounted signal is ignored
 	 */
-	g_signal_connect (G_OBJECT (monitor), "volume-mounted", 
-			  G_CALLBACK (rb_removable_media_manager_volume_mounted_cb), 
+	g_signal_connect (G_OBJECT (monitor), "volume-mounted",
+			  G_CALLBACK (rb_removable_media_manager_volume_mounted_cb),
 			  manager);
-	g_signal_connect (G_OBJECT (monitor), "volume-pre-unmount", 
-			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb), 
+	g_signal_connect (G_OBJECT (monitor), "volume-pre-unmount",
+			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb),
 			  manager);
-	g_signal_connect (G_OBJECT (monitor), "volume-unmounted", 
-			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb), 
+	g_signal_connect (G_OBJECT (monitor), "volume-unmounted",
+			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb),
 			  manager);
 
-	/*
-	 * Monitor all cd drives for inserted audio cds
-	 *
-	 * This needs to be done seperately from the above, because non-HAL systems don't
-	 * (currently) report audio cd insertions as mount events.
-	 */
-	priv->cd_drive_mapping = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify)end_cd_drive_monitor);
-	drives = nautilus_burn_drive_get_list (FALSE, FALSE);
-	g_list_foreach (drives, (GFunc)begin_cd_drive_monitor, manager);
-	g_list_free (drives);
-
-	/* monitor the playing song, to disable cd drive polling */
-	shell_player = rb_shell_get_player (priv->shell);
-	g_signal_connect (shell_player, "playing-uri-changed",
-			  G_CALLBACK (rb_removable_media_manager_playing_uri_changed_cb),
-			  manager);
-
-	/* scan for media */
-	rb_removable_media_manager_scan (manager);
-
+	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
-static void 
+static void
 rb_removable_media_manager_volume_mounted_cb (GnomeVFSVolumeMonitor *monitor,
-			   GnomeVFSVolume *volume, 
+			   GnomeVFSVolume *volume,
 			   gpointer data)
 {
 	RBRemovableMediaManager *mgr = RB_REMOVABLE_MEDIA_MANAGER (data);
@@ -566,10 +381,28 @@ rb_removable_media_manager_volume_mounted_cb (GnomeVFSVolumeMonitor *monitor,
 	rb_removable_media_manager_mount_volume (mgr, volume);
 }
 
+static gboolean
+remove_volume_by_source (GnomeVFSVolume *volume, RBSource *source,
+			 RBSource *ref_source)
+{
+	return (ref_source == source);
+}
+
+static void
+rb_removable_media_manager_source_deleted_cb (RBSource *source, RBRemovableMediaManager *mgr)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
+
+	rb_debug ("removing source %p", source);
+	g_hash_table_foreach_remove (priv->volume_mapping,
+				     (GHRFunc)remove_volume_by_source,
+				     source);
+	priv->sources = g_list_remove (priv->sources, source);
+}
 
 static void
 rb_removable_media_manager_volume_unmounted_cb (GnomeVFSVolumeMonitor *monitor,
-			     GnomeVFSVolume *volume, 
+			     GnomeVFSVolume *volume,
 			     gpointer data)
 {
 	RBRemovableMediaManager *mgr = RB_REMOVABLE_MEDIA_MANAGER (data);
@@ -578,18 +411,16 @@ rb_removable_media_manager_volume_unmounted_cb (GnomeVFSVolumeMonitor *monitor,
 	rb_removable_media_manager_unmount_volume (mgr, volume);
 }
 
-
 static void
 rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSVolume *volume)
 {
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
 	RBRemovableMediaSource *source = NULL;
-	RBShell *shell;
 	char *fs_type, *device_path, *display_name, *hal_udi, *icon_name;
 	GnomeVFSDeviceType device_type;
 
 	g_assert (volume != NULL);
-	
+
 	if (g_hash_table_lookup (priv->volume_mapping, volume) != NULL)
 		return;
 
@@ -604,8 +435,6 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 	    device_type == GNOME_VFS_DEVICE_TYPE_NETWORK)
 		return;
 
-	g_object_get (G_OBJECT (mgr), "shell", &shell, NULL);
-
 	fs_type = gnome_vfs_volume_get_filesystem_type (volume);
 	device_path = gnome_vfs_volume_get_device_path (volume);
 	display_name = gnome_vfs_volume_get_display_name (volume);
@@ -619,39 +448,27 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 	rb_debug ("detecting new media - hal udi=%s", hal_udi);
 	rb_debug ("detecting new media - icon=%s", icon_name);
 
-	
 	/* rb_xxx_source_new first checks if the 'volume' parameter corresponds
 	 * to a medium of type 'xxx', and returns NULL if it doesn't.
 	 * When volume is of the appropriate type, it creates a new source
 	 * to handle this volume
 	 */
-	if (source == NULL && rb_audiocd_is_volume_audiocd (volume))
-		source = rb_audiocd_source_new (shell, volume);
-#ifdef WITH_IPOD_SUPPORT
-	if (source == NULL && rb_ipod_is_volume_ipod (volume))
-		source = rb_ipod_source_new (shell, volume);
-#endif
-	if (source == NULL && rb_psp_is_volume_player (volume))
-		source = rb_psp_source_new (shell, volume);
-#ifdef HAVE_HAL
-	if (source == NULL && rb_nokia770_is_volume_player (volume))
-		source = rb_nokia770_source_new (shell, volume);
-#endif
-	if (source == NULL && rb_generic_player_is_volume_player (volume))
-		source = rb_generic_player_source_new (shell, volume);
+
+	g_signal_emit (G_OBJECT (mgr), rb_removable_media_manager_signals[CREATE_SOURCE], 0,
+		       volume, &source);
 
 	if (source) {
 		g_hash_table_insert (priv->volume_mapping, volume, source);
 		rb_removable_media_manager_append_media_source (mgr, source);
-	} else
+	} else {
 		rb_debug ("Unhandled media");
+	}
 
 	g_free (fs_type);
 	g_free (device_path);
 	g_free (display_name);
 	g_free (hal_udi);
 	g_free (icon_name);
-	g_object_unref (G_OBJECT (shell));
 }
 
 static void
@@ -666,19 +483,9 @@ rb_removable_media_manager_unmount_volume (RBRemovableMediaManager *mgr, GnomeVF
 	source = g_hash_table_lookup (priv->volume_mapping, volume);
 	if (source) {
 		rb_source_delete_thyself (RB_SOURCE (source));
-		g_hash_table_remove (priv->volume_mapping, volume);
 	}
 }
 
-static void
-rb_removable_media_manager_source_deleted_cb (RBSource *source, RBRemovableMediaManager *mgr)
-{
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
-
-	rb_debug ("removing source %p", source);
-	priv->sources = g_list_remove (priv->sources, source);
-}
-
 static void
 rb_removable_media_manager_append_media_source (RBRemovableMediaManager *mgr, RBRemovableMediaSource *source)
 {
@@ -687,12 +494,13 @@ rb_removable_media_manager_append_media_source (RBRemovableMediaManager *mgr, RB
 	priv->sources = g_list_prepend (priv->sources, source);
 	g_signal_connect_object (G_OBJECT (source), "deleted",
 				 G_CALLBACK (rb_removable_media_manager_source_deleted_cb), mgr, 0);
+
 	g_signal_emit (G_OBJECT (mgr), rb_removable_media_manager_signals[MEDIUM_ADDED], 0,
 		       source);
 }
 
 static void
-rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr, 
+rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					  GtkUIManager *uimanager)
 {
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
@@ -708,6 +516,10 @@ rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 
 	priv->uimanager = uimanager;
 
+	if (priv->uimanager != NULL) {
+		g_object_ref (priv->uimanager);
+	}
+
 	if (priv->actiongroup == NULL) {
 		priv->actiongroup = gtk_action_group_new ("RemovableMediaActions");
 		gtk_action_group_set_translation_domain (priv->actiongroup,
@@ -718,6 +530,15 @@ rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					      mgr);
 	}
 
+#ifndef ENABLE_TRACK_TRANSFER
+	{
+		GtkAction *action;
+
+		action = gtk_action_group_get_action (priv->actiongroup, "RemovableSourceCopyAllTracks");
+		gtk_action_set_visible (action, FALSE);
+	}
+#endif
+
 	gtk_ui_manager_insert_action_group (priv->uimanager,
 					    priv->actiongroup,
 					    0);
@@ -732,7 +553,7 @@ rb_removable_media_manager_eject_medium_cb (gboolean succeeded,
 	if (succeeded)
 		return;
 
-	rb_error_dialog (NULL, error, detailed_error);
+	rb_error_dialog (NULL, error, "%s", detailed_error);
 }
 
 static void
@@ -742,7 +563,7 @@ rb_removable_media_manager_cmd_eject_medium (GtkAction *action, RBRemovableMedia
 	RBRemovableMediaSource *source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
 	GnomeVFSVolume *volume;
 
-	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
+	g_object_get (source, "volume", &volume, NULL);
 	rb_removable_media_manager_unmount_volume (mgr, volume);
 	gnome_vfs_volume_eject (volume, (GnomeVFSVolumeOpCallback)rb_removable_media_manager_eject_medium_cb, mgr);
 	gnome_vfs_volume_unref (volume);
@@ -777,7 +598,7 @@ rb_removable_media_manager_unmount_volume_swap (GnomeVFSVolume *volume, RBRemova
 	rb_removable_media_manager_unmount_volume (manager, volume);
 }
 
-static void
+void
 rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 {
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
@@ -785,9 +606,11 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 	GList *list, *it;
 	GnomeVFSVolume *volume;
 	struct VolumeCheckData check_data;
-		
+
+	priv->scanned = TRUE;
+
 	list = gnome_vfs_volume_monitor_get_mounted_volumes (monitor);
-	
+
 	/* see if any removable media has gone */
 	check_data.volume_list = list;
 	check_data.manager = manager;
@@ -799,7 +622,7 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 			(GFunc) rb_removable_media_manager_unmount_volume_swap,
 			manager);
 	g_list_free (check_data.volumes_to_remove);
-	
+
 	/* look for new volume media */
 	for (it = list; it != NULL; it = g_list_next (it)) {
 		volume = GNOME_VFS_VOLUME (it->data);
@@ -807,21 +630,8 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 		gnome_vfs_volume_unref (volume);
 	}
 	g_list_free (list);
-
-	/* scan cd drives */
-	list = nautilus_burn_drive_get_list (FALSE, FALSE);
-	for  (it = list; it != NULL; it = g_list_next (it)) {
-		NautilusBurnDrive *drive = (NautilusBurnDrive*)it->data;
-		volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, drive->device);
-		if (volume) {
-			rb_removable_media_manager_mount_volume (manager, volume);
-			gnome_vfs_volume_unref (volume);
-		}
-	}
-	g_list_free (list);
 }
 
-
 #ifdef ENABLE_TRACK_TRANSFER
 /* Track transfer */
 
@@ -894,7 +704,7 @@ do_transfer (RBRemovableMediaManager *manager)
 	RBEncoder *encoder;
 
 	g_assert (rb_is_main_thread ());
-	
+
 	emit_progress (manager);
 
 	if (priv->transfer_running)
@@ -974,15 +784,14 @@ rb_removable_media_manager_cmd_copy_tracks (GtkAction *action, RBRemovableMediaM
 	GList *list = NULL;
 
 	source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
-	g_object_get (G_OBJECT (source), "query-model", &model, NULL);
-	g_object_get (G_OBJECT (priv->shell), "library-source", &library, NULL);
+	g_object_get (source, "query-model", &model, NULL);
+	g_object_get (priv->shell, "library-source", &library, NULL);
 
 	gtk_tree_model_foreach (GTK_TREE_MODEL (model), (GtkTreeModelForeachFunc)copy_entry, &list);
 	rb_source_paste (RB_SOURCE (library), list);
 	g_list_free (list);
 
-	g_object_unref (G_OBJECT (model));
-	g_object_unref (G_OBJECT (library));
+	g_object_unref (model);
+	g_object_unref (library);
 #endif
 }
-
