@@ -142,7 +142,7 @@ static GtkActionEntry rb_shell_clipboard_actions [] =
 	{ "EditDelete", GTK_STOCK_REMOVE, N_("_Remove"), NULL,
 	  N_("Remove selection"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_delete) },
-	{ "EditMovetoTrash", GTK_STOCK_DELETE, N_("_Move to Trash"), "<control>T",
+	{ "EditMovetoTrash", GTK_STOCK_DELETE, N_("_Move to Trash"), NULL,
 	  N_("Move selection to the trash"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_move_to_trash) },
 
@@ -505,6 +505,7 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	gboolean can_add_to_queue = FALSE;
 	gboolean can_move_to_trash = FALSE;
 	gboolean can_select_all = FALSE;
+	gboolean can_show_properties = FALSE;
 	GtkAction *action;
 	RhythmDBEntryType entry_type;
 
@@ -534,6 +535,7 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 		can_delete = rb_source_can_delete (clipboard->priv->source);
 		can_copy = rb_source_can_copy (clipboard->priv->source);
 		can_move_to_trash = rb_source_can_move_to_trash (clipboard->priv->source);
+		can_show_properties = rb_source_can_show_properties (clipboard->priv->source);
 
 		if (clipboard->priv->queue_source)
 			can_add_to_queue = rb_source_can_add_to_queue (clipboard->priv->source);
@@ -561,7 +563,7 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	g_object_set (G_OBJECT (action), "sensitive", can_add_to_queue, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "MusicProperties");
-	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
+	g_object_set (G_OBJECT (action), "sensitive", can_show_properties, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueMusicProperties");
 	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
@@ -584,15 +586,35 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 }
 
+static GtkWidget*
+get_focussed_widget (RBShellClipboard *clipboard)
+{
+	GtkWidget *window;
+	GtkWidget *widget;
+
+	/* FIXME: this should be better */
+	window = gtk_widget_get_toplevel (GTK_WIDGET (clipboard->priv->source));
+	widget = gtk_window_get_focus (GTK_WINDOW (window));
+
+	return widget;
+}
+
 static void
 rb_shell_clipboard_cmd_select_all (GtkAction *action,
 				   RBShellClipboard *clipboard)
 {
 	RBEntryView *entryview;
-	rb_debug ("select all");
+	GtkWidget *widget;
 
-	entryview = rb_source_get_entry_view (clipboard->priv->source);
-	rb_entry_view_select_all (entryview);
+	rb_debug ("select all");
+	widget = get_focussed_widget (clipboard);
+	if (GTK_IS_EDITABLE (widget)) {
+		gtk_editable_select_region (GTK_EDITABLE (widget), 0, -1);
+	} else {
+		/* select all tracks in the entry view */
+		entryview = rb_source_get_entry_view (clipboard->priv->source);
+		rb_entry_view_select_all (entryview);
+	}
 }
 
 static void
@@ -600,10 +622,16 @@ rb_shell_clipboard_cmd_select_none (GtkAction *action,
 				    RBShellClipboard *clipboard)
 {
 	RBEntryView *entryview;
-	rb_debug ("select none");
+	GtkWidget *widget;
 
-	entryview = rb_source_get_entry_view (clipboard->priv->source);
-	rb_entry_view_select_none (entryview);
+	rb_debug ("select none");
+	widget = get_focussed_widget (clipboard);
+	if (GTK_IS_EDITABLE (widget)) {
+		gtk_editable_select_region (GTK_EDITABLE (widget), -1, -1);
+	} else {
+		entryview = rb_source_get_entry_view (clipboard->priv->source);
+		rb_entry_view_select_none (entryview);
+	}
 }
 
 static void
