@@ -54,7 +54,6 @@
 #include "rb-header.h"
 #include "totem-pl-parser.h"
 #include "rb-metadata.h"
-#include "rb-iradio-source.h"
 #include "rb-library-source.h"
 #include "eel-gconf-extensions.h"
 #include "rb-util.h"
@@ -125,13 +124,8 @@ static void rb_shell_player_sync_replaygain (RBShellPlayer *player,
                                              RhythmDBEntry *entry);
 static void tick_cb (RBPlayer *player, long elapsed, gpointer data);
 static void error_cb (RBPlayer *player, const GError *err, gpointer data);
-static void buffering_cb (RBPlayer *player, guint progress, gpointer data);
 static void rb_shell_player_error (RBShellPlayer *player, gboolean async, const GError *err);
 
-static void info_available_cb (RBPlayer *player,
-                               RBMetaDataField field,
-                               GValue *value,
-                               gpointer data);
 static void rb_shell_player_set_play_order (RBShellPlayer *player,
 					    const gchar *new_val);
 static void rb_shell_player_play_order_update_cb (RBPlayOrder *porder,
@@ -151,6 +145,12 @@ static void gconf_song_position_slider_visibility_changed (GConfClient *client,g
 static void rb_shell_player_playing_changed_cb (RBShellPlayer *player,
 						GParamSpec *arg1,
 						gpointer user_data);
+static void rb_shell_player_extra_metadata_cb (RhythmDB *db,
+					       RhythmDBEntry *entry,
+					       const char *field,
+					       GValue *metadata,
+					       RBShellPlayer *player);
+
 
 static gboolean rb_shell_player_jump_to_current_idle (RBShellPlayer *player);
 
@@ -188,7 +188,7 @@ struct RBShellPlayerPrivate
 
 	char *song;
 	char *url;
-	long elapsed;
+	guint elapsed;
 
 	RBPlayOrder *play_order;
 	RBPlayOrder *queue_play_order;
@@ -222,7 +222,6 @@ enum
 	PROP_STATUSBAR,
 	PROP_QUEUE_SOURCE,
 	PROP_QUEUE_ONLY,
-	PROP_STREAM_SONG,
 	PROP_PLAYING_FROM_QUEUE,
 	PROP_PLAYER,
 };
@@ -352,8 +351,6 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 							      G_TYPE_OBJECT,
 							      G_PARAM_READABLE));
 
-	/* If you change these, be sure to update the CORBA interface
-	 * in rb-remote-bonobo.c! */
 	g_object_class_install_property (object_class,
 					 PROP_PLAY_ORDER,
 					 g_param_spec_string ("play-order",
@@ -385,14 +382,6 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 							      RB_TYPE_STATUSBAR,
 							      G_PARAM_READWRITE));
 
-	g_object_class_install_property (object_class,
-					 PROP_STREAM_SONG,
-					 g_param_spec_string ("stream-song",
-							      "stream-song",
-							      "Current stream song title",
-							      "",
-							      G_PARAM_READABLE));
-
 	rb_shell_player_signals[WINDOW_TITLE_CHANGED] =
 		g_signal_new ("window_title_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -488,6 +477,10 @@ rb_shell_player_constructor (GType type,
 	player = RB_SHELL_PLAYER (G_OBJECT_CLASS (rb_shell_player_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
 
+	player->priv->header_widget = rb_header_new (player, player->priv->db);
+	gtk_widget_show (GTK_WIDGET (player->priv->header_widget));
+	gtk_box_pack_start (GTK_BOX (player), GTK_WIDGET (player->priv->header_widget), TRUE, TRUE, 0);
+
 	gtk_action_group_add_actions (player->priv->actiongroup,
 				      rb_shell_player_actions,
 				      rb_shell_player_n_actions,
@@ -709,11 +702,6 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_container_set_border_width (GTK_CONTAINER (player), 3);
 
 	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
-				 "info",
-				 G_CALLBACK (info_available_cb),
-				 player, 0);
-
-	g_signal_connect_object (player->priv->mmplayer,
 				 "eos",
 				 G_CALLBACK (rb_shell_player_handle_eos),
 				 player, G_CONNECT_SWAPPED);
@@ -728,11 +716,6 @@ rb_shell_player_init (RBShellPlayer *player)
 				 G_CALLBACK (error_cb),
 				 player, 0);
 
-	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
-				 "buffering",
-				 G_CALLBACK (buffering_cb),
-				 player, 0);
-
 	g_signal_connect (G_OBJECT (gnome_vfs_get_volume_monitor ()),
 			  "volume-pre-unmount",
 			  G_CALLBACK (volume_pre_unmount_cb),
@@ -743,10 +726,6 @@ rb_shell_player_init (RBShellPlayer *player)
 					    (GConfClientNotifyFunc)gconf_play_order_changed,
 					    player);
 
-	player->priv->header_widget = rb_header_new (player);
-	gtk_widget_show (GTK_WIDGET (player->priv->header_widget));
-	gtk_box_pack_start (GTK_BOX (player), GTK_WIDGET (player->priv->header_widget), TRUE, TRUE, 0);
-
 	player->priv->volume = eel_gconf_get_float (CONF_STATE_VOLUME);
 
 	g_signal_connect (player, "notify::playing",
@@ -832,6 +811,9 @@ rb_shell_player_set_db_internal (RBShellPlayer *player,
 		g_signal_handlers_disconnect_by_func (player->priv->db,
 						      G_CALLBACK (rb_shell_player_entry_changed_cb),
 						      player);
+		g_signal_handlers_disconnect_by_func (player->priv->db,
+						      G_CALLBACK (rb_shell_player_extra_metadata_cb),
+						      player);
 	}
 
 	player->priv->db = db;
@@ -842,6 +824,10 @@ rb_shell_player_set_db_internal (RBShellPlayer *player,
 					 "entry_changed",
 					 G_CALLBACK (rb_shell_player_entry_changed_cb),
 					 player, 0);
+		g_signal_connect_object (G_OBJECT (player->priv->db),
+					 "entry_extra_metadata_notify",
+					 G_CALLBACK (rb_shell_player_extra_metadata_cb),
+					 player, 0);
 	}
 }
 
@@ -1003,9 +989,6 @@ rb_shell_player_get_property (GObject *object,
 	case PROP_QUEUE_ONLY:
 		g_value_set_boolean (value, player->priv->queue_only);
 		break;
-	case PROP_STREAM_SONG:
-		g_value_set_string (value, player->priv->song);
-		break;
 	case PROP_PLAYING_FROM_QUEUE:
 		g_value_set_boolean (value, player->priv->current_playing_source == RB_SOURCE (player->priv->queue_source));
 		break;
@@ -1109,10 +1092,6 @@ open_location_thread (OpenLocationThreadData *data)
 	TotemPlParser *playlist;
 	TotemPlParserResult playlist_result;
 
-	GDK_THREADS_ENTER ();
-	rb_statusbar_set_progress (data->player->priv->statusbar_widget, 0.0, _("Connecting"));
-	GDK_THREADS_LEAVE ();
-
 	playlist = totem_pl_parser_new ();
 	g_signal_connect_data (G_OBJECT (playlist), "entry",
 			       G_CALLBACK (playlist_entry_cb),
@@ -1163,10 +1142,6 @@ rb_shell_player_open_location (RBShellPlayer *player,
 
 	was_playing = rb_player_playing (player->priv->mmplayer);
 
-	g_free (player->priv->song);
-	player->priv->song = NULL;
-	g_object_notify (G_OBJECT (player), "stream-song");
-
 	if (rb_source_try_playlist (player->priv->source)) {
 		OpenLocationThreadData *data;
 
@@ -1238,6 +1213,9 @@ rb_shell_player_play (RBShellPlayer *player,
 {
 	RBEntryView *songs;
 
+	if (player->priv->current_playing_source == NULL)
+		return FALSE;
+
 	if (rb_player_playing (player->priv->mmplayer))
 		return TRUE;
 
@@ -1999,8 +1977,10 @@ rb_shell_player_sync_replaygain (RBShellPlayer *player,
              	entry_album_peak = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_ALBUM_PEAK);
 	}
 
-	rb_player_set_replaygain (player->priv->mmplayer, entry_track_gain,
-				  entry_track_peak, entry_album_gain, entry_album_peak);
+	if (eel_gconf_get_boolean (CONF_USE_REPLAYGAIN)) {
+		rb_player_set_replaygain (player->priv->mmplayer, entry_track_gain,
+					  entry_track_peak, entry_album_gain, entry_album_peak);
+	}
 }
 
 gboolean
@@ -2208,16 +2188,14 @@ rb_shell_player_entry_activated_cb (RBEntryView *view,
 static void
 rb_shell_player_property_row_activated_cb (RBPropertyView *view,
 					   const char *name,
-					   RBShellPlayer *playa)
+					   RBShellPlayer *player)
 {
 	RhythmDBEntry *entry = NULL;
-	RhythmDBQueryModel *model;
-	GtkTreeIter iter;
 	GError *error = NULL;
 
 	rb_debug  ("got property activated");
 
-	rb_shell_player_set_playing_source (playa, playa->priv->selected_source);
+	rb_shell_player_set_playing_source (player, player->priv->selected_source);
 
 	/* RHYTHMDBFIXME - do we need to wait here until the query is finished?
 	 * in theory, yes, but in practice the query is started when the row is
@@ -2225,27 +2203,22 @@ rb_shell_player_property_row_activated_cb (RBPropertyView *view,
 	 * keyboard to select then activate) and is pretty much always done by
 	 * the time we get in here.
 	 */
-	g_object_get (playa->priv->selected_source, "query-model", &model, NULL);
 
-	if (!model)
+	entry = rb_play_order_get_next (player->priv->play_order);
+	if (entry == NULL) {
 		return;
-
-	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)) {
-		goto done;
 	}
 
-	entry = rhythmdb_query_model_iter_to_entry (model, &iter);
-	if (!rb_shell_player_set_playing_entry (playa, entry, TRUE, &error)) {
-		rb_shell_player_error (playa, FALSE, error);
+	rb_play_order_go_next (player->priv->play_order);
+
+	if (!rb_shell_player_set_playing_entry (player, entry, TRUE, &error)) {
+		rb_shell_player_error (player, FALSE, error);
 		g_clear_error (&error);
 	}
 
 	if (entry != NULL) {
 		rhythmdb_entry_unref (entry);
 	}
-
- done:
-	g_object_unref (model);
 }
 
 static void
@@ -2311,46 +2284,124 @@ rb_shell_player_entry_changed_cb (RhythmDB *db,
 	}
 }
 
+static void
+rb_shell_player_extra_metadata_cb (RhythmDB *db,
+				   RhythmDBEntry *entry,
+				   const char *field,
+				   GValue *metadata,
+				   RBShellPlayer *player)
+{
+
+	RhythmDBEntry *playing_entry;
+
+	playing_entry = rb_shell_player_get_playing_entry (player);
+	if (entry != playing_entry) {
+		if (playing_entry != NULL) {
+			rhythmdb_entry_unref (playing_entry);
+		}
+		return;
+	}
+
+	rb_shell_player_sync_with_source (player);
+
+	/* emit dbus signals for changes with easily marshallable types */
+	switch (G_VALUE_TYPE (metadata)) {
+	case G_TYPE_STRING:
+	case G_TYPE_BOOLEAN:
+	case G_TYPE_ULONG:
+	case G_TYPE_UINT64:
+	case G_TYPE_DOUBLE:
+		g_signal_emit (G_OBJECT (player),
+			       rb_shell_player_signals[PLAYING_SONG_PROPERTY_CHANGED], 0,
+			       rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION),
+			       field,
+			       metadata,		/* slightly silly */
+			       metadata);
+		break;
+	default:
+		break;
+	}
+}
+
+
 static void
 rb_shell_player_sync_with_source (RBShellPlayer *player)
 {
 	const char *entry_title = NULL;
 	const char *artist = NULL;
-	char *title;
+	const char *stream_name = NULL;
+	char *streaming_title = NULL;
+	char *streaming_artist = NULL;
 	RhythmDBEntry *entry;
+	char *title = NULL;
+	long elapsed;
 
 	entry = rb_shell_player_get_playing_entry (player);
 	rb_debug ("playing source: %p, active entry: %p", player->priv->current_playing_source, entry);
 
 	if (entry != NULL) {
+		GValue *value;
+
 		entry_title = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
 		artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+
+		value = rhythmdb_entry_request_extra_metadata (player->priv->db,
+							       entry,
+							       RHYTHMDB_PROP_STREAM_SONG_TITLE);
+		if (value != NULL) {
+			streaming_title = g_value_dup_string (value);
+			g_value_unset (value);
+			g_free (value);
+
+			rb_debug ("got streaming title \"%s\"", streaming_title);
+			/* use entry title for stream name */
+			stream_name = entry_title;
+			entry_title = streaming_title;
+		}
+
+		value = rhythmdb_entry_request_extra_metadata (player->priv->db,
+							       entry,
+							       RHYTHMDB_PROP_STREAM_SONG_ARTIST);
+		if (value != NULL) {
+			streaming_artist = g_value_dup_string (value);
+			g_value_unset (value);
+			g_free (value);
+
+			rb_debug ("got streaming artist \"%s\"", streaming_artist);
+			/* override artist from entry */
+			artist = streaming_artist;
+		}
 	}
 
-	if (player->priv->song && entry_title)
-		title = g_strdup_printf ("%s (%s)", player->priv->song,
-					 entry_title);
-	else if (entry_title && artist && artist[0] != '\0')
-		title = g_strdup_printf ("%s - %s", artist, entry_title);
-	else if (entry_title)
-		title = g_strdup (entry_title);
-	else
-		title = NULL;
+	if ((artist && artist[0] != '\0') || entry_title || stream_name) {
 
-	player->priv->elapsed = rb_player_get_time (player->priv->mmplayer);
+		GString *title_str = g_string_sized_new (100);
+		if (artist && artist[0] != '\0') {
+			g_string_append (title_str, artist);
+			g_string_append (title_str, " - ");
+		}
+		if (entry_title != NULL)
+			g_string_append (title_str, entry_title);
+
+		if (stream_name != NULL)
+			g_string_append_printf (title_str, " (%s)", stream_name);
+
+		title = g_string_free (title_str, FALSE);
+	}
+
+	elapsed = rb_player_get_time (player->priv->mmplayer);
+	if (elapsed < 0)
+		elapsed = 0;
+	player->priv->elapsed = elapsed;
 
 	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[WINDOW_TITLE_CHANGED], 0,
 		       title);
-	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED], 0,
-		       (guint) player->priv->elapsed);
-
-	/* Sync the header */
-	if (player->priv->song)
-		rb_header_set_title (player->priv->header_widget, title);
-	else
-		rb_header_set_title (player->priv->header_widget, entry_title);
 	g_free (title);
 
+	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED], 0,
+		       player->priv->elapsed);
+
+	/* Sync the header */
 	rb_header_set_playing_entry (player->priv->header_widget,
 				     entry,
 				     TRUE /*rb_player_seekable (player->priv->mmplayer)*/);
@@ -2359,6 +2410,9 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	if (entry != NULL) {
 		rhythmdb_entry_unref (entry);
 	}
+
+	g_free (streaming_artist);
+	g_free (streaming_title);
 }
 
 static void
@@ -2490,8 +2544,6 @@ rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
 	}
 
 	g_free (player->priv->url);
-	g_free (player->priv->song);
-	player->priv->song = NULL;
 	player->priv->url = NULL;
 
 	if (player->priv->current_playing_source == NULL)
@@ -2539,6 +2591,12 @@ rb_shell_player_stop (RBShellPlayer *player)
 	}
 
 	rb_shell_player_sync_with_source (player);
+	g_signal_emit (G_OBJECT (player),
+		       rb_shell_player_signals[PLAYING_SONG_CHANGED], 0,
+		       NULL);
+	g_signal_emit (G_OBJECT (player),
+		       rb_shell_player_signals[PLAYING_URI_CHANGED], 0,
+		       NULL);
 	g_object_notify (G_OBJECT (player), "playing");
 	rb_shell_player_sync_buttons (player);
 }
@@ -2628,6 +2686,8 @@ rb_shell_player_seek (RBShellPlayer *player, long offset)
 
 	if (rb_player_seekable (player->priv->mmplayer)) {
 		long t = rb_player_get_time (player->priv->mmplayer);
+		if (t < 0)
+			t = 0;
 		rb_player_set_time (player->priv->mmplayer, t + offset);
 	}
 }
@@ -2671,6 +2731,7 @@ rb_shell_player_error (RBShellPlayer *player,
 		       const GError *err)
 {
 	RhythmDBEntry *entry;
+	gboolean do_next;
 
 	g_return_if_fail (player->priv->handling_error == FALSE);
 
@@ -2686,14 +2747,26 @@ rb_shell_player_error (RBShellPlayer *player,
 	if (err->code == RB_PLAYER_ERROR_NO_AUDIO) {
 		/* stream has completely ended */
 		rb_shell_player_set_playing_source (player, NULL);
+		do_next = FALSE;
 	} else if ((player->priv->current_playing_source != NULL) &&
 		   (rb_source_handle_eos (player->priv->current_playing_source) == RB_SOURCE_EOF_RETRY)) {
-		/* receiving an error means a broken stream or non-audio stream, so abort */
-		rb_error_dialog (NULL,
-				 _("Couldn't start playback"),
-				 "%s", (err) ? err->message : "(null)");
-		rb_shell_player_set_playing_source (player, NULL);
-	} else if (player->priv->do_next_idle_id == 0) {
+		/* receiving an error means a broken stream or non-audio stream, so abort
+		 * unless we've got more URLs to try */
+		if (g_queue_is_empty (player->priv->playlist_urls)) {
+			rb_error_dialog (NULL,
+					 _("Couldn't start playback"),
+					 "%s", (err) ? err->message : "(null)");
+			rb_shell_player_set_playing_source (player, NULL);
+			do_next = FALSE;
+		} else {
+			rb_debug ("haven't yet exhausted the URLs from the playlist");
+			do_next = TRUE;
+		}
+	} else {
+		do_next = TRUE;
+	}
+
+	if (do_next && player->priv->do_next_idle_id == 0) {
 		player->priv->do_next_idle_id = g_idle_add ((GSourceFunc)do_next_idle, player);
 	}
 
@@ -2737,172 +2810,19 @@ tick_cb (RBPlayer *mmplayer,
 	GDK_THREADS_ENTER ();
 
 	if (rb_player_playing (mmplayer)) {
+		if (elapsed < 0)
+			elapsed = 0;
+
 		if (player->priv->elapsed != elapsed) {
 			player->priv->elapsed = elapsed;
 			g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED],
-				       0, (guint) elapsed);
+				       0, player->priv->elapsed);
 		}
 	}
 
 	GDK_THREADS_LEAVE ();
 }
 
-static void
-info_available_cb (RBPlayer *mmplayer,
-                   RBMetaDataField field,
-                   GValue *value,
-                   gpointer data)
-{
-        RBShellPlayer *player = RB_SHELL_PLAYER (data);
-        RhythmDBEntry *entry;
-        RhythmDBPropType entry_field = 0;
-        gboolean changed = FALSE;
-        gboolean set_field = FALSE;
-	RhythmDBEntryType type;
-
-        rb_debug ("info: %d", field);
-
-        /* Sanity check, this signal may come in after we stopped the
-         * player */
-        if (player->priv->source == NULL
-            || !rb_player_opened (player->priv->mmplayer)) {
-                rb_debug ("Got info_available but no playing source!");
-                return;
-        }
-
-        GDK_THREADS_ENTER ();
-
-	entry = rb_shell_player_get_playing_entry (player);
-        if (entry == NULL) {
-                rb_debug ("Got info_available but no playing entry!");
-                goto out_unlock;
-        }
-
-	type = rhythmdb_entry_get_entry_type (entry);
-	if (type != RHYTHMDB_ENTRY_TYPE_IRADIO_STATION) {
-		rb_debug ("Got info_available but entry isn't an iradio station");
-		goto out_unlock;
-	}
-
-	switch (field)	{
-	case RB_METADATA_FIELD_TITLE:
-	{
-		char *song = g_value_dup_string (value);
-		if (!g_utf8_validate (song, -1, NULL)) {
-			g_warning ("Invalid UTF-8 from internet radio: %s", song);
-			goto out_unlock;
-		}
-
-		if ((!song && player->priv->song)
-		    || !player->priv->song
-		    || strcmp (song, player->priv->song)) {
-			changed = TRUE;
-			g_free (player->priv->song);
-			player->priv->song = song;
-			g_object_notify (G_OBJECT (player), "stream-song");
-		}
-		else
-			g_free (song);
-		break;
-	}
-	case RB_METADATA_FIELD_GENRE:
-	{
-		const char *genre = g_value_get_string (value);
-		const char *existing;
-		if (!g_utf8_validate (genre, -1, NULL)) {
-			g_warning ("Invalid UTF-8 from internet radio: %s", genre);
-			goto out_unlock;
-		}
-
-		/* check if the db entry already has a genre; if so, don't change it */
-		existing = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
-		if ((existing == NULL) ||
-		    (strcmp (existing, "") == 0) ||
-		    (strcmp (existing, _("Unknown")) == 0)) {
-			entry_field = RHYTHMDB_PROP_GENRE;
-			rb_debug ("setting genre of iradio station to %s", genre);
-			set_field = TRUE;
-		} else {
-			rb_debug ("iradio station already has genre: %s; ignoring %s", existing, genre);
-		}
-		break;
-	}
-	case RB_METADATA_FIELD_COMMENT:
-	{
-		const char *name = g_value_get_string (value);
-		const char *existing;
-		const char *location;
-
-		if (!g_utf8_validate (name, -1, NULL)) {
-			g_warning ("Invalid UTF-8 from internet radio: %s", name);
-			goto out_unlock;
-		}
-
-		/* check if the db entry already has a title; if so, don't change it.
-		 * consider title==URI to be the same as no title, since that's what
-		 * happens for stations imported by DnD or commandline args.
-		 * if the station title really is the same as the URI, then surely
-		 * the station title in the stream metadata will say that too..
-		 */
-		existing = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
-		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-		if ((existing == NULL) ||
-		    (strcmp (existing, "") == 0) ||
-		    (strcmp (existing, location) == 0)) {
-			entry_field = RHYTHMDB_PROP_TITLE;
-			rb_debug ("setting title of iradio station to %s", name);
-			set_field = TRUE;
-		} else {
-			rb_debug ("iradio station already has title: %s; ignoring %s", existing, name);
-		}
-		break;
-	}
-	case RB_METADATA_FIELD_BITRATE:
-		if (!rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE)) {
-			gulong bitrate;
-
-			/* GStreamer sends us bitrate in bps, but we need it in kbps*/
-			bitrate = g_value_get_ulong (value);
-			g_value_set_ulong (value, bitrate/1000);
-
-			rb_debug ("setting bitrate of iradio station to %lu",
-				  g_value_get_ulong (value));
-			entry_field = RHYTHMDB_PROP_BITRATE;
-			set_field = TRUE;
-		}
-		break;
-	default:
-		break;
-	}
-
-	if (changed)
-		rb_shell_player_sync_with_source (player);
-
-	if (set_field && entry_field != 0) {
-		rhythmdb_entry_set (player->priv->db, entry, entry_field, value);
-		rhythmdb_commit (player->priv->db);
-	}
-
- out_unlock:
-	if (entry != NULL) {
-		rhythmdb_entry_unref (entry);
-	}
-
-	GDK_THREADS_LEAVE ();
-}
-
-static void
-buffering_cb (RBPlayer *mmplayer,
-	      guint progress,
-	      gpointer data)
-{
- 	RBShellPlayer *player = RB_SHELL_PLAYER (data);
-
-	GDK_THREADS_ENTER ();
-	rb_statusbar_set_progress (player->priv->statusbar_widget, ((double)progress)/100, _("Buffering"));
-	GDK_THREADS_LEAVE ();
-}
-
 gboolean
 rb_shell_player_get_playing_path (RBShellPlayer *shell_player,
 				  const gchar **path,
