@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Implementation of search entry/browse toggle container
  *
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
@@ -29,7 +29,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "disclosure-widget.h"
 #include "rb-source-header.h"
 #include "rb-stock-icons.h"
 #include "rb-preferences.h"
@@ -55,9 +54,6 @@ static void rb_source_header_filter_changed_cb (RBSource *source,
 static void rb_source_header_search_cb (RBSearchEntry *search,
 					const char *text,
 					RBSourceHeader *header);
-static void rb_source_header_disclosure_toggled_cb (GObject *object,
-						    GParamSpec *param_spec,
-						    gpointer data);
 static void rb_source_header_search_activate_cb (RBSearchEntry *search,
 						 RBSourceHeader *header);
 static void rb_source_header_view_browser_changed_cb (GtkAction *action,
@@ -88,7 +84,6 @@ struct RBSourceHeaderPrivate
 
 	GtkWidget *search;
 	GtkWidget *search_bar;
-	GtkWidget *disclosure;
 
 	guint browser_notify_id;
 	guint search_notify_id;
@@ -98,7 +93,7 @@ struct RBSourceHeaderPrivate
 	char *browser_key;
 
 	GHashTable *source_states;
-	
+
 };
 
 #define RB_SOURCE_HEADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SOURCE_HEADER, RBSourceHeaderPrivate))
@@ -158,7 +153,7 @@ ui_manager_add_widget_cb (GtkUIManager *ui_manager,
 			gtk_widget_show (header->priv->search_bar);
 			gtk_table_attach (GTK_TABLE (header),
 					  header->priv->search_bar,
-					  1, 3, 0, 1, 
+					  1, 3, 0, 1,
 					  GTK_EXPAND | GTK_FILL,
 					  GTK_EXPAND | GTK_FILL,
 					  5, 0);
@@ -234,14 +229,13 @@ rb_source_header_init (RBSourceHeader *header)
 	header->priv->tooltips = gtk_tooltips_new ();
 	gtk_tooltips_enable (header->priv->tooltips);
 
-	gtk_table_set_homogeneous (GTK_TABLE (header), TRUE);
 	gtk_table_set_col_spacings (GTK_TABLE (header), 5);
 	gtk_table_resize (GTK_TABLE (header), 1, 3);
 
 	ebox = GTK_EVENT_BOX (gtk_event_box_new ());
 	header->priv->search = GTK_WIDGET (rb_search_entry_new ());
-	gtk_tooltips_set_tip (GTK_TOOLTIPS (header->priv->tooltips), 
-			      GTK_WIDGET (ebox), 
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (header->priv->tooltips),
+			      GTK_WIDGET (ebox),
 			      _("Filter music display by genre, artist, album, or title"),
 			      NULL);
 	gtk_container_add (GTK_CONTAINER (ebox), GTK_WIDGET (header->priv->search));
@@ -251,22 +245,11 @@ rb_source_header_init (RBSourceHeader *header)
 	g_signal_connect_object (G_OBJECT (header->priv->search), "activate",
 				 G_CALLBACK (rb_source_header_search_activate_cb), header, 0);
 
-	header->priv->disclosure = cddb_disclosure_new (_("Show _Browser"),
-							_("Hide _Browser"));
-	gtk_widget_set_sensitive (header->priv->disclosure, FALSE);
-	g_signal_connect_object (G_OBJECT (header->priv->disclosure), "notify::expanded",
-				 G_CALLBACK (rb_source_header_disclosure_toggled_cb), header, 0);
-
-#if 0	
-	gtk_table_attach_defaults (GTK_TABLE (header),
-			           header->priv->disclosure, 2, 3, 0, 1);
-#endif
-
 	align = gtk_alignment_new (1.0, 0.5, 1.0, 1.0);
 	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (ebox));
 	gtk_table_attach (GTK_TABLE (header),
 			  align,
-			  0, 1, 0, 1, 
+			  0, 1, 0, 1,
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
 			  5, 0);
@@ -341,7 +324,7 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 
 		source_state = g_hash_table_lookup (header->priv->source_states,
 						    header->priv->selected_source);
-		
+
 		if (source_state) {
 			text = g_strdup (source_state->search_text);
 			disclosed = source_state->disclosed;
@@ -352,13 +335,13 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 
 		g_free (header->priv->browser_key);
 		header->priv->browser_key = rb_source_get_browser_key (header->priv->selected_source);
-	
+
 		rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search), text);
 		g_signal_connect_object (G_OBJECT (header->priv->selected_source),
 					 "filter_changed",
 					 G_CALLBACK (rb_source_header_filter_changed_cb),
 					 header, 0);
-	
+
 		gtk_widget_set_sensitive (GTK_WIDGET (header->priv->search),
 					  rb_source_can_search (header->priv->selected_source));
 		header->priv->have_search = rb_source_can_search (header->priv->selected_source);
@@ -370,7 +353,7 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 		else
 			/* restore the previous state of the source*/
 			header->priv->disclosed = disclosed;
-	
+
 		if (!header->priv->have_browser && !header->priv->have_search)
 			gtk_widget_hide (GTK_WIDGET (header));
 		else
@@ -414,7 +397,7 @@ rb_source_header_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_source_header_get_property (GObject *object,
 			      guint prop_id,
 			      GValue *value,
@@ -497,7 +480,7 @@ rb_source_state_sync (RBSourceHeader *header,
 
 	if (old_state) {
 		if (set_text)
-			old_state->search_text = text ? g_strdup (text) : NULL;	
+			old_state->search_text = text ? g_strdup (text) : NULL;
 		if (set_disclosure)
 			old_state->disclosed = disclosed;
 	} else {
@@ -509,11 +492,11 @@ rb_source_state_sync (RBSourceHeader *header,
 				   header);
 
 		new_state = g_new (SourceState, 1);
-		new_state->search_text = text ? g_strdup (text) : NULL;	
+		new_state->search_text = text ? g_strdup (text) : NULL;
 		new_state->disclosed = disclosed;
 		g_hash_table_insert (header->priv->source_states,
 				     header->priv->selected_source,
-				     new_state); 
+				     new_state);
 	}
 }
 
@@ -538,7 +521,7 @@ rb_source_header_clear_search (RBSourceHeader *header)
 
 	if (!rb_search_entry_searching (RB_SEARCH_ENTRY (header->priv->search)))
 	    return;
-	
+
 	if (header->priv->selected_source) {
 		rb_source_search (header->priv->selected_source, NULL);
 		rb_source_state_sync (header, TRUE, NULL, FALSE, FALSE);
@@ -549,34 +532,22 @@ rb_source_header_clear_search (RBSourceHeader *header)
 }
 
 static void
-rb_source_header_disclosure_toggled_cb (GObject *object,
-					GParamSpec *param_spec,
-					gpointer data)
+rb_source_header_view_browser_changed_cb (GtkAction *action,
+					  RBSourceHeader *header)
 {
-	RBSourceHeader *header = RB_SOURCE_HEADER (data);
-	GtkExpander *expander = GTK_EXPANDER (object);
-
-	header->priv->disclosed = gtk_expander_get_expanded (expander);
+	rb_debug ("got view browser toggle");
+	header->priv->disclosed = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 
 	if (header->priv->browser_key)
-		eel_gconf_set_boolean (header->priv->browser_key, 
+		eel_gconf_set_boolean (header->priv->browser_key,
 				       header->priv->disclosed);
 	else {
 		rb_source_state_sync (header, FALSE, NULL, TRUE, header->priv->disclosed);
 	}
 
 	rb_debug ("got view browser toggle");
-	
-	rb_source_header_sync_control_state (header);
-}
 
-static void
-rb_source_header_view_browser_changed_cb (GtkAction *action,
-					  RBSourceHeader *header)
-{
-	rb_debug ("got view browser toggle");
-	gtk_expander_set_expanded (GTK_EXPANDER (header->priv->disclosure),
-				   gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+	rb_source_header_sync_control_state (header);
 }
 
 void
@@ -587,8 +558,6 @@ rb_source_header_sync_control_state (RBSourceHeader *header)
 	GtkAction *viewall_action;
 	gboolean not_small = !eel_gconf_get_boolean (CONF_UI_SMALL_DISPLAY);
 
-	gtk_widget_set_sensitive (header->priv->disclosure,
-				  header->priv->have_browser);
 	viewbrowser_action = gtk_action_group_get_action (header->priv->actiongroup,
 							  "ViewBrowser");
 	g_object_set (G_OBJECT (viewbrowser_action), "sensitive",
@@ -602,8 +571,6 @@ rb_source_header_sync_control_state (RBSourceHeader *header)
 	g_object_set (G_OBJECT (viewall_action), "sensitive",
 		      (header->priv->have_browser || header->priv->have_search) && not_small, NULL);
 
-	gtk_expander_set_expanded (GTK_EXPANDER (header->priv->disclosure),
-				   header->priv->disclosed);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (viewbrowser_action),
 				      header->priv->disclosed);
 
