@@ -45,8 +45,11 @@ typedef struct
 	RBRemovableMediaSourceClass parent;
 } RBAudioCdSourceClass;
 
-RBRemovableMediaSource *	rb_audiocd_source_new			(RBShell *shell, GnomeVFSVolume *volume);
+RBRemovableMediaSource *	rb_audiocd_source_new		(RBPlugin *plugin,
+								 RBShell *shell,
+								 GnomeVFSVolume *volume);
 GType			rb_audiocd_source_get_type		(void);
+GType			rb_audiocd_source_register_type		(GTypeModule *module);
 
 gboolean		rb_audiocd_is_volume_audiocd		(GnomeVFSVolume *volume);
 
