@@ -19,13 +19,12 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
-
-#include <gtk/gtktreeview.h>
 #include <string.h>
-#include "rhythmdb.h"
-#include <libgnome/gnome-i18n.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #ifdef HAVE_HAL
 #include <libhal.h>
 #include <dbus/dbus.h>
@@ -34,6 +33,7 @@
 #include <libgnomevfs/gnome-vfs-volume.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <gpod/itdb.h>
+
 #include "eel-gconf-extensions.h"
 #include "rb-ipod-source.h"
 #include "rb-debug.h"
@@ -43,13 +43,14 @@
 #include "rb-static-playlist-source.h"
 #include "rb-util.h"
 #include "rhythmdb.h"
+#include "rb-cut-and-paste-code.h"
 
-#ifdef IPOD_PHONE_SUPPORT
+#ifdef IPOD_SUPPORT
 #define PHONE_VENDOR_ID 0x22b8
 #define PHONE_PRODUCT_ID 0x4810
 #endif
 
-static GObject *rb_ipod_source_constructor (GType type, 
+static GObject *rb_ipod_source_constructor (GType type,
 					    guint n_construct_properties,
 					    GObjectConstructParam *construct_properties);
 static void rb_ipod_source_dispose (GObject *object);
@@ -78,22 +79,23 @@ ipod_path_from_unix_path (const gchar *mount_point, const gchar *unix_path);
 #endif
 static void itdb_schedule_save (Itdb_iTunesDB *db);
 
-
 typedef struct
 {
 	Itdb_iTunesDB *ipod_db;
 	gchar *ipod_mount_path;
 	GHashTable *entry_map;
+
+	GList *playlists;
+
+	guint load_idle_id;
 } RBiPodSourcePrivate;
 
-
-RB_PLUGIN_DEFINE_TYPE(RBiPodSource, 
-		      rb_ipod_source, 
+RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
+		      rb_ipod_source,
 		      RB_TYPE_REMOVABLE_MEDIA_SOURCE)
 
 #define IPOD_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_IPOD_SOURCE, RBiPodSourcePrivate))
 
-
 static void
 rb_ipod_source_class_init (RBiPodSourceClass *klass)
 {
@@ -125,10 +127,12 @@ rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
 	mpl = itdb_playlist_mpl (priv->ipod_db);
-	rb_debug ("Renaming iPod from %s to %s", mpl->name, name);
-	if (strcmp (mpl->name, name) == 0) {
-		rb_debug ("iPod is already named %s", name);
-		return;
+	if (mpl->name != NULL) {
+		rb_debug ("Renaming iPod from %s to %s", mpl->name, name);
+		if (strcmp (mpl->name, name) == 0) {
+			rb_debug ("iPod is already named %s", name);
+			return;
+		}
 	}
 	g_free (mpl->name);
 	mpl->name = g_strdup (name);
@@ -136,12 +140,12 @@ rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
 }
 
 static void
-rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec, 
+rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 				gpointer data)
 {
 	char *name;
 
-	g_object_get (RB_SOURCE (source), "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 	rb_ipod_source_set_ipod_name (source, name);
 	g_free (name);
 }
@@ -149,7 +153,7 @@ rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 static void
 rb_ipod_source_init (RBiPodSource *source)
 {
-	g_signal_connect (G_OBJECT (source), "notify::name", 
+	g_signal_connect (G_OBJECT (source), "notify::name",
 			  (GCallback)rb_ipod_source_name_changed_cb, NULL);
 }
 
@@ -174,7 +178,7 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 	return G_OBJECT (source);
 }
 
-static void 
+static void
 rb_ipod_source_dispose (GObject *object)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (object);
@@ -194,18 +198,29 @@ rb_ipod_source_dispose (GObject *object)
 		priv->entry_map = NULL;
  	}
 
+	if (priv->load_idle_id != 0) {
+		g_source_remove (priv->load_idle_id);
+		priv->load_idle_id = 0;
+	}
+
 	G_OBJECT_CLASS (rb_ipod_source_parent_class)->dispose (object);
 }
 
 RBRemovableMediaSource *
-rb_ipod_source_new (RBShell *shell, GnomeVFSVolume *volume)
+rb_ipod_source_new (RBShell *shell,
+                    GnomeVFSVolume *volume)
 {
 	RBiPodSource *source;
 	RhythmDBEntryType entry_type;
+	RhythmDB *db;
 
 	g_assert (rb_ipod_is_volume_ipod (volume));
 
-	entry_type =  rhythmdb_entry_register_type (NULL);
+	g_object_get (shell, "db", &db, NULL);
+	entry_type =  rhythmdb_entry_register_type (db, NULL);
+	entry_type->save_to_disk = FALSE;
+	entry_type->category = RHYTHMDB_ENTRY_NORMAL;
+	g_object_unref (db);
 
 	source = RB_IPOD_SOURCE (g_object_new (RB_TYPE_IPOD_SOURCE,
 					  "entry-type", entry_type,
@@ -219,7 +234,7 @@ rb_ipod_source_new (RBShell *shell, GnomeVFSVolume *volume)
 	return RB_REMOVABLE_MEDIA_SOURCE (source);
 }
 
-static void 
+static void
 entry_set_string_prop (RhythmDB *db, RhythmDBEntry *entry,
 		       RhythmDBPropType propid, const char *str)
 {
@@ -230,19 +245,18 @@ entry_set_string_prop (RhythmDB *db, RhythmDBEntry *entry,
 
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_static_string (&value, str);
-	rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, propid, &value);
+	rhythmdb_entry_set (RHYTHMDB (db), entry, propid, &value);
 	g_value_unset (&value);
 }
 
-
 static char *
 ipod_path_to_uri (const char *mount_point, const char *ipod_path)
 {
  	char *rel_pc_path;
  	char *full_pc_path;
  	char *uri;
-	
- 	rel_pc_path = g_strdup (ipod_path);	
+
+ 	rel_pc_path = g_strdup (ipod_path);
  	itdb_filename_ipod2fs (rel_pc_path);
  	full_pc_path = g_build_filename (mount_point, rel_pc_path, NULL);
  	g_free (rel_pc_path);
@@ -260,32 +274,34 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
 
-  	g_object_get (G_OBJECT (source), 
-		      "shell", &shell, 
+  	g_object_get (source,
+		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
 
-	playlist_source = rb_static_playlist_source_new (shell, 
-							 playlist->name, 
+	playlist_source = rb_static_playlist_source_new (shell,
+							 playlist->name,
 							 FALSE,
 							 entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	for (it = playlist->members; it != NULL; it = it->next) {
 		Itdb_Track *song;
 		char *filename;
 
 		song = (Itdb_Track *)it->data;
- 		filename = ipod_path_to_uri (priv->ipod_mount_path, 
+ 		filename = ipod_path_to_uri (priv->ipod_mount_path,
  					    song->ipod_path);
 		rb_static_playlist_source_add_location (RB_STATIC_PLAYLIST_SOURCE (playlist_source),
 							filename, -1);
 		g_free (filename);
 	}
 
-	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
-	g_object_unref (G_OBJECT (shell));
-}
+	priv->playlists = g_list_prepend (priv->playlists, playlist_source);
 
+	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
+	g_object_unref (shell);
+}
 
 static void
 load_ipod_playlists (RBiPodSource *source)
@@ -342,27 +358,28 @@ create_ipod_song_from_entry (RhythmDBEntry *entry)
 
 static void
 add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
-{	
+{
 	RhythmDBEntry *entry;
 	RhythmDBEntryType entry_type;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	char *pc_path;
 
 	/* Set URI */
-	g_object_get (G_OBJECT (source), "entry-type", &entry_type, 
+	g_object_get (source, "entry-type", &entry_type,
 		      NULL);
 
-	pc_path = ipod_path_to_uri (priv->ipod_mount_path, 
+	pc_path = ipod_path_to_uri (priv->ipod_mount_path,
 				    song->ipod_path);
 	entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type,
 				    pc_path);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	if (entry == NULL) {
 		rb_debug ("cannot create entry %s", pc_path);
 		g_free (pc_path);
 		return;
 	}
-		
+
 	rb_debug ("Adding %s from iPod", pc_path);
 	g_free (pc_path);
 
@@ -371,8 +388,8 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, song->track_nr);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, 
-					       RHYTHMDB_PROP_TRACK_NUMBER, 
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_TRACK_NUMBER,
 					       &value);
 		g_value_unset (&value);
 	}
@@ -382,41 +399,41 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, song->cd_nr);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, 
-					       RHYTHMDB_PROP_DISC_NUMBER, 
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_DISC_NUMBER,
 					       &value);
 		g_value_unset (&value);
 	}
-		
+
 	/* Set bitrate */
 	if (song->bitrate != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, song->bitrate);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, 
-					       RHYTHMDB_PROP_BITRATE, 
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_BITRATE,
 					       &value);
 		g_value_unset (&value);
 	}
-		
+
 	/* Set length */
 	if (song->tracklen != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, song->tracklen/1000);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, 
-					       RHYTHMDB_PROP_DURATION, 
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_DURATION,
 					       &value);
 		g_value_unset (&value);
 	}
-		
+
 	/* Set file size */
 	if (song->size != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_UINT64);
 		g_value_set_uint64 (&value, song->size);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, 
-					       RHYTHMDB_PROP_FILE_SIZE, 
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_FILE_SIZE,
 					       &value);
 		g_value_unset (&value);
 	}
@@ -426,7 +443,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, song->playcount);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_PLAY_COUNT,
 					       &value);
 		g_value_unset (&value);
@@ -437,16 +454,16 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		GDate *date = NULL;
 		GType type;
 		GValue value = {0, };
-			
+
 		date = g_date_new_dmy (1, G_DATE_JANUARY, song->year);
 
 		type = rhythmdb_get_property_type (RHYTHMDB(db),
 						   RHYTHMDB_PROP_DATE);
-			
+
 		g_value_init (&value, type);
 		g_value_set_ulong (&value, (date ? g_date_get_julian (date) : 0));
-			
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_DATE,
 					       &value);
 		g_value_unset (&value);
@@ -459,55 +476,66 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_DOUBLE);
 		g_value_set_double (&value, song->rating/20.0);
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_RATING,
 					       &value);
 		g_value_unset (&value);
 	}
-		
+
 	/* Set last played */
 	if (song->time_played != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_played));
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_LAST_PLAYED,
 					       &value);
 		g_value_unset (&value);
 	}
-		
-	/* Set title */		
-	entry_set_string_prop (RHYTHMDB (db), entry, 
-			       RHYTHMDB_PROP_TITLE, song->title);
-		
-	/* Set album, artist and genre from iTunesDB */
-	entry_set_string_prop (RHYTHMDB (db), entry, 
-			       RHYTHMDB_PROP_ARTIST, song->artist);
-		
-	entry_set_string_prop (RHYTHMDB (db), entry, 
-			       RHYTHMDB_PROP_ALBUM, song->album);
-		
-	entry_set_string_prop (RHYTHMDB (db), entry, 
-			       RHYTHMDB_PROP_GENRE, song->genre);
 
+	/* Set title */
+	entry_set_string_prop (RHYTHMDB (db), entry,
+			       RHYTHMDB_PROP_TITLE, song->title);
+
+	/* Set album, artist and genre from iTunesDB */
+	entry_set_string_prop (RHYTHMDB (db), entry,
+			       RHYTHMDB_PROP_ARTIST, song->artist);
+
+	entry_set_string_prop (RHYTHMDB (db), entry,
+			       RHYTHMDB_PROP_ALBUM, song->album);
+
+	entry_set_string_prop (RHYTHMDB (db), entry,
+			       RHYTHMDB_PROP_GENRE, song->genre);
 
 	g_hash_table_insert (priv->entry_map, entry, song);
 
 	rhythmdb_commit (RHYTHMDB (db));
 }
 
-static gboolean
-load_ipod_db_idle_cb (RBiPodSource *source)
+static RhythmDB *
+get_db_for_source (RBiPodSource *source)
 {
 	RBShell *shell;
 	RhythmDB *db;
+
+  	g_object_get (source, "shell", &shell, NULL);
+  	g_object_get (shell, "db", &db, NULL);
+  	g_object_unref (shell);
+
+        return db;
+}
+
+static gboolean
+load_ipod_db_idle_cb (RBiPodSource *source)
+{
+	RhythmDB *db;
  	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
-  
-  	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-  	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-  	g_object_unref (G_OBJECT (shell));
-  
+
+	GDK_THREADS_ENTER ();
+
+        db = get_db_for_source (source);
+
   	g_assert (db != NULL);
  	for (it = priv->ipod_db->tracks; it != NULL; it = it->next) {
 		add_ipod_song_to_db (source, db, (Itdb_Track *)it->data);
@@ -515,7 +543,10 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 
 	load_ipod_playlists (source);
 
-	g_object_unref (G_OBJECT (db));
+	g_object_unref (db);
+
+	GDK_THREADS_LEAVE ();
+	priv->load_idle_id = 0;
 	return FALSE;
 }
 
@@ -525,20 +556,27 @@ rb_ipod_load_songs (RBiPodSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GnomeVFSVolume *volume;
 
-	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
+	g_object_get (source, "volume", &volume, NULL);
 	priv->ipod_mount_path = rb_ipod_get_mount_path (volume);
 
  	priv->ipod_db = itdb_parse (priv->ipod_mount_path, NULL);
 	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
+		Itdb_Playlist *mpl;
+
 		/* FIXME: we could set a different icon depending on the iPod
 		 * model
 		 */
-		g_object_set (RB_SOURCE (source), 
-			      "name", itdb_playlist_mpl (priv->ipod_db)->name, 
-			      NULL);
-		g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
+		mpl = itdb_playlist_mpl (priv->ipod_db);
+		if (mpl && mpl->name) {
+			g_object_set (RB_SOURCE (source),
+				      "name", mpl->name,
+				      NULL);
+		}
+		priv->load_idle_id = g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
 	}
+
+        g_object_unref (volume);
 }
 
 static gchar *
@@ -571,15 +609,15 @@ rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
 	if (mount_point == NULL) {
 		return NULL;
 	}
-	
-#ifdef IPOD_PHONE_SUPPORT
+
+#ifdef IPOD_SUPPORT
 	result = itdb_get_itunesdb_path (mount_point);
 #else
-	result = g_build_filename (mount_point, 
+	result = g_build_filename (mount_point,
 				   "iPod_Control/iTunes/iTunesDB",
 				   NULL);
 #endif
-	
+
 	g_free (mount_point);
 	return result;
 }
@@ -624,7 +662,7 @@ rb_ipod_is_volume_ipod (GnomeVFSVolume *volume)
 		}
 	}
 #endif
-	
+
 	return rb_ipod_volume_has_ipod_db (volume);
 }
 
@@ -635,17 +673,22 @@ hal_udi_is_ipod (const char *udi)
 {
 	LibHalContext *ctx;
 	DBusConnection *conn;
-	char *parent_udi, *parent_name;
+	char *parent_udi;
+        char *parent_name;
 	gboolean result;
 	DBusError error;
+	gboolean inited = FALSE;
 
 	result = FALSE;
 	dbus_error_init (&error);
-	
+
 	conn = NULL;
+        parent_udi = NULL;
+        parent_name = NULL;
+
 	ctx = libhal_ctx_new ();
 	if (ctx == NULL) {
-		/* FIXME: should we return an error somehow so that we can 
+		/* FIXME: should we return an error somehow so that we can
 		 * fall back to a check for iTunesDB presence instead ?
 		 */
 		rb_debug ("cannot connect to HAL");
@@ -659,14 +702,15 @@ hal_udi_is_ipod (const char *udi)
 	if (!libhal_ctx_init (ctx, &error) || dbus_error_is_set (&error))
 		goto end;
 
+	inited = TRUE;
 	parent_udi = libhal_device_get_property_string (ctx, udi,
 			"info.parent", &error);
 	if (parent_udi == NULL || dbus_error_is_set (&error))
 		goto end;
-		
+
 	parent_name = libhal_device_get_property_string (ctx, parent_udi,
 			"storage.model", &error);
-#ifdef IPOD_PHONE_SUPPORT
+#ifdef IPOD_SUPPORT
 	{
 		char *spider_udi;
 		int vnd_id = 0;
@@ -701,22 +745,23 @@ hal_udi_is_ipod (const char *udi)
 				product_id = 0;
 			}
 		}
-		g_free(spider_udi);
+		g_free (spider_udi);
 
 		if (vnd_id == PHONE_VENDOR_ID && product_id == PHONE_PRODUCT_ID) {
 			result = TRUE;
 		}
 	}
 #endif
-	g_free (parent_udi);
 	if (parent_name == NULL || dbus_error_is_set (&error))
 		goto end;
 
 	if (strcmp (parent_name, "iPod") == 0)
 		result = TRUE;
 
-	g_free (parent_name);
 end:
+	g_free (parent_udi);
+	g_free (parent_name);
+
 	if (dbus_error_is_set (&error)) {
 		rb_debug ("Error: %s\n", error.message);
 		dbus_error_free (&error);
@@ -724,7 +769,8 @@ end:
 	}
 
 	if (ctx) {
-		libhal_ctx_shutdown (ctx, &error);
+		if (inited)
+			libhal_ctx_shutdown (ctx, &error);
 		libhal_ctx_free(ctx);
 	}
 
@@ -739,13 +785,14 @@ static gboolean
 hal_udi_is_ipod (const char *udi)
 {
 	LibHalContext *ctx;
-	char *parent_udi, *parent_name;
+	char *parent_udi;
+        char *parent_name;
 	gboolean result;
 
 	result = FALSE;
 	ctx = hal_initialize (NULL, FALSE);
 	if (ctx == NULL) {
-		/* FIXME: should we return an error somehow so that we can 
+		/* FIXME: should we return an error somehow so that we can
 		 * fall back to a check for iTunesDB presence instead ?
 		 */
 		return FALSE;
@@ -772,13 +819,12 @@ static GList*
 impl_get_ui_actions (RBSource *source)
 {
 	GList *actions = NULL;
- 
+
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceEject"));
- 
+
 	return actions;
 }
 
-
 static gboolean
 impl_show_popup (RBSource *source)
 {
@@ -786,7 +832,6 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-
 static void
 remove_track_from_db (Itdb_Track *track)
 {
@@ -803,14 +848,11 @@ impl_move_to_trash (RBSource *asource)
 {
 	GList *sel, *tem;
 	RBEntryView *songs;
-	RBShell *shell;
 	RhythmDB *db;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (asource);
 	RBiPodSource *source = RB_IPOD_SOURCE (asource);
 
-  	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-  	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-  	g_object_unref (G_OBJECT (shell));
+        db = get_db_for_source (source);
 
 	songs = rb_source_get_entry_view (RB_SOURCE (asource));
 	sel = rb_entry_view_get_selected_entries (songs);
@@ -820,7 +862,7 @@ impl_move_to_trash (RBSource *asource)
 		Itdb_Track *track;
 
 		entry = (RhythmDBEntry *)tem->data;
-		uri = rhythmdb_entry_get_string (entry, 
+		uri = rhythmdb_entry_get_string (entry,
 						 RHYTHMDB_PROP_LOCATION);
 		track = g_hash_table_lookup (priv->entry_map, entry);
 		if (track == NULL) {
@@ -833,10 +875,13 @@ impl_move_to_trash (RBSource *asource)
 		rhythmdb_entry_move_to_trash (db, entry);
 		rhythmdb_commit (db);
 	}
+
 	if (sel != NULL) {
 		itdb_write (priv->ipod_db, NULL);
 	}
 
+  	g_object_unref (db);
+
 	g_list_free (sel);
 }
 
@@ -853,7 +898,6 @@ itdb_schedule_save (Itdb_iTunesDB *db)
        itdb_write (db, NULL);
 }
 
-
 #ifdef ENABLE_IPOD_WRITING
 static char *
 build_filename (RBSource *asource, RhythmDBEntry *entry)
@@ -878,13 +922,10 @@ build_filename (RBSource *asource, RhythmDBEntry *entry)
 static void
 completed_cb (RhythmDBEntry *entry, const char *dest, RBiPodSource *source)
 {
-	RBShell *shell;
 	RhythmDB *db;
 	Itdb_Track *song;
 
-  	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-  	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-  	g_object_unref (G_OBJECT (shell));
+        db = get_db_for_source (source);
 
 	song = create_ipod_song_from_entry (entry);
 	if (song != NULL) {
@@ -912,11 +953,11 @@ impl_paste (RBSource *asource, GList *entries)
 	GList *l;
 	RBShell *shell;
 
-	g_object_get (G_OBJECT (asource), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell),
+	g_object_get (asource, "shell", &shell, NULL);
+	g_object_get (shell,
 		      "removable-media-manager", &rm_mgr,
 		      NULL);
-	g_object_unref (G_OBJECT (shell));
+	g_object_unref (shell);
 
 	for (l = entries; l != NULL; l = l->next) {
 		RhythmDBEntry *entry;
@@ -926,28 +967,30 @@ impl_paste (RBSource *asource, GList *entries)
 
 		entry = (RhythmDBEntry *)l->data;
 		entry_type = rhythmdb_entry_get_entry_type (entry);
-		g_object_get (G_OBJECT (asource),
+		g_object_get (asource,
 			      "entry-type", &ipod_entry_type,
 			      NULL);
-		if (entry_type == RHYTHMDB_ENTRY_TYPE_IRADIO_STATION ||
-		    entry_type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED ||
-//		    entry_type == RHYTHMDB_ENTRY_TYPE_PODCAST_EPISODE ||
-		    entry_type == ipod_entry_type)
+		if (entry_type == ipod_entry_type ||
+		    entry_type->category != RHYTHMDB_ENTRY_NORMAL) {
+			g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 			continue;
+		}
 
 		dest = build_filename (asource, entry);
 		if (dest == NULL) {
 			rb_debug ("could not create destination path for entry");
+			g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 			continue;
 		}
 		rb_removable_media_manager_queue_transfer (rm_mgr, entry,
 							   dest, NULL,
 							   (RBTranferCompleteCallback)completed_cb, asource);
 		g_free (dest);
+		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	}
-	g_object_unref (G_OBJECT (rm_mgr));
-}
 
+	g_object_unref (rm_mgr);
+}
 
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
@@ -955,22 +998,21 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	RBBrowserSource *source = RB_BROWSER_SOURCE (asource);
 	GList *list, *i;
 	GList *entries = NULL;
-	RBShell *shell;
 	RhythmDB *db;
+	gboolean is_id;
 
 	rb_debug ("parsing uri list");
 	list = rb_uri_list_parse ((const char *) data->data);
+	is_id = (data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
 
-  	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-  	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-  	g_object_unref (G_OBJECT (shell));
+        db = get_db_for_source (RB_IPOD_SOURCE (source));
 
 	for (i = list; i != NULL; i = g_list_next (i)) {
 		if (i->data != NULL) {
 			char *uri = i->data;
 			RhythmDBEntry *entry;
 
-			entry = rhythmdb_entry_lookup_by_location (db, uri);
+			entry = rhythmdb_entry_lookup_from_string (db, uri, is_id);
 
 			if (entry == NULL) {
 				/* add to the library */
@@ -993,11 +1035,9 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 		g_list_free (entries);
 	}
 
-
 	return TRUE;
 }
 
-
 /* Generation of the filename for the ipod */
 
 #define IPOD_MAX_PATH_LEN 56
@@ -1011,7 +1051,7 @@ test_dir_on_ipod (const char *mountpoint, const char *dirname)
 	fullpath  = g_build_filename (mountpoint, dirname, NULL);
 	result = g_file_test (fullpath, G_FILE_TEST_IS_DIR);
 	g_free (fullpath);
-	
+
 	return result;
 }
 
@@ -1024,7 +1064,7 @@ ipod_mkdir_with_parents (const char *mountpoint, const char *dirname)
 	fullpath  = g_build_filename (mountpoint, dirname, NULL);
 	result = g_mkdir_with_parents (fullpath, 0770);
 	g_free (fullpath);
-	
+
 	return result;
 }
 
@@ -1037,7 +1077,7 @@ build_ipod_dir_name (const char *mountpoint)
 	char *dirname;
 	char *relpath;
 	gint32 suffix;
-	
+
 	suffix = g_random_int_range (0, 100);
 	dirname = g_strdup_printf ("F%02d", suffix);
 	relpath = g_build_filename (G_DIR_SEPARATOR_S, "iPod_Control",
@@ -1120,47 +1160,6 @@ utf8_to_ascii (const gchar *utf8)
 	return g_string_free (string, FALSE);
 }
 
-/* Copied from eel-vfs-extensions.c from eel CVS HEAD on 2004-05-09
- * This function is (C) 1999, 2000 Eazel, Inc.
- */
-static char *
-eel_make_valid_utf8 (const char *name)
-{
-	GString *string;
-	const char *remainder, *invalid;
-	int remaining_bytes, valid_bytes;
-
-	string = NULL;
-	remainder = name;
-	remaining_bytes = strlen (name);
-
-	while (remaining_bytes != 0) {
-		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
-			break;
-		}
-		valid_bytes = invalid - remainder;
-
-		if (string == NULL) {
-			string = g_string_sized_new (remaining_bytes);
-		}
-		g_string_append_len (string, remainder, valid_bytes);
-		g_string_append_c (string, '_');
-
-		remaining_bytes -= valid_bytes + 1;
-		remainder = invalid + 1;
-	}
-
-	if (string == NULL) {
-		return g_strdup (name);
-	}
-
-	g_string_append (string, remainder);
-	g_assert (g_utf8_validate (string->str, -1, NULL));
-
-	return g_string_free (string, FALSE);
-}
-
-
 static gchar *
 generate_ipod_filename (const gchar *mount_point, const gchar *filename)
 {
@@ -1170,7 +1169,7 @@ generate_ipod_filename (const gchar *mount_point, const gchar *filename)
 	gint tries = 0;
 
 	/* First, we need a valid UTF-8 filename, strip all non-UTF-8 chars */
-	tmp = eel_make_valid_utf8 (filename);
+	tmp = rb_make_valid_utf8 (filename, '_');
 	/* The iPod doesn't seem to recognize non-ascii chars in filenames,
 	 * so we strip them
 	 */
@@ -1188,7 +1187,7 @@ generate_ipod_filename (const gchar *mount_point, const gchar *filename)
 		if (tries > MAX_TRIES) {
 			break;
 		}
-	} while ((ipod_filename == NULL) 
+	} while ((ipod_filename == NULL)
 		 || (g_file_test (ipod_filename, G_FILE_TEST_EXISTS)));
 
 	g_free (pc_filename);
@@ -1204,17 +1203,11 @@ generate_ipod_filename (const gchar *mount_point, const gchar *filename)
 static gchar *
 ipod_get_filename_for_uri (const gchar *mount_point, const gchar *uri_str)
 {
-	GnomeVFSURI *uri;
 	gchar *escaped;
 	gchar *filename;
 	gchar *result;
 
-	uri = gnome_vfs_uri_new (uri_str);
-	if (uri == NULL) {
-		return NULL;
-	}
-	escaped = gnome_vfs_uri_extract_short_path_name (uri);
-	gnome_vfs_uri_unref (uri);
+	escaped = rb_uri_get_short_path_name (uri_str);
 	if (escaped == NULL) {
 		return NULL;
 	}
@@ -1254,7 +1247,6 @@ ipod_path_from_unix_path (const gchar *mount_point, const gchar *unix_path)
         /* Make sure the filename doesn't contain any ':' */
         g_strdelimit (ipod_path, ":", ';');
 
-
         /* Convert path to a Mac path where the dir separator is ':' */
 	itdb_filename_fs2ipod (ipod_path);
 
@@ -1266,10 +1258,17 @@ static void
 impl_delete_thyself (RBSource *source)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	GList *p;
+
+	for (p = priv->playlists; p != NULL; p = p->next) {
+		RBSource *playlist = RB_SOURCE (p->data);
+		rb_source_delete_thyself (playlist);
+	}
+	g_list_free (priv->playlists);
+	priv->playlists = NULL;
 
 	itdb_free (priv->ipod_db);
 	priv->ipod_db = NULL;
 
 	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
 }
-
