@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Implementation of Internet Radio source object
  *
  *  Copyright (C) 2002,2003 Colin Walters <walters@debian.org>
@@ -16,11 +16,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 
@@ -46,6 +46,9 @@
 #include "rb-new-station-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
+#include "rb-shell-player.h"
+#include "rb-player.h"
+#include "rb-metadata.h"
 
 static void rb_iradio_source_class_init (RBIRadioSourceClass *klass);
 static void rb_iradio_source_init (RBIRadioSource *source);
@@ -85,15 +88,16 @@ static void rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view
 static char *guess_uri_scheme (const char *uri);
 
 /* source methods */
-static char *impl_get_status (RBSource *source);
+static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
 static char *impl_get_browser_key (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
 static void impl_delete (RBSource *source);
 static void impl_song_properties (RBSource *source);
-static RBSourceEOFType impl_handle_eos (RBSource *asource);
 static gboolean impl_show_popup (RBSource *source);
 static GList *impl_get_ui_actions (RBSource *source);
+static guint impl_want_uri (RBSource *source, const char *uri);
+static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
 
 static void rb_iradio_source_do_query (RBIRadioSource *source);
 
@@ -108,6 +112,10 @@ static void stations_view_drag_data_received_cb (GtkWidget *widget,
 static void rb_iradio_source_cmd_new_station (GtkAction *action,
 					      RBIRadioSource *source);
 
+static void playing_source_changed_cb (RBShellPlayer *player,
+				       RBSource *source,
+				       RBIRadioSource *iradio_source);
+
 #define CMD_PATH_SHOW_BROWSER "/commands/ShowBrowser"
 #define CMD_PATH_CURRENT_STATION "/commands/CurrentStation"
 #define CMD_PATH_SONG_INFO    "/commands/SongInfo"
@@ -136,9 +144,15 @@ struct RBIRadioSourcePrivate
 	char *search_text;
 	char *selected_genre;
 
+	guint prefs_notify_id;
+	guint first_time_notify_id;
 	gboolean firstrun_done;
-	
-	RhythmDBEntryType entry_type;
+
+	RBShellPlayer *player;
+
+	gint info_available_id;
+
+	gboolean dispose_has_run;
 };
 
 #define RB_IRADIO_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_IRADIO_SOURCE, RBIRadioSourcePrivate))
@@ -155,13 +169,7 @@ static const GtkTargetEntry stations_view_drag_types[] = {
 	{  "_NETSCAPE_URL", 0, 1 },
 };
 
-enum
-{
-	PROP_0,
-	PROP_ENTRY_TYPE
-};
-
-G_DEFINE_TYPE (RBIRadioSource, rb_iradio_source, RB_TYPE_SOURCE)
+G_DEFINE_TYPE (RBIRadioSource, rb_iradio_source, RB_TYPE_STREAMING_SOURCE)
 
 static void
 rb_iradio_source_class_init (RBIRadioSourceClass *klass)
@@ -186,22 +194,11 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_get_status  = impl_get_status;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
-	source_class->impl_handle_eos = impl_handle_eos;
-	source_class->impl_have_url = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_song_properties = impl_song_properties;
-	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_true_function;
-
-	g_object_class_install_property (object_class,
-					 PROP_ENTRY_TYPE,
-					 g_param_spec_uint ("entry-type",
-							    "Entry type",
-							    "Type of the entries which should be displayed by this source",
-							    0,
-							    G_MAXINT,
-							    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
-							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	source_class->impl_want_uri = impl_want_uri;
+	source_class->impl_add_uri = impl_add_uri;
 
 	g_type_class_add_private (klass, sizeof (RBIRadioSourcePrivate));
 }
@@ -217,7 +214,7 @@ rb_iradio_source_init (RBIRadioSource *source)
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
-	
+
 	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					   "stock_channel",
@@ -236,6 +233,18 @@ rb_iradio_source_dispose (GObject *object)
 
 	source = RB_IRADIO_SOURCE (object);
 
+	if (source->priv->dispose_has_run) {
+		/* If dispose did already run, return. */
+		return;
+	}
+	/* Make sure dispose does not run twice. */
+	source->priv->dispose_has_run = TRUE;
+
+	if (source->priv->player) {
+		g_object_unref (source->priv->player);
+		source->priv->player = NULL;
+	}
+
 	if (source->priv->db) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
@@ -258,17 +267,24 @@ rb_iradio_source_finalize (GObject *object)
 
 	rb_debug ("finalizing iradio source");
 
+	if (source->priv->action_group != NULL) {
+		g_object_unref (source->priv->action_group);
+	}
+
+	eel_gconf_notification_remove (source->priv->prefs_notify_id);
+	eel_gconf_notification_remove (source->priv->first_time_notify_id);
+
 	G_OBJECT_CLASS (rb_iradio_source_parent_class)->finalize (object);
 }
 
 static GObject *
-rb_iradio_source_constructor (GType type, guint n_construct_properties,
+rb_iradio_source_constructor (GType type,
+			      guint n_construct_properties,
 			      GObjectConstructParam *construct_properties)
 {
 	RBIRadioSource *source;
 	RBIRadioSourceClass *klass;
 	RBShell *shell;
-	GObject *shell_player;
 
 	klass = RB_IRADIO_SOURCE_CLASS (g_type_class_peek (RB_TYPE_IRADIO_SOURCE));
 
@@ -278,9 +294,11 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	source->priv->paned = gtk_hpaned_new ();
 
 	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
-	shell_player = rb_shell_get_player (shell);
-	g_object_unref (G_OBJECT (shell));
+	g_object_get (G_OBJECT (shell),
+		      "db", &source->priv->db,
+		      "shell-player", &source->priv->player,
+		      NULL);
+	g_object_unref (shell);
 
 	source->priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
 								       "IRadioActions",
@@ -289,25 +307,26 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 								       source);
 
 	/* set up stations view */
-	source->priv->stations = rb_entry_view_new (source->priv->db, shell_player,
+	source->priv->stations = rb_entry_view_new (source->priv->db, G_OBJECT (source->priv->player),
 						    CONF_STATE_IRADIO_SORTING,
 						    FALSE, FALSE);
+
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE, TRUE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE, FALSE);
 /* 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY, FALSE); */
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING, FALSE);
 /*	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT, FALSE);*/
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
-	g_signal_connect_object (G_OBJECT (source->priv->stations),
+	g_signal_connect_object (source->priv->stations,
 				 "sort-order-changed",
 				 G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
 				 source, 0);
-	
+
 	/* set up drag and drop for the song tree view.
 	 * we don't use RBEntryView's DnD support because it does too much.
 	 * we just want to be able to drop stations in to add them.
 	 */
-	g_signal_connect_object (G_OBJECT (source->priv->stations), 
+	g_signal_connect_object (source->priv->stations,
 				 "drag_data_received",
 				 G_CALLBACK (stations_view_drag_data_received_cb),
 				 source, 0);
@@ -316,46 +335,55 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 			   stations_view_drag_types, 2,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
-	g_signal_connect_object (G_OBJECT (source->priv->stations),
+	g_signal_connect_object (source->priv->stations,
 				 "size_allocate",
 				 G_CALLBACK (paned_size_allocate_cb),
 				 source, 0);
-	g_signal_connect_object (G_OBJECT (source->priv->stations), "show_popup",
+	g_signal_connect_object (source->priv->stations, "show_popup",
 				 G_CALLBACK (rb_iradio_source_songs_show_popup_cb), source, 0);
-	
+
 	/* set up genre entry view */
 	source->priv->genres = rb_property_view_new (source->priv->db,
 						     RHYTHMDB_PROP_GENRE,
 						     _("Genre"));
-	g_signal_connect_object (G_OBJECT (source->priv->genres),
+	g_signal_connect_object (source->priv->genres,
 				 "property-selected",
 				 G_CALLBACK (genre_selected_cb),
 				 source, 0);
-	g_signal_connect_object (G_OBJECT (source->priv->genres),
+	g_signal_connect_object (source->priv->genres,
 				 "property-selection-reset",
 				 G_CALLBACK (genre_selection_reset_cb),
 				 source, 0);
 
-	g_object_set (G_OBJECT (source->priv->genres), "vscrollbar_policy",
+	g_object_set (source->priv->genres, "vscrollbar_policy",
 		      GTK_POLICY_AUTOMATIC, NULL);
-	g_object_ref (G_OBJECT (source->priv->genres));
-
 
+	gtk_paned_pack1 (GTK_PANED (source->priv->paned),
+			 GTK_WIDGET (source->priv->genres), FALSE, FALSE);
 	gtk_paned_pack2 (GTK_PANED (source->priv->paned),
 			 GTK_WIDGET (source->priv->stations), TRUE, FALSE);
 
 	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), source->priv->paned);
 
-	rb_iradio_source_state_prefs_sync (source);
-	eel_gconf_notification_add (CONF_STATE_IRADIO_DIR,
-				    (GConfClientNotifyFunc) rb_iradio_source_state_pref_changed,
-				    source);
+	source->priv->prefs_notify_id =
+		eel_gconf_notification_add (CONF_STATE_IRADIO_DIR,
+					    (GConfClientNotifyFunc) rb_iradio_source_state_pref_changed,
+					    source);
 	source->priv->firstrun_done = eel_gconf_get_boolean (CONF_FIRST_TIME);
-	eel_gconf_notification_add (CONF_FIRST_TIME,
-				    (GConfClientNotifyFunc) rb_iradio_source_first_time_changed,
-				    source);
+
+	source->priv->first_time_notify_id =
+		eel_gconf_notification_add (CONF_FIRST_TIME,
+					    (GConfClientNotifyFunc) rb_iradio_source_first_time_changed,
+					    source);
+
 	gtk_widget_show_all (GTK_WIDGET (source));
 
+	rb_iradio_source_state_prefs_sync (source);
+
+	g_signal_connect_object (source->priv->player, "playing-source-changed",
+				 G_CALLBACK (playing_source_changed_cb),
+				 source, 0);
+
 	rb_iradio_source_do_query (source);
 
 	return G_OBJECT (source);
@@ -363,16 +391,13 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 
 static void
 rb_iradio_source_set_property (GObject *object,
-			      guint prop_id,
-			      const GValue *value,
-			      GParamSpec *pspec)
+			       guint prop_id,
+			       const GValue *value,
+			       GParamSpec *pspec)
 {
-	RBIRadioSource *source = RB_IRADIO_SOURCE (object);
+	/*RBIRadioSource *source = RB_IRADIO_SOURCE (object);*/
 
 	switch (prop_id) {
-	case PROP_ENTRY_TYPE:
-		source->priv->entry_type = g_value_get_uint (value);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -381,16 +406,13 @@ rb_iradio_source_set_property (GObject *object,
 
 static void
 rb_iradio_source_get_property (GObject *object,
-			      guint prop_id,
-			      GValue *value,
-			      GParamSpec *pspec)
+			       guint prop_id,
+			       GValue *value,
+			       GParamSpec *pspec)
 {
-	RBIRadioSource *source = RB_IRADIO_SOURCE (object);
+	/*RBIRadioSource *source = RB_IRADIO_SOURCE (object);*/
 
 	switch (prop_id) {
-	case PROP_ENTRY_TYPE:
-		g_value_set_uint (value, source->priv->entry_type);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -401,15 +423,27 @@ RBSource *
 rb_iradio_source_new (RBShell *shell)
 {
 	RBSource *source;
+	RhythmDBEntryType entry_type;
+	RhythmDB *db;
+
+	g_object_get (shell, "db", &db, NULL);
+
+	entry_type = rhythmdb_entry_type_get_by_name (db, "iradio");
+	if (entry_type == RHYTHMDB_ENTRY_TYPE_INVALID) {
+		entry_type = rhythmdb_entry_register_type (db, "iradio");
+		entry_type->save_to_disk = TRUE;
+		entry_type->category = RHYTHMDB_ENTRY_STREAM;
+		entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc) rb_true_function;
+		entry_type->sync_metadata = (RhythmDBEntrySyncFunc) rb_null_function;
+	}
+	g_object_unref (db);
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_IRADIO_SOURCE,
 					  "name", _("Radio"),
 					  "shell", shell,
+					  "entry-type", entry_type,
 					  NULL));
-
-	rb_shell_register_entry_type_for_source (shell, source, 
-						 RHYTHMDB_ENTRY_TYPE_IRADIO_STATION);
-
+	rb_shell_register_entry_type_for_source (shell, source, entry_type);
 	return source;
 }
 
@@ -417,7 +451,7 @@ static char *
 guess_uri_scheme (const char *uri)
 {
 	const char *scheme;
-	
+
 	/* if the URI has no scheme, it might be an absolute path, or it might be
 	 * host:port for HTTP.
 	 */
@@ -433,14 +467,16 @@ guess_uri_scheme (const char *uri)
 	return NULL;
 }
 
-
 void
 rb_iradio_source_add_station (RBIRadioSource *source,
-			      const char *uri, const char *title, const char *genre)
+			      const char *uri,
+			      const char *title,
+			      const char *genre)
 {
 	RhythmDBEntry *entry;
 	GValue val = { 0, };
 	char *real_uri = NULL;
+	RhythmDBEntryType entry_type;
 
 	real_uri = guess_uri_scheme (uri);
 	if (real_uri)
@@ -448,11 +484,14 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
 	if (entry) {
-		rb_debug ("uri %s already in db", uri); 
+		rb_debug ("uri %s already in db", uri);
 		g_free (real_uri);
 		return;
 	}
-	entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
+
+	g_object_get (source, "entry-type", &entry_type, NULL);
+	entry = rhythmdb_entry_new (source->priv->db, entry_type, uri);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	if (entry == NULL) {
 		g_free (real_uri);
 		return;
@@ -465,27 +504,28 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 		g_value_take_string (&val,
 				     gnome_vfs_format_uri_for_display (uri));
 
-	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
+	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
 	g_value_reset (&val);
-		
+
 	if ((!genre) || (strcmp (genre, "") == 0))
 		genre = _("Unknown");
 	g_value_set_string (&val, genre);
-	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
+	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	g_value_unset (&val);
-	
+
 	g_value_init (&val, G_TYPE_DOUBLE);
 	g_value_set_double (&val, 0.0);
-	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
+	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
 	g_value_unset (&val);
-	
+
 	rhythmdb_commit (source->priv->db);
 
 	g_free (real_uri);
 }
 
 static void
-impl_search (RBSource *asource, const char *search_text)
+impl_search (RBSource *asource,
+	     const char *search_text)
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
 
@@ -517,59 +557,24 @@ impl_get_entry_view (RBSource *asource)
 	return source->priv->stations;
 }
 
-static RBSourceEOFType
-impl_handle_eos (RBSource *asource)
+static void
+impl_get_status (RBSource *asource,
+		 char **text,
+		 char **progress_text,
+		 float *progress)
 {
-	return RB_SOURCE_EOF_RETRY;
-}
-
-#if 0
-struct RBIRadioAsyncPlayStatisticsData
-{
-	RBIRadioSource *source;
-	RhythmDBEntry *entry;
-};
-
-static gboolean
-rb_iradio_source_async_update_play_statistics (gpointer data)
-{
-	RBIRadioSource *source = RB_IRADIO_SOURCE (data);
-	RhythmDBEntry *playing_entry;
-
-	rb_debug ("entering async handler");
-
-	GDK_THREADS_ENTER ();
-
-	playing_entry = rb_entry_view_get_playing_entry (source->priv->stations);
-	rb_debug ("async updating play statistics, entry: %p playing entry: %p",
-		  source->priv->async_update_entry, playing_entry);
-	if (source->priv->async_update_entry == playing_entry)
-		rb_source_update_play_statistics (RB_SOURCE (source),
-						  source->priv->db,
-						  source->priv->async_update_entry);
-	rhythmdb_entry_unref (source->priv->db, source->priv->async_update_entry);
-	source->priv->async_update_entry = NULL;
-		
-	GDK_THREADS_LEAVE ();
-
-	return FALSE;
-}
-#endif
-
-static char *
-impl_get_status (RBSource *asource)
-{
-	char *ret;
 	RhythmDBQueryModel *model;
 	guint num_entries;
+	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
 
-	g_object_get (G_OBJECT (asource), "query-model", &model, NULL);
+	g_object_get (asource, "query-model", &model, NULL);
 	num_entries = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
-	g_object_unref (G_OBJECT (model));
+	g_object_unref (model);
 
-	ret = g_strdup_printf (ngettext ("%d station", "%d stations", num_entries),
-			       num_entries);
-	return ret;
+	*text = g_strdup_printf (ngettext ("%d station", "%d stations", num_entries),
+				 num_entries);
+
+	rb_streaming_source_get_progress (RB_STREAMING_SOURCE (source), progress_text, progress);
 }
 
 static char *
@@ -582,13 +587,17 @@ static void
 impl_delete (RBSource *asource)
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
+	GList *sel;
 	GList *l;
 
-	for (l = rb_entry_view_get_selected_entries (source->priv->stations); l != NULL;
-	     l = g_list_next (l)) {
+	sel = rb_entry_view_get_selected_entries (source->priv->stations);
+	for (l = sel; l != NULL; l = g_list_next (l)) {
 		rhythmdb_entry_delete (source->priv->db, l->data);
 		rhythmdb_commit (source->priv->db);
 	}
+
+	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
+	g_list_free (sel);
 }
 
 static void
@@ -596,6 +605,7 @@ impl_song_properties (RBSource *asource)
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
 	GtkWidget *dialog = rb_station_properties_dialog_new (source->priv->stations);
+
 	rb_debug ("in song properties");
 	if (dialog)
 		gtk_widget_show_all (dialog);
@@ -603,6 +613,36 @@ impl_song_properties (RBSource *asource)
 		rb_debug ("no selection!");
 }
 
+static guint
+impl_want_uri (RBSource *source, const char *uri)
+{
+	if (g_str_has_prefix (uri, "http://")) {
+		/* other entry types might have
+		 * more specific guesses for HTTP
+		 */
+		return 50;
+	} else if (g_str_has_prefix (uri, "pnm://") ||
+		   g_str_has_prefix (uri, "rtsp://") ||
+		   g_str_has_prefix (uri, "mms://") ||
+		   g_str_has_prefix (uri, "mmsh://")) {
+		return 100;
+	}
+
+	return 0;
+}
+
+static gboolean
+impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+{
+	if (rb_uri_is_local (uri)) {
+		rb_iradio_source_add_from_playlist (RB_IRADIO_SOURCE (source), uri);
+	} else {
+		rb_iradio_source_add_station (RB_IRADIO_SOURCE (source),
+					      uri, title, genre);
+	}
+	return TRUE;
+}
+
 static void
 paned_size_allocate_cb (GtkWidget *widget,
 			GtkAllocation *allocation,
@@ -648,9 +688,10 @@ rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
 				      gboolean over_entry,
 				      RBIRadioSource *source)
 {
-	if (G_OBJECT (source) == NULL) {
+	if (source == NULL) {
 		return;
 	}
+
 	if (over_entry)
 		_rb_source_show_popup (RB_SOURCE (source), "/IRadioViewPopup");
 	else
@@ -672,11 +713,12 @@ genre_selected_cb (RBPropertyView *propview, const char *name,
 }
 
 static void
-genre_selection_reset_cb (RBPropertyView *propview, RBIRadioSource *iradio_source)
+genre_selection_reset_cb (RBPropertyView *propview,
+			  RBIRadioSource *iradio_source)
 {
 	if (iradio_source->priv->setting_new_query)
 		return;
-	
+
 	g_free (iradio_source->priv->selected_genre);
 	iradio_source->priv->selected_genre = NULL;
 
@@ -692,25 +734,20 @@ rb_iradio_source_show_browser (RBIRadioSource *source,
 	GtkWidget *genreswidget = GTK_WIDGET (source->priv->genres);
 
 	if (show == TRUE) {
-		gtk_paned_pack1 (GTK_PANED (source->priv->paned), genreswidget, FALSE, FALSE);
-		gtk_widget_show_all (genreswidget);
-	} else if (show == FALSE) {
-		GList *children = gtk_container_get_children (GTK_CONTAINER (source->priv->paned));
+		gtk_widget_show (genreswidget);
+	} else {
 		gtk_widget_hide (genreswidget);
-		if (g_list_find (children, genreswidget))
-		    gtk_container_remove (GTK_CONTAINER (source->priv->paned), genreswidget);
-		g_list_free (children);
 	}
 }
 
 static void
 rb_iradio_source_do_query (RBIRadioSource *source)
 {
-	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *genre_query_model = NULL;
 	RhythmDBQueryModel *station_query_model = NULL;
 	RhythmDBPropertyModel *genre_model;
 	GPtrArray *query;
+	RhythmDBEntryType entry_type;
 
 	/* don't update the selection while we're rebuilding the query */
 	source->priv->setting_new_query = TRUE;
@@ -718,46 +755,53 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 	/* construct and run the query for the search box.
 	 * this is used as the model for the genre view.
 	 */
-	
-	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+
+	g_object_get (source, "entry-type", &entry_type, NULL);
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
 				      entry_type,
 				      RHYTHMDB_QUERY_END);
-	genre_query_model = rhythmdb_query_model_new_empty (source->priv->db);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	if (source->priv->search_text) {
-		GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_GENRE_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_DISJUNCTION,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_TITLE_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_END);
+	if (source->priv->search_text != NULL) {
+		GPtrArray *subquery;
+
+		subquery = rhythmdb_query_parse (source->priv->db,
+						 RHYTHMDB_QUERY_PROP_LIKE,
+						 RHYTHMDB_PROP_GENRE_FOLDED,
+						 source->priv->search_text,
+						 RHYTHMDB_QUERY_DISJUNCTION,
+						 RHYTHMDB_QUERY_PROP_LIKE,
+						 RHYTHMDB_PROP_TITLE_FOLDED,
+						 source->priv->search_text,
+						 RHYTHMDB_QUERY_END);
 		rb_debug ("searching for \"%s\"", source->priv->search_text);
 		rhythmdb_query_append (source->priv->db,
 				       query,
 				       RHYTHMDB_QUERY_SUBQUERY,
 				       subquery,
 				       RHYTHMDB_QUERY_END);
+
+		rhythmdb_query_free (subquery);
 	}
 
 	genre_model = rb_property_view_get_model (source->priv->genres);
-	g_object_set (G_OBJECT (genre_model), "query-model", genre_query_model, NULL);
 
-	rhythmdb_do_full_query_parsed (source->priv->db, 
-				       RHYTHMDB_QUERY_RESULTS (genre_query_model), 
+	genre_query_model = rhythmdb_query_model_new_empty (source->priv->db);
+	g_object_set (genre_model, "query-model", genre_query_model, NULL);
+
+	rhythmdb_do_full_query_parsed (source->priv->db,
+				       RHYTHMDB_QUERY_RESULTS (genre_query_model),
 				       query);
 
 	rhythmdb_query_free (query);
 	query = NULL;
-	
+
 	/* check the selected genre is still available, and if not, select 'all' */
 	if (source->priv->selected_genre != NULL) {
 		GList *sel = NULL;
+
 		if (!rhythmdb_property_model_iter_from_string (genre_model,
 							       source->priv->selected_genre,
 							       NULL)) {
@@ -768,16 +812,16 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 		sel = g_list_prepend (sel, source->priv->selected_genre);
 		rb_property_view_set_selection (source->priv->genres, sel);
 		g_list_free (sel);
-	} 
-	g_object_unref (G_OBJECT (genre_model));
+	}
 
 	/* if a genre is selected, construct a new query for it, and create
 	 * a new model based on the search box query model.  otherwise, just
 	 * reuse the search box query model.
 	 */
-	
+
 	if (source->priv->selected_genre != NULL) {
 		rb_debug ("matching on genre \"%s\"", source->priv->selected_genre);
+
 		station_query_model = rhythmdb_query_model_new_empty (source->priv->db);
 		query = rhythmdb_query_parse (source->priv->db,
 					      RHYTHMDB_QUERY_PROP_EQUALS,
@@ -785,34 +829,36 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 					      source->priv->selected_genre,
 					      RHYTHMDB_QUERY_END);
 
-		g_object_set (station_query_model, 
+		g_object_set (station_query_model,
 			      "query", query,
 			      "base-model", genre_query_model,
 			      NULL);
+
 		rhythmdb_query_free (query);
 		query = NULL;
 	} else {
-		station_query_model = genre_query_model;
+		station_query_model = g_object_ref (genre_query_model);
 	}
 
 	rb_entry_view_set_model (source->priv->stations, station_query_model);
-	g_object_set (G_OBJECT (source), "query-model", station_query_model, NULL);
+	g_object_set (source, "query-model", station_query_model, NULL);
 
-	g_object_unref (G_OBJECT (genre_query_model));
-	if (station_query_model != genre_query_model)
-		g_object_unref (G_OBJECT (station_query_model));
+	g_object_unref (genre_query_model);
+	g_object_unref (station_query_model);
 
 	source->priv->setting_new_query = FALSE;
 }
 
 static void
-handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri, const char *title,
-			  const char *genre, RBIRadioSource *source)
+handle_playlist_entry_cb (TotemPlParser *playlist,
+			  const char *uri,
+			  const char *title,
+			  const char *genre,
+			  RBIRadioSource *source)
 {
 	rb_iradio_source_add_station (source, uri, title, genre);
 }
 
-
 void
 rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 				    const char     *uri)
@@ -824,11 +870,11 @@ rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 	if (real_uri)
 		uri = real_uri;
 
-	g_signal_connect_object (G_OBJECT (parser), "entry",
+	g_signal_connect_object (parser, "entry",
 				 G_CALLBACK (handle_playlist_entry_cb),
 				 source, 0);
 	if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
-		g_object_set (G_OBJECT (parser), "recurse", FALSE, NULL);
+		g_object_set (parser, "recurse", FALSE, NULL);
 
 	switch (totem_pl_parser_parse (parser, uri, TRUE)) {
 	case TOTEM_PL_PARSER_RESULT_UNHANDLED:
@@ -841,7 +887,7 @@ rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 	case TOTEM_PL_PARSER_RESULT_ERROR:
 		break;
 	}
-	g_object_unref (G_OBJECT (parser));
+	g_object_unref (parser);
 	g_free (real_uri);
 }
 
@@ -866,9 +912,11 @@ rb_iradio_source_first_time_changed (GConfClient *client,
 static void
 stations_view_drag_data_received_cb (GtkWidget *widget,
 				     GdkDragContext *dc,
-				     gint x, gint y,
-				     GtkSelectionData *selection_data, 
-				     guint info, guint time,
+				     gint x,
+				     gint y,
+				     GtkSelectionData *selection_data,
+				     guint info,
+				     guint time,
 				     RBIRadioSource *source)
 {
 	GList *list, *uri_list, *i;
@@ -910,7 +958,7 @@ stations_view_drag_data_received_cb (GtkWidget *widget,
 		}
 
 		g_free (uri);
-		
+
 		if (i != NULL)
 			i = i->next;
 	}
@@ -962,3 +1010,166 @@ rb_iradio_source_cmd_new_station (GtkAction *action,
 	gtk_widget_destroy (dialog);
 }
 
+static gboolean
+check_entry_type (RBIRadioSource *source, RhythmDBEntry *entry)
+{
+	RhythmDBEntryType entry_type;
+	gboolean matches = FALSE;
+
+	g_object_get (source, "entry-type", &entry_type, NULL);
+	if (entry != NULL && rhythmdb_entry_get_entry_type (entry) == entry_type)
+		matches = TRUE;
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+
+	return matches;
+}
+
+static void
+info_available_cb (RBPlayer *backend,
+		   RBMetaDataField field,
+		   GValue *value,
+		   RBIRadioSource *source)
+{
+	RhythmDBEntry *entry;
+        RhythmDBPropType entry_field = 0;
+        gboolean set_field = FALSE;
+	char *str = NULL;
+
+	/* sanity check */
+	if (!rb_player_opened (backend)) {
+		rb_debug ("Got info_available but not playing");
+		return;
+	}
+
+	GDK_THREADS_ENTER ();
+
+	entry = rb_shell_player_get_playing_entry (source->priv->player);
+	if (check_entry_type (source, entry) == FALSE)
+		goto out_unlock;
+
+	/* validate the value */
+	switch (field) {
+	case RB_METADATA_FIELD_TITLE:
+	case RB_METADATA_FIELD_ARTIST:
+	case RB_METADATA_FIELD_GENRE:
+	case RB_METADATA_FIELD_COMMENT:
+		str = g_value_dup_string (value);
+		if (!g_utf8_validate (str, -1, NULL)) {
+			g_warning ("Invalid UTF-8 from internet radio: %s", str);
+			g_free (str);
+			goto out_unlock;
+		}
+		break;
+	default:
+		break;
+	}
+
+
+	switch (field) {
+		/* streaming song information */
+	case RB_METADATA_FIELD_TITLE:
+	{
+		rb_streaming_source_set_streaming_title (RB_STREAMING_SOURCE (source), str);
+		break;
+	}
+	case RB_METADATA_FIELD_ARTIST:
+	{
+		rb_streaming_source_set_streaming_artist (RB_STREAMING_SOURCE (source), str);
+		break;
+	}
+
+		/* station information */
+	case RB_METADATA_FIELD_GENRE:
+	{
+		const char *existing;
+
+		/* check if the db entry already has a genre; if so, don't change it */
+		existing = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
+		if ((existing == NULL) ||
+		    (strcmp (existing, "") == 0) ||
+		    (strcmp (existing, _("Unknown")) == 0)) {
+			entry_field = RHYTHMDB_PROP_GENRE;
+			rb_debug ("setting genre of iradio station to %s", str);
+			set_field = TRUE;
+		} else {
+			rb_debug ("iradio station already has genre: %s; ignoring %s", existing, str);
+		}
+		break;
+	}
+	case RB_METADATA_FIELD_COMMENT:
+	{
+		const char *existing;
+		const char *location;
+
+		/* check if the db entry already has a title; if so, don't change it.
+		 * consider title==URI to be the same as no title, since that's what
+		 * happens for stations imported by DnD or commandline args.
+		 * if the station title really is the same as the URI, then surely
+		 * the station title in the stream metadata will say that too..
+		 */
+		existing = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
+		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+		if ((existing == NULL) ||
+		    (strcmp (existing, "") == 0) ||
+		    (strcmp (existing, location) == 0)) {
+			entry_field = RHYTHMDB_PROP_TITLE;
+			rb_debug ("setting title of iradio station to %s", str);
+			set_field = TRUE;
+		} else {
+			rb_debug ("iradio station already has title: %s; ignoring %s", existing, str);
+		}
+		break;
+	}
+	case RB_METADATA_FIELD_BITRATE:
+		if (!rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE)) {
+			gulong bitrate;
+
+			/* GStreamer sends us bitrate in bps, but we need it in kbps*/
+			bitrate = g_value_get_ulong (value);
+			g_value_set_ulong (value, bitrate/1000);
+
+			rb_debug ("setting bitrate of iradio station to %lu",
+				  g_value_get_ulong (value));
+			entry_field = RHYTHMDB_PROP_BITRATE;
+			set_field = TRUE;
+		}
+		break;
+	default:
+		break;
+	}
+
+	if (set_field && entry_field != 0) {
+		rhythmdb_entry_set (source->priv->db, entry, entry_field, value);
+		rhythmdb_commit (source->priv->db);
+	}
+
+	g_free (str);
+ out_unlock:
+	GDK_THREADS_LEAVE ();
+}
+
+static void
+playing_source_changed_cb (RBShellPlayer *player,
+			   RBSource *source,
+			   RBIRadioSource *iradio_source)
+{
+	GObject *backend;
+
+	g_object_get (player, "player", &backend, NULL);
+
+	if (source == RB_SOURCE (iradio_source) && (iradio_source->priv->info_available_id == 0)) {
+		rb_debug ("connecting info-available signal handler");
+		iradio_source->priv->info_available_id =
+			g_signal_connect_object (backend, "info",
+						 G_CALLBACK (info_available_cb),
+						 iradio_source, 0);
+	} else if (iradio_source->priv->info_available_id) {
+		rb_debug ("disconnecting info-available signal handler");
+		g_signal_handler_disconnect (backend,
+					     iradio_source->priv->info_available_id);
+		iradio_source->priv->info_available_id = 0;
+	}
+
+	g_object_unref (backend);
+}
+
