@@ -696,16 +696,18 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 	GList *list, *i;
 	GList *entries = NULL;
+	gboolean is_id;
 
 	rb_debug ("parsing uri list");
 	list = rb_uri_list_parse ((const char *) data->data);
+	is_id = (data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
 
 	for (i = list; i != NULL; i = g_list_next (i)) {
 		if (i->data != NULL) {
 			char *uri = i->data;
 			RhythmDBEntry *entry;
 
-			entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
+			entry = rhythmdb_entry_lookup_from_string (source->priv->db, uri, is_id);
 
 			if (entry == NULL) {
 				/* add to the library */
@@ -1089,6 +1091,12 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	}
 
 	uri = gnome_vfs_uri_new ((const char *)list->data);
+	if (uri == NULL) {
+		/* do something.. */
+		g_slist_free (list);
+		return NULL;
+	}
+
 	g_slist_free (list);
 
 	realpath = filepath_parse_pattern (layout_path, entry);
@@ -1105,10 +1113,19 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	}
 
 	if (extension == NULL) {
+		const char *uri;
+		const char *loc;
 		char *tmp;
 
 		/* use the old extension. strip anything after a '?' for http/daap/etc */
-		extension = g_strdup (g_utf8_strrchr (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION), -1, '.') + 1);
+		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+		loc = g_utf8_strrchr (uri, -1, '.');
+		if (loc == NULL)
+			loc = g_utf8_strrchr (uri, -1, '/');
+		if (loc == NULL)
+			loc = uri;
+
+		extension = g_strdup (loc + 1);
 
 		tmp = g_utf8_strchr (extension, -1, '?');
 		if (tmp)
@@ -1192,6 +1209,7 @@ impl_paste (RBSource *asource, GList *entries)
 	for (l = entries; l != NULL; l = g_list_next (l)) {
 		RhythmDBEntry *entry = (RhythmDBEntry *)l->data;
 		RhythmDBEntryType entry_type;
+		RBSource *source_source;
 		char *dest;
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
@@ -1202,7 +1220,8 @@ impl_paste (RBSource *asource, GList *entries)
 			continue;
 
 		/* see if the responsible source lets us copy */
-		if (!rb_source_can_copy (rb_shell_get_source_by_entry_type (shell, entry_type)))
+		source_source = rb_shell_get_source_by_entry_type (shell, entry_type);
+		if ((source_source != NULL) && !rb_source_can_copy (source_source))
 			continue;
 
 		dest = build_filename (source, entry);
@@ -1253,15 +1272,26 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	char *name;
 	GdkPixbuf *icon;
 	RhythmDBEntryType entry_type;
+	char *sort_column;
+	int sort_order;
 
 	g_object_get (library_source,
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
 	uri = gnome_vfs_uri_new (path);
+	if (uri == NULL) {
+		g_object_unref (shell);
+		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+		return;
+	}
+
 	name = gnome_vfs_uri_extract_short_name (uri);
 	gnome_vfs_uri_unref (uri);
 
+	rb_entry_view_get_sorting_order (rb_source_get_entry_view (RB_SOURCE (library_source)),
+					 &sort_column, &sort_order);
+
 	source = rb_auto_playlist_source_new (shell, name, FALSE);
 	query = rhythmdb_query_parse (library_source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, entry_type,
@@ -1269,8 +1299,9 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 				      RHYTHMDB_QUERY_END);
 	rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (source), query,
 					   RHYTHMDB_QUERY_MODEL_LIMIT_NONE, NULL,
-					   NULL, 0);
+					   sort_column, sort_order);
 	rhythmdb_query_free (query);
+	g_free (sort_column);
 
 	g_object_get (library_source, "icon", &icon, NULL);
 	g_object_set (source, "icon", icon, NULL);
