@@ -637,7 +637,7 @@ burn_cd (RBPlaylistSourceRecorder *source,
 
                 finished_msg = _("Finished creating audio CD.");
 
-                rb_shell_hidden_notify (source->priv->shell, 0, finished_msg, source->priv->cd_icon, "");
+                rb_shell_hidden_notify (source->priv->shell, 0, finished_msg, source->priv->cd_icon, "", FALSE);
 
                 /* save the write speed that was used */
                 eel_gconf_set_integer (CONF_STATE_BURN_SPEED, speed);
@@ -1530,6 +1530,10 @@ check_dir_has_space (const char *path,
                 return FALSE;
 
         dir_uri = gnome_vfs_uri_new (path);
+	if (dir_uri == NULL) {
+		g_warning (_("Cannot get free space at %s"), path);
+		return FALSE;
+	}
 
         result = gnome_vfs_get_volume_free_space (dir_uri, &free_bytes);
 
