@@ -18,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -64,14 +64,13 @@ static void rb_library_browser_views_changed (GConfClient *client,
 					      GConfEntry *entry,
 					      RBLibraryBrowser *widget);
 
-
-
 G_DEFINE_TYPE (RBLibraryBrowser, rb_library_browser, GTK_TYPE_HBOX)
 #define RB_LIBRARY_BROWSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_BROWSER, RBLibraryBrowserPrivate))
 
 typedef struct
 {
 	RhythmDB *db;
+	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *input_model;
 	RhythmDBQueryModel *output_model;
 
@@ -82,13 +81,13 @@ typedef struct
 	GHashTable *selections;
 } RBLibraryBrowserPrivate;
 
-
 enum
 {
 	PROP_0,
 	PROP_DB,
 	PROP_INPUT_MODEL,
-	PROP_OUTPUT_MODEL
+	PROP_OUTPUT_MODEL,
+	PROP_ENTRY_TYPE
 };
 
 typedef struct {
@@ -134,7 +133,13 @@ rb_library_browser_class_init (RBLibraryBrowserClass *klass)
 							      "output RhythmDBQueryModel instance",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READABLE));
-
+	g_object_class_install_property (object_class,
+					 PROP_ENTRY_TYPE,
+					 g_param_spec_boxed ("entry-type",
+						 	     "Entry type",
+							     "Type of entry to display in this browser",
+							     RHYTHMDB_TYPE_ENTRY_TYPE,
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (klass, sizeof (RBLibraryBrowserPrivate));
 }
@@ -151,7 +156,8 @@ rb_library_browser_init (RBLibraryBrowser *widget)
 }
 
 static GObject *
-rb_library_browser_constructor (GType type, guint n_construct_properties,
+rb_library_browser_constructor (GType type,
+				guint n_construct_properties,
 				GObjectConstructParam *construct_properties)
 {
 	RBLibraryBrowserClass *klass;
@@ -170,9 +176,9 @@ rb_library_browser_constructor (GType type, guint n_construct_properties,
 
 		view = rb_property_view_new (priv->db,
 					     browser_properties[i].type,
-					     browser_properties[i].name);
+					     _(browser_properties[i].name));
 		g_hash_table_insert (priv->property_views, (gpointer)(browser_properties[i].type), view);
-	
+
 		rb_property_view_set_selection_mode (view, GTK_SELECTION_MULTIPLE);
 		g_signal_connect_object (G_OBJECT (view),
 					 "properties-selected",
@@ -184,7 +190,7 @@ rb_library_browser_constructor (GType type, guint n_construct_properties,
 					 browser, 0);
 		gtk_widget_show_all (GTK_WIDGET (view));
 		gtk_widget_set_no_show_all (GTK_WIDGET (view), TRUE);
-		gtk_box_pack_start_defaults (GTK_BOX (browser), GTK_WIDGET (view));	     
+		gtk_box_pack_start_defaults (GTK_BOX (browser), GTK_WIDGET (view));
 	}
 
 	update_browser_views_visibility (browser);
@@ -201,7 +207,15 @@ rb_library_browser_finalize (GObject *object)
 
 	eel_gconf_notification_remove (priv->browser_view_notify_id);
 
-	g_object_unref (G_OBJECT (priv->db));
+	if (priv->db != NULL) {
+		g_object_unref (priv->db);
+	}
+	if (priv->input_model != NULL) {
+		g_object_unref (priv->input_model);
+	}
+	if (priv->output_model != NULL) {
+		g_object_unref (priv->output_model);
+	}
 
 	g_hash_table_destroy (priv->property_views);
 	g_hash_table_destroy (priv->selections);
@@ -217,16 +231,19 @@ rb_library_browser_set_property (GObject *object,
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
-		if (priv->db) {
-			g_object_unref (G_OBJECT (priv->db));
+		if (priv->db != NULL) {
+			g_object_unref (priv->db);
 		}
 		priv->db = g_value_get_object (value);
 
-		if (priv->db)
+		if (priv->db != NULL) {
 			g_object_ref (priv->db);
+		}
+		break;
+	case PROP_ENTRY_TYPE:
+		priv->entry_type = g_value_get_boxed (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -242,8 +259,7 @@ rb_library_browser_get_property (GObject *object,
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, priv->db);
 		break;
@@ -253,6 +269,9 @@ rb_library_browser_get_property (GObject *object,
 	case PROP_OUTPUT_MODEL:
 		g_value_set_object (value, priv->output_model);
 		break;
+	case PROP_ENTRY_TYPE:
+		g_value_set_boxed (value, priv->entry_type);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -260,22 +279,26 @@ rb_library_browser_get_property (GObject *object,
 }
 
 RBLibraryBrowser *
-rb_library_browser_new (RhythmDB *db)
+rb_library_browser_new (RhythmDB *db,
+			RhythmDBEntryType entry_type)
 {
 	RBLibraryBrowser *widget;
 
 	g_assert (db);
 	widget = RB_LIBRARY_BROWSER (g_object_new (RB_TYPE_LIBRARY_BROWSER,
 						   "db", db,
+						   "entry-type", entry_type,
 						   NULL));
 	return widget;
 }
 
 static void
-update_browser_property_visibilty (RhythmDBPropType prop, RBPropertyView *view, GList *properties)
+update_browser_property_visibilty (RhythmDBPropType prop,
+				   RBPropertyView *view,
+				   GList *properties)
 {
 	gboolean old_vis, new_vis;
-	
+
 	old_vis = GTK_WIDGET_VISIBLE (view);
 	new_vis = (g_list_find (properties, (gpointer)prop) != NULL);
 
@@ -284,7 +307,7 @@ update_browser_property_visibilty (RhythmDBPropType prop, RBPropertyView *view,
 			gtk_widget_show (GTK_WIDGET (view));
 		} else {
 			gtk_widget_hide (GTK_WIDGET (view));
-			rb_property_view_reset (view);
+			rb_property_view_set_selection (view, NULL);
 		}
 	}
 }
@@ -328,7 +351,8 @@ view_property_selected_cb (RBPropertyView *view,
 	rb_library_browser_set_selection (widget, prop, selection);
 }
 static void
-view_selection_reset_cb (RBPropertyView *view, RBLibraryBrowser *widget)
+view_selection_reset_cb (RBPropertyView *view,
+			 RBLibraryBrowser *widget)
 {
 	RhythmDBPropType prop;
 
@@ -337,7 +361,9 @@ view_selection_reset_cb (RBPropertyView *view, RBLibraryBrowser *widget)
 }
 
 static void
-reset_view_cb (RhythmDBPropType prop, RBPropertyView *view, RBLibraryBrowser *widget)
+reset_view_cb (RhythmDBPropType prop,
+	       RBPropertyView *view,
+	       RBLibraryBrowser *widget)
 {
 	rb_property_view_set_selection (view, NULL);
 }
@@ -358,11 +384,13 @@ rb_library_browser_reset (RBLibraryBrowser *widget)
 typedef struct {
 	RBLibraryBrowser *widget;
 	RhythmDB *db;
-	GPtrArray *query;
+	RhythmDBQuery *query;
 } ConstructQueryData;
 
 static void
-construct_query_cb (RhythmDBPropType type, GList *selections, ConstructQueryData *data)
+construct_query_cb (RhythmDBPropType type,
+		    GList *selections,
+		    ConstructQueryData *data)
 {
 	rhythmdb_query_append_prop_multiple (data->db,
 					     data->query,
@@ -370,13 +398,13 @@ construct_query_cb (RhythmDBPropType type, GList *selections, ConstructQueryData
 					     selections);
 }
 
-GPtrArray*
+RhythmDBQuery *
 rb_library_browser_construct_query (RBLibraryBrowser *widget)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
-	GPtrArray *query;
+	RhythmDBQuery *query;
 	ConstructQueryData *data;
-	
+
 	query = g_ptr_array_new ();
 	data = g_new0 (ConstructQueryData, 1);
 	data->widget = widget;
@@ -410,7 +438,9 @@ prop_to_index (RhythmDBPropType type)
 }
 
 static void
-ignore_selection_changes (RBLibraryBrowser *widget, RBPropertyView *view, gboolean block)
+ignore_selection_changes (RBLibraryBrowser *widget,
+			  RBPropertyView *view,
+			  gboolean block)
 {
 	if (block) {
 		g_signal_handlers_block_by_func (view, view_selection_reset_cb, widget);
@@ -460,7 +490,7 @@ restore_selection (RBLibraryBrowser *widget,
 	selections = g_hash_table_lookup (priv->selections, (gpointer)browser_properties[property_index].type);
 
 	if (query_pending) {
-		g_object_ref (G_OBJECT (widget));
+		g_object_ref (widget);
 
 		data = g_new0 (SelectionRestoreData, 1);
 		data->widget = widget;
@@ -468,7 +498,7 @@ restore_selection (RBLibraryBrowser *widget,
 		data->selections = selections;
 		data->model = priv->input_model;
 
-		data->handler_id = 
+		data->handler_id =
 			g_signal_connect_data (priv->input_model,
 					       "complete",
 					       G_CALLBACK (query_complete_cb),
@@ -482,116 +512,99 @@ restore_selection (RBLibraryBrowser *widget,
 }
 
 static void
-rebuild_output_model (RBLibraryBrowser *widget)
-{
-	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
-	RBPropertyView *view;
-	RhythmDBPropertyModel *prop_model;
-	int prop_index = num_browser_properties - 1;
-	GList *selections;
-	RhythmDBQueryModel *base_model;
-	GPtrArray *query;
-
-	if (priv->output_model)
-		g_object_unref (G_OBJECT (priv->output_model));
-
-	/* get the query model for the previous property view */
-	view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[prop_index].type);
-	prop_model = rb_property_view_get_model (view);
-	g_object_get (G_OBJECT (prop_model), "query-model", &base_model, NULL);
-	g_object_unref (G_OBJECT (prop_model));
-
-	selections = g_hash_table_lookup (priv->selections, (gpointer)browser_properties[prop_index].type);
-	if (selections != NULL) {
-		query = g_ptr_array_new ();
-		rhythmdb_query_append_prop_multiple (priv->db,
-						     query,
-						     browser_properties[prop_index].type,
-						     selections);
-
-		priv->output_model = rhythmdb_query_model_new_empty (priv->db);
-		g_object_set (G_OBJECT (priv->output_model), "base-model", base_model, NULL);
-
-
-		g_object_set (G_OBJECT (priv->output_model), "query", query, NULL);
-		rhythmdb_query_model_reapply_query (priv->output_model, TRUE);
-		rhythmdb_query_free (query);
-	} else {
-		priv->output_model = base_model;
-		g_object_ref (G_OBJECT (priv->output_model));
-	}
-
-	g_object_notify (G_OBJECT (widget), "output-model");
-	g_object_unref (G_OBJECT (base_model));
-}
-
-static void
-rebuild_child_model (RBLibraryBrowser *widget, gint property_index, gboolean query_pending)
+rebuild_child_model (RBLibraryBrowser *widget,
+		     gint property_index,
+		     gboolean query_pending)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	RhythmDBPropertyModel *prop_model;
 	RhythmDBQueryModel *base_model, *child_model;
 	RBPropertyView *view;
-	GPtrArray *query;
+	RhythmDBQuery *query;
 	GList *selections;
 
 	g_assert (property_index >= 0);
 	g_assert (property_index < num_browser_properties);
 
-	/* there is no model after the last one to update*/
-	if (property_index == num_browser_properties - 1) {
-		rebuild_output_model (widget);
-		return;
-	}
-
 	/* get the query model for the previous property view */
 	view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[property_index].type);
 	prop_model = rb_property_view_get_model (view);
-	g_object_get (G_OBJECT (prop_model), "query-model", &base_model, NULL);
-	g_object_unref (G_OBJECT (prop_model));
+	g_object_get (prop_model, "query-model", &base_model, NULL);
 
-	/* create a new query model based on it, filtered by the selections of the previous property view */
 	selections = g_hash_table_lookup (priv->selections, (gpointer)browser_properties[property_index].type);
 	if (selections != NULL) {
-		query = g_ptr_array_new ();
+
+		/* create a new query model based on it, filtered by
+		 * the selections of the previous property view.
+		 * we need the entry type query criteria to allow the
+		 * backend to optimise the query.
+		 */
+		query = rhythmdb_query_parse (priv->db,
+				              RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, priv->entry_type,
+					      RHYTHMDB_QUERY_END);
 		rhythmdb_query_append_prop_multiple (priv->db,
 						     query,
 						     browser_properties[property_index].type,
 						     selections);
 
 		child_model = rhythmdb_query_model_new_empty (priv->db);
-		g_object_set (G_OBJECT (child_model), "base-model", base_model, NULL);
-
-		g_object_set (G_OBJECT (child_model), "query", query, NULL);
-		rhythmdb_query_model_reapply_query (child_model, TRUE);
+		if (query_pending) {
+			rb_debug ("rebuilding child model for browser %d; query is pending", property_index);
+			g_object_set (child_model,
+				      "query", query,
+				      "base-model", base_model,
+				      NULL);
+		} else {
+			rb_debug ("rebuilding child model for browser %d; running new query", property_index);
+			rhythmdb_query_model_chain (child_model, base_model, FALSE);
+			rhythmdb_do_full_query_parsed (priv->db,
+						       RHYTHMDB_QUERY_RESULTS (child_model),
+						       query);
+		}
 		rhythmdb_query_free (query);
 	} else {
-		child_model = base_model;
+		rb_debug ("no selection for browser %d - reusing parent model", property_index);
+		child_model = g_object_ref (base_model);
 	}
-	
-	/* apply it as the query model of the current property view */
-	view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[property_index+1].type);
-	ignore_selection_changes (widget, view, TRUE);
 
-	prop_model = rb_property_view_get_model (view);
-	g_object_set (G_OBJECT (prop_model), "query-model", child_model, NULL);
-	g_object_unref (G_OBJECT (prop_model));
+	/* If this is the last property, use the child model as the output model
+	 * for the browser.  Otherwise, use it as the input for the next property
+	 * view.
+	 */
+	if (property_index == num_browser_properties-1) {
+		if (priv->output_model != NULL) {
+			g_object_unref (priv->output_model);
+		}
 
-	g_object_unref (G_OBJECT (base_model));
-	if (child_model != base_model)
-		g_object_unref (G_OBJECT (child_model));
+		priv->output_model = child_model;
 
-	rebuild_child_model (widget, property_index + 1, query_pending);
-	restore_selection (widget, property_index + 1, query_pending);
+		g_object_notify (G_OBJECT (widget), "output-model");
+
+	} else {
+		view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[property_index+1].type);
+		ignore_selection_changes (widget, view, TRUE);
+
+		prop_model = rb_property_view_get_model (view);
+		g_object_set (prop_model, "query-model", child_model, NULL);
+
+		g_object_unref (child_model);
+
+		rebuild_child_model (widget, property_index + 1, query_pending);
+		restore_selection (widget, property_index + 1, query_pending);
+	}
+
+	g_object_unref (base_model);
 }
 
 void
-rb_library_browser_set_selection (RBLibraryBrowser *widget, RhythmDBPropType type, GList *selection)
+rb_library_browser_set_selection (RBLibraryBrowser *widget,
+				  RhythmDBPropType type,
+				  GList *selection)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	GList *old_selection;
 	RBPropertyView *view;
-	
+
 	old_selection = g_hash_table_lookup (priv->selections, (gpointer)type);
 
 	if (rb_string_list_equal (old_selection, selection))
@@ -619,8 +632,19 @@ rb_library_browser_get_property_views (RBLibraryBrowser *widget)
 	return rb_collate_hash_table_values (priv->property_views);
 }
 
+RBPropertyView *
+rb_library_browser_get_property_view (RBLibraryBrowser *widget,
+				      RhythmDBPropType type)
+{
+	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
+	RBPropertyView *view;
+
+	view = g_hash_table_lookup (priv->property_views, GINT_TO_POINTER (type));
+	return view;
+}
+
 void
-rb_library_browser_set_model (RBLibraryBrowser *widget, 
+rb_library_browser_set_model (RBLibraryBrowser *widget,
 			      RhythmDBQueryModel *model,
 			      gboolean query_pending)
 {
@@ -628,20 +652,22 @@ rb_library_browser_set_model (RBLibraryBrowser *widget,
 	RBPropertyView *view;
 	RhythmDBPropertyModel *prop_model;
 
-	if (priv->input_model)
-		g_object_unref (G_OBJECT (priv->input_model));
+	if (priv->input_model != NULL) {
+		g_object_unref (priv->input_model);
+	}
 
 	priv->input_model = model;
-	g_object_ref (G_OBJECT (model));
+
+	if (priv->input_model != NULL) {
+		g_object_ref (priv->input_model);
+	}
 
 	view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[0].type);
 	ignore_selection_changes (widget, view, TRUE);
 
 	prop_model = rb_property_view_get_model (view);
-	g_object_set (G_OBJECT (prop_model), "query-model", priv->input_model, NULL);
-	g_object_unref (G_OBJECT (prop_model));
+	g_object_set (prop_model, "query-model", priv->input_model, NULL);
 
 	rebuild_child_model (widget, 0, query_pending);
 	restore_selection (widget, 0, query_pending);
 }
-
